###################################################################
# - Find a XercesC Library
# This module finds XercesC based on $XERCESCROOT 
# and selects a default configuration to use.
#
# XXX_FOUND Set to false, or undefined, if we haven't found, or don't want to use XXX.
# XXX_INCLUDE_DIRS
# XXX_LIBRARY_DIRS
# XXX_LIBRARIES
# You do not have to provide all of the above variables. You should provide XXX_FOUND under most circumstances. 
# 
# Sample usage:
#
#   FIND_PACKAGE(XXX REQUIRED)
#   IF(XXX_FOUND)
#     INCLUDE_DIRECTORIES(${XXX_INCLUDE_DIRS})
#     LINK_DIRECTORIES($(XXX_LIBRARY_DIRS))
#     TARGET_LINK_LIBRARIES(<YourTarget> ${XXX_LIBRARIES})
#   ENDIF(XXX_FOUND)
#   ...
#
# Clear return values in case the module is loaded more than once.
###################################################################


SET(XercesC_FOUND FALSE)

IF(WIN32)
  FIND_PATH(XercesC_ROOT_DIR include/xercesc/util/PlatformUtils.hpp
    HINTS ENV XERCESCROOT ENV ${XERCESCROOT}
    DOC "Xerces C++ base installation directory from $XERCESCROOT"
    NO_DEFAULT_PATH
    NO_CMAKE_PATH
    NO_SYSTEM_ENVIRONMENT_PATH
    NO_CMAKE_SYSTEM_PATH
  )

  IF(XercesC_ROOT_DIR)
    SET(XercesC_INCLUDE_DIRS ${XercesC_ROOT_DIR}/include)
    SET(XercesC_LIBRARY_DIRS ${XercesC_ROOT_DIR}/lib)

    FIND_FILE(FULLLIBNAME2 xerces-c_2.lib HINTS ${XercesC_LIBRARY_DIRS}) #THIS is tricky!!!
    IF(FULLLIBNAME2)
      SET(XercesC_LIBRARIES 
        optimized xerces-c_2
        debug xerces-c_2D
      )
      SET(XercesC_FOUND TRUE)
    ELSE(FULLLIBNAME2)
      FIND_FILE(FULLLIBNAME3 xerces-c_3.lib HINTS ${XercesC_LIBRARY_DIRS}) #THIS is tricky!!!
      IF(FULLLIBNAME3)
        SET(XercesC_LIBRARIES 
          optimized xerces-c_3
          debug xerces-c_3D
        )
        SET(XercesC_FOUND TRUE)
      ELSE(FULLLIBNAME3)
        MESSAGE(FATAL_ERROR, "Cannot find xerces-c_2.lib or xerces-c_3.lib in $XERCESCROOT/lib")
      ENDIF(FULLLIBNAME3)
    ENDIF(FULLLIBNAME2)
  ELSE(XercesC_ROOT_DIR)
	IF(XercesC_FIND_REQUIRED)
		MESSAGE(FATAL_ERROR "You have to set XERCESCROOT environment variable")
	ELSE(XercesC_FIND_REQUIRED )
		MESSAGE(STATUS "You have to set XERCESCROOT environment variable")
	ENDIF(XercesC_FIND_REQUIRED)		
  ENDIF(XercesC_ROOT_DIR)
  
IF(XercesC_FOUND)
#   MESSAGE(STATUS "Found XercesC at $XERCESCROOT=${XercesC_ROOT_DIR}" )
ELSE(XercesC_FOUND)
  MESSAGE(FATAL_ERROR "Failed to find XercesC via environment variable XERCESCROOT=${XercesC_ROOT_DIR}")
ENDIF(XercesC_FOUND)
  
ENDIF(WIN32)  
    
IF(UNIX)
  FIND_PATH(XercesC_INCLUDE_DIRS util/PlatformUtils.hpp
    HINTS ENV XERCESCROOT ENV ${XERCESCROOT}/include/xercesc
    DOC "Xerces C++ base installation directory from $XERCESCROOT"
    /usr/local/include/xercesc/
    NO_CMAKE_PATH
    NO_SYSTEM_ENVIRONMENT_PATH
    NO_CMAKE_SYSTEM_PATH
  )

  FIND_PATH(XercesC_LIBRARY_DIRS libxerces-c.so
    HINTS ENV XERCESCROOT ENV ${XERCESCROOT}/lib
    DOC "Xerces C++ base installation directory from $XERCESCROOT"
    /usr/local/lib
    NO_CMAKE_PATH
    NO_SYSTEM_ENVIRONMENT_PATH
    NO_CMAKE_SYSTEM_PATH
  )

  FIND_FILE(FULLLIBNAME2 libxerces-c.so HINTS ${XercesC_LIBRARY_DIRS})
  IF(FULLLIBNAME2)
	SET(XercesC_LIBRARIES 
	  optimized xerces-c
	  debug xerces-c
	)
	SET(XercesC_FOUND TRUE)
  ELSE(FULLLIBNAME2)
	FIND_FILE(FULLLIBNAME3 libxerces-c.so HINTS ${XercesC_LIBRARY_DIRS}) #THIS is tricky!!!
	IF(FULLLIBNAME3)
	  SET(XercesC_LIBRARIES 
		optimized xerces-c
		debug xerces-c
	  )
	  SET(XercesC_FOUND TRUE)
	ELSE(FULLLIBNAME3)
	  MESSAGE(FATAL_ERROR, "Cannot find libxerces-c.so in $XERCESC_ROOT/lib")
	ENDIF(FULLLIBNAME3)
  ENDIF(FULLLIBNAME2)

IF(XercesC_FOUND)
#   MESSAGE(STATUS "Found XercesC at $XERCESCROOT=${XercesC_ROOT_DIR}" )
ELSE(XercesC_FOUND)
  MESSAGE(FATAL_ERROR "Failed to find XercesC in /usr/local/lib or via XERCESCROOT=${XercesC_ROOT_DIR}")
ENDIF(XercesC_FOUND)
  
ENDIF(UNIX)
