 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtImplInclude.h



#include "Mp7JrsIntegerMatrixType.h"
#include "Mp7Jrsdim_LocalType.h"
#include "Mp7JrsBoxListType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsBoxListType::IMp7JrsBoxListType()
{

// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtPropInit.h

}

IMp7JrsBoxListType::~IMp7JrsBoxListType()
{
// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtPropCleanup.h

}

Mp7JrsBoxListType::Mp7JrsBoxListType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsBoxListType::~Mp7JrsBoxListType()
{
	Cleanup();
}

void Mp7JrsBoxListType::Init()
{
	// Init base
	m_Base = CreateIntegerMatrixType;
	m_Base->m_myself = m_myself.getPointer();
	




// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtMyPropInit.h

}

void Mp7JrsBoxListType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsBoxListType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use BoxListTypePtr, since we
	// might need GetBase(), which isn't defined in IBoxListType
    const Mp7JrsPtr< Mp7JrsBoxListType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsIntegerMatrixPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsBoxListType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
}

Mp7JrsNodePtr Mp7JrsBoxListType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsBoxListType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsIntegerMatrixType > Mp7JrsBoxListType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsintegerVector > Mp7JrsBoxListType::GetBaseCollection() const
{
	return GetBase()->GetBase();
}

void Mp7JrsBoxListType::Initialize(unsigned int maxElems)
{
	GetBase()->GetBase()->Initialize(maxElems);
}

void Mp7JrsBoxListType::addElement(const int &toAdd, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::addElement().");
	}
	GetBase()->GetBase()->addElement(toAdd);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsBoxListType::setElementAt(const int &toSet, unsigned int setAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::setElementAt().");
	}
	GetBase()->GetBase()->setElementAt(toSet, setAt);
	Mp7JrsClientManager::SendUpdateNotification(
	m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
	client, m_myself.getPointer());
}

void Mp7JrsBoxListType::insertElementAt(const int &toInsert, unsigned int insertAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::insertElementAt().");
	}
	GetBase()->GetBase()->insertElementAt(toInsert, insertAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsBoxListType::removeAllElements(Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::removeAllElements().");
	}
	GetBase()->GetBase()->removeAllElements();
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsBoxListType::removeElementAt(unsigned int removeAt, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::removeElementAt().");
	}
	GetBase()->GetBase()->removeElementAt(removeAt);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsBoxListType::containsElement(const int &toCheck)
{
	return GetBase()->GetBase()->containsElement(toCheck);
}

unsigned int Mp7JrsBoxListType::curCapacity() const
{
	return GetBase()->GetBase()->curCapacity();
}

const int Mp7JrsBoxListType::elementAt(unsigned int getAt) const
{
	return GetBase()->GetBase()->elementAt(getAt);
}

int Mp7JrsBoxListType::elementAt(unsigned int getAt)
{
	return GetBase()->GetBase()->elementAt(getAt);
}


unsigned int Mp7JrsBoxListType::size() const
{
	return GetBase()->GetBase()->size();
}

void Mp7JrsBoxListType::ensureExtraCapacity(unsigned int length)
{
	GetBase()->GetBase()->ensureExtraCapacity(length);
}

int Mp7JrsBoxListType::elementIndexOf(const int &toCheck) const
{
	return GetBase()->GetBase()->elementIndexOf(toCheck);
}

Mp7Jrsdim_LocalPtr Mp7JrsBoxListType::Getdim() const
{
	return GetBase()->Getdim();
}

void Mp7JrsBoxListType::Setdim(const Mp7Jrsdim_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsBoxListType::Setdim().");
	}
	GetBase()->Setdim(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsBoxListType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for BoxListType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "BoxListType");
	}
	
	return result;
}

XMLCh * Mp7JrsBoxListType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsBoxListType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsBoxListType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsBoxListType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsBoxListType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("BoxListType"));
	// Element serialization:

}

bool Mp7JrsBoxListType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsIntegerMatrixType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsBoxListType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsBoxListType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsBoxListType_ExtMethodImpl.h


