 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsAudioDType.h"
#include "Mp7JrsContentCollectionType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsContentCollectionType_LocalType::IMp7JrsContentCollectionType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtPropInit.h

}

IMp7JrsContentCollectionType_LocalType::~IMp7JrsContentCollectionType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtPropCleanup.h

}

Mp7JrsContentCollectionType_LocalType::Mp7JrsContentCollectionType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsContentCollectionType_LocalType::~Mp7JrsContentCollectionType_LocalType()
{
	Cleanup();
}

void Mp7JrsContentCollectionType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_VisualFeature = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_AudioFeature = Mp7JrsPtr< Mp7JrsNode >(); // Class


// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMyPropInit.h

}

void Mp7JrsContentCollectionType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_VisualFeature);
	// Mp7JrsFactory::DeleteObject(m_AudioFeature);
}

void Mp7JrsContentCollectionType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ContentCollectionType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IContentCollectionType_LocalType
    const Mp7JrsPtr< Mp7JrsContentCollectionType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_VisualFeature);
		this->SetVisualFeature(Mp7JrsFactory::CloneObject(tmp->GetVisualFeature()));
		// Mp7JrsFactory::DeleteObject(m_AudioFeature);
		this->SetAudioFeature(Mp7JrsFactory::CloneObject(tmp->GetAudioFeature()));
}

Mp7JrsNodePtr Mp7JrsContentCollectionType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsContentCollectionType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsPtr< Mp7JrsNode > Mp7JrsContentCollectionType_LocalType::GetVisualFeature() const
{
	return m_VisualFeature;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsContentCollectionType_LocalType::GetAudioFeature() const
{
	return m_AudioFeature;
}

// implementing setter for choice 
/* element */
void Mp7JrsContentCollectionType_LocalType::SetVisualFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContentCollectionType_LocalType::SetVisualFeature().");
	}
	if (m_VisualFeature != item)
	{
		// Mp7JrsFactory::DeleteObject(m_VisualFeature);
		m_VisualFeature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_VisualFeature) m_VisualFeature->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_VisualFeature) m_VisualFeature->UseTypeAttribute = true;
		if(m_VisualFeature != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_VisualFeature->SetContentName(XMLString::transcode("VisualFeature"));
		}
	// Mp7JrsFactory::DeleteObject(m_AudioFeature);
	m_AudioFeature = Mp7JrsPtr< Mp7JrsNode >();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsContentCollectionType_LocalType::SetAudioFeature(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContentCollectionType_LocalType::SetAudioFeature().");
	}
	if (m_AudioFeature != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AudioFeature);
		m_AudioFeature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AudioFeature) m_AudioFeature->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_AudioFeature) m_AudioFeature->UseTypeAttribute = true;
		if(m_AudioFeature != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AudioFeature->SetContentName(XMLString::transcode("AudioFeature"));
		}
	// Mp7JrsFactory::DeleteObject(m_VisualFeature);
	m_VisualFeature = Mp7JrsPtr< Mp7JrsNode >();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: ContentCollectionType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsContentCollectionType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsContentCollectionType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsContentCollectionType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsContentCollectionType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_VisualFeature != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_VisualFeature->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("VisualFeature"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_VisualFeature->UseTypeAttribute = true;
		}
		m_VisualFeature->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_AudioFeature != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_AudioFeature->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("AudioFeature"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_AudioFeature->UseTypeAttribute = true;
		}
		m_AudioFeature->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsContentCollectionType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("VisualFeature"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("VisualFeature")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateVisualDType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetVisualFeature(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("AudioFeature"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("AudioFeature")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateAudioDType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAudioFeature(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsContentCollectionType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("VisualFeature")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateVisualDType;
    }
    this->SetVisualFeature(child);
  }
  if (XMLString::compareString(elementname, X("AudioFeature")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateAudioDType;
    }
    this->SetAudioFeature(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsContentCollectionType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetVisualFeature() != Mp7JrsNodePtr())
		result.Insert(GetVisualFeature());
	if (GetAudioFeature() != Mp7JrsNodePtr())
		result.Insert(GetAudioFeature());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsContentCollectionType_LocalType_ExtMethodImpl.h


