 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrscurvatureType.h"
#include "Mp7Jrsunsigned7.h"
#include "Mp7JrsContourShapeType_Peak_CollectionType.h"
#include "Mp7JrsContourShapeType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsContourShapeType_Peak_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Peak

#include <assert.h>
IMp7JrsContourShapeType::IMp7JrsContourShapeType()
{

// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtPropInit.h

}

IMp7JrsContourShapeType::~IMp7JrsContourShapeType()
{
// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtPropCleanup.h

}

Mp7JrsContourShapeType::Mp7JrsContourShapeType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsContourShapeType::~Mp7JrsContourShapeType()
{
	Cleanup();
}

void Mp7JrsContourShapeType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_GlobalCurvature = Mp7JrscurvaturePtr(); // Class
	m_PrototypeCurvature = Mp7JrscurvaturePtr(); // Class
	m_PrototypeCurvature_Exist = false;
	m_HighestPeakY = Mp7Jrsunsigned7Ptr(); // Class with content 
	m_Peak = Mp7JrsContourShapeType_Peak_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtMyPropInit.h

}

void Mp7JrsContourShapeType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_GlobalCurvature);
	// Mp7JrsFactory::DeleteObject(m_PrototypeCurvature);
	// Mp7JrsFactory::DeleteObject(m_HighestPeakY);
	// Mp7JrsFactory::DeleteObject(m_Peak);
}

void Mp7JrsContourShapeType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ContourShapeTypePtr, since we
	// might need GetBase(), which isn't defined in IContourShapeType
    const Mp7JrsPtr< Mp7JrsContourShapeType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsContourShapeType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_GlobalCurvature);
		this->SetGlobalCurvature(Mp7JrsFactory::CloneObject(tmp->GetGlobalCurvature()));
	if (tmp->IsValidPrototypeCurvature())
	{
		// Mp7JrsFactory::DeleteObject(m_PrototypeCurvature);
		this->SetPrototypeCurvature(Mp7JrsFactory::CloneObject(tmp->GetPrototypeCurvature()));
	}
	else
	{
		InvalidatePrototypeCurvature();
	}
		// Mp7JrsFactory::DeleteObject(m_HighestPeakY);
		this->SetHighestPeakY(Mp7JrsFactory::CloneObject(tmp->GetHighestPeakY()));
		// Mp7JrsFactory::DeleteObject(m_Peak);
		this->SetPeak(Mp7JrsFactory::CloneObject(tmp->GetPeak()));
}

Mp7JrsNodePtr Mp7JrsContourShapeType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsContourShapeType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsContourShapeType::GetBase() const
{
	return m_Base;
}

Mp7JrscurvaturePtr Mp7JrsContourShapeType::GetGlobalCurvature() const
{
	return m_GlobalCurvature;
}

Mp7JrscurvaturePtr Mp7JrsContourShapeType::GetPrototypeCurvature() const
{
	return m_PrototypeCurvature;
}

// Element is optional
bool Mp7JrsContourShapeType::IsValidPrototypeCurvature() const
{
	return m_PrototypeCurvature_Exist;
}

Mp7Jrsunsigned7Ptr Mp7JrsContourShapeType::GetHighestPeakY() const
{
	return m_HighestPeakY;
}

Mp7JrsContourShapeType_Peak_CollectionPtr Mp7JrsContourShapeType::GetPeak() const
{
	return m_Peak;
}

void Mp7JrsContourShapeType::SetGlobalCurvature(const Mp7JrscurvaturePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContourShapeType::SetGlobalCurvature().");
	}
	if (m_GlobalCurvature != item)
	{
		// Mp7JrsFactory::DeleteObject(m_GlobalCurvature);
		m_GlobalCurvature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GlobalCurvature) m_GlobalCurvature->SetParent(m_myself.getPointer());
		if (m_GlobalCurvature && m_GlobalCurvature->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:curvatureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_GlobalCurvature->UseTypeAttribute = true;
		}
		if(m_GlobalCurvature != Mp7JrscurvaturePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GlobalCurvature->SetContentName(XMLString::transcode("GlobalCurvature"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsContourShapeType::SetPrototypeCurvature(const Mp7JrscurvaturePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContourShapeType::SetPrototypeCurvature().");
	}
	if (m_PrototypeCurvature != item || m_PrototypeCurvature_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_PrototypeCurvature);
		m_PrototypeCurvature = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PrototypeCurvature) m_PrototypeCurvature->SetParent(m_myself.getPointer());
		if (m_PrototypeCurvature && m_PrototypeCurvature->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:curvatureType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PrototypeCurvature->UseTypeAttribute = true;
		}
		m_PrototypeCurvature_Exist = true;
		if(m_PrototypeCurvature != Mp7JrscurvaturePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PrototypeCurvature->SetContentName(XMLString::transcode("PrototypeCurvature"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsContourShapeType::InvalidatePrototypeCurvature()
{
	m_PrototypeCurvature_Exist = false;
}
void Mp7JrsContourShapeType::SetHighestPeakY(const Mp7Jrsunsigned7Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContourShapeType::SetHighestPeakY().");
	}
	if (m_HighestPeakY != item)
	{
		// Mp7JrsFactory::DeleteObject(m_HighestPeakY);
		m_HighestPeakY = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HighestPeakY) m_HighestPeakY->SetParent(m_myself.getPointer());
		if (m_HighestPeakY && m_HighestPeakY->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned7"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HighestPeakY->UseTypeAttribute = true;
		}
		if(m_HighestPeakY != Mp7Jrsunsigned7Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HighestPeakY->SetContentName(XMLString::transcode("HighestPeakY"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsContourShapeType::SetPeak(const Mp7JrsContourShapeType_Peak_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsContourShapeType::SetPeak().");
	}
	if (m_Peak != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Peak);
		m_Peak = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Peak) m_Peak->SetParent(m_myself.getPointer());
		if(m_Peak != Mp7JrsContourShapeType_Peak_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Peak->SetContentName(XMLString::transcode("Peak"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsContourShapeType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("GlobalCurvature")) == 0)
	{
		// GlobalCurvature is simple element curvatureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetGlobalCurvature()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:curvatureType")))) != empty)
			{
				// Is type allowed
				Mp7JrscurvaturePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetGlobalCurvature(p, client);
					if((p = GetGlobalCurvature()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PrototypeCurvature")) == 0)
	{
		// PrototypeCurvature is simple element curvatureType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPrototypeCurvature()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:curvatureType")))) != empty)
			{
				// Is type allowed
				Mp7JrscurvaturePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPrototypeCurvature(p, client);
					if((p = GetPrototypeCurvature()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HighestPeakY")) == 0)
	{
		// HighestPeakY is simple element unsigned7
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHighestPeakY()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned7")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned7Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHighestPeakY(p, client);
					if((p = GetHighestPeakY()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Peak")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Peak is item of type ContourShapeType_Peak_LocalType
		// in element collection ContourShapeType_Peak_CollectionType
		
		context->Found = true;
		Mp7JrsContourShapeType_Peak_CollectionPtr coll = GetPeak();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateContourShapeType_Peak_CollectionType;
				SetPeak(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 62))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ContourShapeType_Peak_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsContourShapeType_Peak_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ContourShapeType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ContourShapeType");
	}		
	
	return result;
}

XMLCh * Mp7JrsContourShapeType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsContourShapeType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsContourShapeType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ContourShapeType"));
	// Element serialization:
	// Class
	
	if (m_GlobalCurvature != Mp7JrscurvaturePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_GlobalCurvature->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("GlobalCurvature"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_GlobalCurvature->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_PrototypeCurvature != Mp7JrscurvaturePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PrototypeCurvature->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PrototypeCurvature"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PrototypeCurvature->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_HighestPeakY != Mp7Jrsunsigned7Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HighestPeakY->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HighestPeakY"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HighestPeakY->Serialize(doc, element, &elem);
	}
	if (m_Peak != Mp7JrsContourShapeType_Peak_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Peak->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsContourShapeType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("GlobalCurvature"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("GlobalCurvature")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatecurvatureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetGlobalCurvature(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PrototypeCurvature"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PrototypeCurvature")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatecurvatureType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPrototypeCurvature(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HighestPeakY"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HighestPeakY")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = Createunsigned7;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHighestPeakY(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Peak"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Peak")) == 0))
        {
            // Deserialize factory type
            Mp7JrsContourShapeType_Peak_CollectionPtr tmp = CreateContourShapeType_Peak_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetPeak(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsContourShapeType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("GlobalCurvature")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatecurvatureType;
    }
    this->SetGlobalCurvature(child);
  }
  if (XMLString::compareString(elementname, X("PrototypeCurvature")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatecurvatureType;
    }
    this->SetPrototypeCurvature(child);
  }
  if (XMLString::compareString(elementname, X("HighestPeakY")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = Createunsigned7;
    }
    this->SetHighestPeakY(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Peak")) == 0))
  {
    Mp7JrsContourShapeType_Peak_CollectionPtr tmp = CreateContourShapeType_Peak_CollectionType;
    this->SetPeak(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsContourShapeType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetGlobalCurvature() != Mp7JrsNodePtr())
		result.Insert(GetGlobalCurvature());
	if (GetPrototypeCurvature() != Mp7JrsNodePtr())
		result.Insert(GetPrototypeCurvature());
	if (GetHighestPeakY() != Mp7JrsNodePtr())
		result.Insert(GetHighestPeakY());
	if (GetPeak() != Mp7JrsNodePtr())
		result.Insert(GetPeak());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsContourShapeType_ExtMethodImpl.h


