 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCreationType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsCreationType_Title_CollectionType.h"
#include "Mp7JrsTitleMediaType.h"
#include "Mp7JrsCreationType_Abstract_CollectionType.h"
#include "Mp7JrsCreationType_Creator_CollectionType.h"
#include "Mp7JrsCreationType_CreationCoordinates_CollectionType.h"
#include "Mp7JrsCreationType_CreationTool_CollectionType.h"
#include "Mp7JrsCreationType_CopyrightString_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsCreationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTitleType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Title
#include "Mp7JrsTextAnnotationType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Abstract
#include "Mp7JrsCreatorType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Creator
#include "Mp7JrsCreationType_CreationCoordinates_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:CreationCoordinates
#include "Mp7JrsCreationToolType.h" // Element collection urn:mpeg:mpeg7:schema:2004:CreationTool
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:CopyrightString

#include <assert.h>
IMp7JrsCreationType::IMp7JrsCreationType()
{

// no includefile for extension defined 
// file Mp7JrsCreationType_ExtPropInit.h

}

IMp7JrsCreationType::~IMp7JrsCreationType()
{
// no includefile for extension defined 
// file Mp7JrsCreationType_ExtPropCleanup.h

}

Mp7JrsCreationType::Mp7JrsCreationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCreationType::~Mp7JrsCreationType()
{
	Cleanup();
}

void Mp7JrsCreationType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Title = Mp7JrsCreationType_Title_CollectionPtr(); // Collection
	m_TitleMedia = Mp7JrsTitleMediaPtr(); // Class
	m_TitleMedia_Exist = false;
	m_Abstract = Mp7JrsCreationType_Abstract_CollectionPtr(); // Collection
	m_Creator = Mp7JrsCreationType_Creator_CollectionPtr(); // Collection
	m_CreationCoordinates = Mp7JrsCreationType_CreationCoordinates_CollectionPtr(); // Collection
	m_CreationTool = Mp7JrsCreationType_CreationTool_CollectionPtr(); // Collection
	m_CopyrightString = Mp7JrsCreationType_CopyrightString_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCreationType_ExtMyPropInit.h

}

void Mp7JrsCreationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCreationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Title);
	// Mp7JrsFactory::DeleteObject(m_TitleMedia);
	// Mp7JrsFactory::DeleteObject(m_Abstract);
	// Mp7JrsFactory::DeleteObject(m_Creator);
	// Mp7JrsFactory::DeleteObject(m_CreationCoordinates);
	// Mp7JrsFactory::DeleteObject(m_CreationTool);
	// Mp7JrsFactory::DeleteObject(m_CopyrightString);
}

void Mp7JrsCreationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CreationTypePtr, since we
	// might need GetBase(), which isn't defined in ICreationType
    const Mp7JrsPtr< Mp7JrsCreationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsCreationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Title);
		this->SetTitle(Mp7JrsFactory::CloneObject(tmp->GetTitle()));
	if (tmp->IsValidTitleMedia())
	{
		// Mp7JrsFactory::DeleteObject(m_TitleMedia);
		this->SetTitleMedia(Mp7JrsFactory::CloneObject(tmp->GetTitleMedia()));
	}
	else
	{
		InvalidateTitleMedia();
	}
		// Mp7JrsFactory::DeleteObject(m_Abstract);
		this->SetAbstract(Mp7JrsFactory::CloneObject(tmp->GetAbstract()));
		// Mp7JrsFactory::DeleteObject(m_Creator);
		this->SetCreator(Mp7JrsFactory::CloneObject(tmp->GetCreator()));
		// Mp7JrsFactory::DeleteObject(m_CreationCoordinates);
		this->SetCreationCoordinates(Mp7JrsFactory::CloneObject(tmp->GetCreationCoordinates()));
		// Mp7JrsFactory::DeleteObject(m_CreationTool);
		this->SetCreationTool(Mp7JrsFactory::CloneObject(tmp->GetCreationTool()));
		// Mp7JrsFactory::DeleteObject(m_CopyrightString);
		this->SetCopyrightString(Mp7JrsFactory::CloneObject(tmp->GetCopyrightString()));
}

Mp7JrsNodePtr Mp7JrsCreationType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsCreationType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsCreationType::GetBase() const
{
	return m_Base;
}

Mp7JrsCreationType_Title_CollectionPtr Mp7JrsCreationType::GetTitle() const
{
	return m_Title;
}

Mp7JrsTitleMediaPtr Mp7JrsCreationType::GetTitleMedia() const
{
	return m_TitleMedia;
}

// Element is optional
bool Mp7JrsCreationType::IsValidTitleMedia() const
{
	return m_TitleMedia_Exist;
}

Mp7JrsCreationType_Abstract_CollectionPtr Mp7JrsCreationType::GetAbstract() const
{
	return m_Abstract;
}

Mp7JrsCreationType_Creator_CollectionPtr Mp7JrsCreationType::GetCreator() const
{
	return m_Creator;
}

Mp7JrsCreationType_CreationCoordinates_CollectionPtr Mp7JrsCreationType::GetCreationCoordinates() const
{
	return m_CreationCoordinates;
}

Mp7JrsCreationType_CreationTool_CollectionPtr Mp7JrsCreationType::GetCreationTool() const
{
	return m_CreationTool;
}

Mp7JrsCreationType_CopyrightString_CollectionPtr Mp7JrsCreationType::GetCopyrightString() const
{
	return m_CopyrightString;
}

void Mp7JrsCreationType::SetTitle(const Mp7JrsCreationType_Title_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetTitle().");
	}
	if (m_Title != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Title);
		m_Title = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Title) m_Title->SetParent(m_myself.getPointer());
		if(m_Title != Mp7JrsCreationType_Title_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Title->SetContentName(XMLString::transcode("Title"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::SetTitleMedia(const Mp7JrsTitleMediaPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetTitleMedia().");
	}
	if (m_TitleMedia != item || m_TitleMedia_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TitleMedia);
		m_TitleMedia = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TitleMedia) m_TitleMedia->SetParent(m_myself.getPointer());
		if (m_TitleMedia && m_TitleMedia->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TitleMediaType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TitleMedia->UseTypeAttribute = true;
		}
		m_TitleMedia_Exist = true;
		if(m_TitleMedia != Mp7JrsTitleMediaPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TitleMedia->SetContentName(XMLString::transcode("TitleMedia"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::InvalidateTitleMedia()
{
	m_TitleMedia_Exist = false;
}
void Mp7JrsCreationType::SetAbstract(const Mp7JrsCreationType_Abstract_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetAbstract().");
	}
	if (m_Abstract != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Abstract);
		m_Abstract = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Abstract) m_Abstract->SetParent(m_myself.getPointer());
		if(m_Abstract != Mp7JrsCreationType_Abstract_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Abstract->SetContentName(XMLString::transcode("Abstract"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::SetCreator(const Mp7JrsCreationType_Creator_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetCreator().");
	}
	if (m_Creator != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Creator);
		m_Creator = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Creator) m_Creator->SetParent(m_myself.getPointer());
		if(m_Creator != Mp7JrsCreationType_Creator_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Creator->SetContentName(XMLString::transcode("Creator"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::SetCreationCoordinates(const Mp7JrsCreationType_CreationCoordinates_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetCreationCoordinates().");
	}
	if (m_CreationCoordinates != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CreationCoordinates);
		m_CreationCoordinates = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CreationCoordinates) m_CreationCoordinates->SetParent(m_myself.getPointer());
		if(m_CreationCoordinates != Mp7JrsCreationType_CreationCoordinates_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CreationCoordinates->SetContentName(XMLString::transcode("CreationCoordinates"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::SetCreationTool(const Mp7JrsCreationType_CreationTool_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetCreationTool().");
	}
	if (m_CreationTool != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CreationTool);
		m_CreationTool = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CreationTool) m_CreationTool->SetParent(m_myself.getPointer());
		if(m_CreationTool != Mp7JrsCreationType_CreationTool_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CreationTool->SetContentName(XMLString::transcode("CreationTool"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::SetCopyrightString(const Mp7JrsCreationType_CopyrightString_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetCopyrightString().");
	}
	if (m_CopyrightString != item)
	{
		// Mp7JrsFactory::DeleteObject(m_CopyrightString);
		m_CopyrightString = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CopyrightString) m_CopyrightString->SetParent(m_myself.getPointer());
		if(m_CopyrightString != Mp7JrsCreationType_CopyrightString_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CopyrightString->SetContentName(XMLString::transcode("CopyrightString"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsCreationType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsCreationType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsCreationType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsCreationType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsCreationType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsCreationType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsCreationType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsCreationType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsCreationType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsCreationType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsCreationType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsCreationType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsCreationType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsCreationType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsCreationType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreationType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsCreationType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsCreationType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreationType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCreationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Title")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Title is item of type TitleType
		// in element collection CreationType_Title_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_Title_CollectionPtr coll = GetTitle();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_Title_CollectionType;
				SetTitle(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TitleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTitlePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TitleMedia")) == 0)
	{
		// TitleMedia is simple element TitleMediaType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTitleMedia()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TitleMediaType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTitleMediaPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTitleMedia(p, client);
					if((p = GetTitleMedia()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Abstract")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Abstract is item of type TextAnnotationType
		// in element collection CreationType_Abstract_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_Abstract_CollectionPtr coll = GetAbstract();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_Abstract_CollectionType;
				SetAbstract(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextAnnotationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextAnnotationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Creator")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Creator is item of type CreatorType
		// in element collection CreationType_Creator_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_Creator_CollectionPtr coll = GetCreator();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_Creator_CollectionType;
				SetCreator(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CreationCoordinates")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:CreationCoordinates is item of type CreationType_CreationCoordinates_LocalType
		// in element collection CreationType_CreationCoordinates_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_CreationCoordinates_CollectionPtr coll = GetCreationCoordinates();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_CreationCoordinates_CollectionType;
				SetCreationCoordinates(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationType_CreationCoordinates_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationType_CreationCoordinates_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CreationTool")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:CreationTool is item of type CreationToolType
		// in element collection CreationType_CreationTool_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_CreationTool_CollectionPtr coll = GetCreationTool();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_CreationTool_CollectionType;
				SetCreationTool(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationToolType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationToolPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("CopyrightString")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:CopyrightString is item of type TextualType
		// in element collection CreationType_CopyrightString_CollectionType
		
		context->Found = true;
		Mp7JrsCreationType_CopyrightString_CollectionPtr coll = GetCopyrightString();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreationType_CopyrightString_CollectionType;
				SetCopyrightString(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CreationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CreationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCreationType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsCreationType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsCreationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsCreationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CreationType"));
	// Element serialization:
	if (m_Title != Mp7JrsCreationType_Title_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Title->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_TitleMedia != Mp7JrsTitleMediaPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TitleMedia->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TitleMedia"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TitleMedia->Serialize(doc, element, &elem);
	}
	if (m_Abstract != Mp7JrsCreationType_Abstract_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Abstract->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Creator != Mp7JrsCreationType_Creator_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Creator->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_CreationCoordinates != Mp7JrsCreationType_CreationCoordinates_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_CreationCoordinates->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_CreationTool != Mp7JrsCreationType_CreationTool_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_CreationTool->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_CopyrightString != Mp7JrsCreationType_CopyrightString_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_CopyrightString->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCreationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Title"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Title")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_Title_CollectionPtr tmp = CreateCreationType_Title_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTitle(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TitleMedia"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TitleMedia")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTitleMediaType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTitleMedia(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Abstract"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Abstract")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_Abstract_CollectionPtr tmp = CreateCreationType_Abstract_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAbstract(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Creator"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Creator")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_Creator_CollectionPtr tmp = CreateCreationType_Creator_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCreator(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("CreationCoordinates"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("CreationCoordinates")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_CreationCoordinates_CollectionPtr tmp = CreateCreationType_CreationCoordinates_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCreationCoordinates(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("CreationTool"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("CreationTool")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_CreationTool_CollectionPtr tmp = CreateCreationType_CreationTool_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCreationTool(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("CopyrightString"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("CopyrightString")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreationType_CopyrightString_CollectionPtr tmp = CreateCreationType_CopyrightString_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCopyrightString(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCreationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCreationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Title")) == 0))
  {
    Mp7JrsCreationType_Title_CollectionPtr tmp = CreateCreationType_Title_CollectionType;
    this->SetTitle(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("TitleMedia")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTitleMediaType;
    }
    this->SetTitleMedia(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Abstract")) == 0))
  {
    Mp7JrsCreationType_Abstract_CollectionPtr tmp = CreateCreationType_Abstract_CollectionType;
    this->SetAbstract(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Creator")) == 0))
  {
    Mp7JrsCreationType_Creator_CollectionPtr tmp = CreateCreationType_Creator_CollectionType;
    this->SetCreator(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("CreationCoordinates")) == 0))
  {
    Mp7JrsCreationType_CreationCoordinates_CollectionPtr tmp = CreateCreationType_CreationCoordinates_CollectionType;
    this->SetCreationCoordinates(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("CreationTool")) == 0))
  {
    Mp7JrsCreationType_CreationTool_CollectionPtr tmp = CreateCreationType_CreationTool_CollectionType;
    this->SetCreationTool(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("CopyrightString")) == 0))
  {
    Mp7JrsCreationType_CopyrightString_CollectionPtr tmp = CreateCreationType_CopyrightString_CollectionType;
    this->SetCopyrightString(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCreationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTitle() != Mp7JrsNodePtr())
		result.Insert(GetTitle());
	if (GetTitleMedia() != Mp7JrsNodePtr())
		result.Insert(GetTitleMedia());
	if (GetAbstract() != Mp7JrsNodePtr())
		result.Insert(GetAbstract());
	if (GetCreator() != Mp7JrsNodePtr())
		result.Insert(GetCreator());
	if (GetCreationCoordinates() != Mp7JrsNodePtr())
		result.Insert(GetCreationCoordinates());
	if (GetCreationTool() != Mp7JrsNodePtr())
		result.Insert(GetCreationTool());
	if (GetCopyrightString() != Mp7JrsNodePtr())
		result.Insert(GetCopyrightString());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCreationType_ExtMethodImpl.h


