 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtImplInclude.h



#include "Mp7JrsMediaAgentType.h"
#include "Mp7JrsCreatorType_Character_CollectionType.h"
#include "Mp7JrsCreatorType_Instrument_CollectionType.h"
#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsAgentType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsCreatorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsPersonNameType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Character
#include "Mp7JrsCreationToolType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Instrument

#include <assert.h>
IMp7JrsCreatorType::IMp7JrsCreatorType()
{

// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtPropInit.h

}

IMp7JrsCreatorType::~IMp7JrsCreatorType()
{
// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtPropCleanup.h

}

Mp7JrsCreatorType::Mp7JrsCreatorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsCreatorType::~Mp7JrsCreatorType()
{
	Cleanup();
}

void Mp7JrsCreatorType::Init()
{
	// Init base
	m_Base = CreateMediaAgentType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Character = Mp7JrsCreatorType_Character_CollectionPtr(); // Collection
	m_Instrument = Mp7JrsCreatorType_Instrument_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtMyPropInit.h

}

void Mp7JrsCreatorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Character);
	// Mp7JrsFactory::DeleteObject(m_Instrument);
}

void Mp7JrsCreatorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use CreatorTypePtr, since we
	// might need GetBase(), which isn't defined in ICreatorType
    const Mp7JrsPtr< Mp7JrsCreatorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsMediaAgentPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsCreatorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Character);
		this->SetCharacter(Mp7JrsFactory::CloneObject(tmp->GetCharacter()));
		// Mp7JrsFactory::DeleteObject(m_Instrument);
		this->SetInstrument(Mp7JrsFactory::CloneObject(tmp->GetInstrument()));
}

Mp7JrsNodePtr Mp7JrsCreatorType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsCreatorType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsMediaAgentType > Mp7JrsCreatorType::GetBase() const
{
	return m_Base;
}

Mp7JrsCreatorType_Character_CollectionPtr Mp7JrsCreatorType::GetCharacter() const
{
	return m_Character;
}

Mp7JrsCreatorType_Instrument_CollectionPtr Mp7JrsCreatorType::GetInstrument() const
{
	return m_Instrument;
}

void Mp7JrsCreatorType::SetCharacter(const Mp7JrsCreatorType_Character_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreatorType::SetCharacter().");
	}
	if (m_Character != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Character);
		m_Character = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Character) m_Character->SetParent(m_myself.getPointer());
		if(m_Character != Mp7JrsCreatorType_Character_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Character->SetContentName(XMLString::transcode("Character"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsCreatorType::SetInstrument(const Mp7JrsCreatorType_Instrument_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreatorType::SetInstrument().");
	}
	if (m_Instrument != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Instrument);
		m_Instrument = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Instrument) m_Instrument->SetParent(m_myself.getPointer());
		if(m_Instrument != Mp7JrsCreatorType_Instrument_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Instrument->SetContentName(XMLString::transcode("Instrument"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

Mp7JrsControlledTermUsePtr Mp7JrsCreatorType::GetRole() const
{
	return GetBase()->GetRole();
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsCreatorType::GetAgent() const
{
	return GetBase()->GetAgent();
}

Mp7JrsReferencePtr Mp7JrsCreatorType::GetAgentRef() const
{
	return GetBase()->GetAgentRef();
}

void Mp7JrsCreatorType::SetRole(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreatorType::SetRole().");
	}
	GetBase()->SetRole(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsCreatorType::SetAgent(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreatorType::SetAgent().");
	}
	GetBase()->SetAgent(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsCreatorType::SetAgentRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsCreatorType::SetAgentRef().");
	}
	GetBase()->SetAgentRef(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsCreatorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Character")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Character is item of type PersonNameType
		// in element collection CreatorType_Character_CollectionType
		
		context->Found = true;
		Mp7JrsCreatorType_Character_CollectionPtr coll = GetCharacter();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreatorType_Character_CollectionType;
				SetCharacter(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PersonNameType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPersonNamePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Instrument")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Instrument is item of type CreationToolType
		// in element collection CreatorType_Instrument_CollectionType
		
		context->Found = true;
		Mp7JrsCreatorType_Instrument_CollectionPtr coll = GetInstrument();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateCreatorType_Instrument_CollectionType;
				SetInstrument(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:CreationToolType")))) != empty)
			{
				// Is type allowed
				Mp7JrsCreationToolPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for CreatorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "CreatorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsCreatorType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsCreatorType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsCreatorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("CreatorType"));
	// Element serialization:
	if (m_Character != Mp7JrsCreatorType_Character_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Character->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Instrument != Mp7JrsCreatorType_Instrument_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Instrument->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsCreatorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsMediaAgentType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Character"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Character")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreatorType_Character_CollectionPtr tmp = CreateCreatorType_Character_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCharacter(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Instrument"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Instrument")) == 0))
        {
            // Deserialize factory type
            Mp7JrsCreatorType_Instrument_CollectionPtr tmp = CreateCreatorType_Instrument_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetInstrument(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsCreatorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Character")) == 0))
  {
    Mp7JrsCreatorType_Character_CollectionPtr tmp = CreateCreatorType_Character_CollectionType;
    this->SetCharacter(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Instrument")) == 0))
  {
    Mp7JrsCreatorType_Instrument_CollectionPtr tmp = CreateCreatorType_Instrument_CollectionType;
    this->SetInstrument(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsCreatorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetCharacter() != Mp7JrsNodePtr())
		result.Insert(GetCharacter());
	if (GetInstrument() != Mp7JrsNodePtr())
		result.Insert(GetInstrument());
	if (GetRole() != Mp7JrsNodePtr())
		result.Insert(GetRole());
	if (GetAgent() != Mp7JrsNodePtr())
		result.Insert(GetAgent());
	if (GetAgentRef() != Mp7JrsNodePtr())
		result.Insert(GetAgentRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsCreatorType_ExtMethodImpl.h


