 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDSType_ExtImplInclude.h



#include "Mp7JrsMpeg7BaseType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsDSType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsDSType::IMp7JrsDSType()
{

// no includefile for extension defined 
// file Mp7JrsDSType_ExtPropInit.h

}

IMp7JrsDSType::~IMp7JrsDSType()
{
// no includefile for extension defined 
// file Mp7JrsDSType_ExtPropCleanup.h

}

Mp7JrsDSType::Mp7JrsDSType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDSType::~Mp7JrsDSType()
{
	Cleanup();
}

void Mp7JrsDSType::Init()
{
	// Init base
	m_Base = CreateMpeg7BaseType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_id = NULL; // String
	m_id_Exist = false;
	m_timeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_timeBase_Exist = false;
	m_timeUnit = Mp7JrsdurationPtr(); // Pattern
	m_timeUnit_Exist = false;
	m_mediaTimeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_mediaTimeBase_Exist = false;
	m_mediaTimeUnit = Mp7JrsmediaDurationPtr(); // Pattern
	m_mediaTimeUnit_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Header = Mp7JrsDSType_Header_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsDSType_ExtMyPropInit.h

}

void Mp7JrsDSType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDSType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_id);
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); 
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
	// Mp7JrsFactory::DeleteObject(m_Header);
}

void Mp7JrsDSType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DSTypePtr, since we
	// might need GetBase(), which isn't defined in IDSType
    const Mp7JrsPtr< Mp7JrsDSType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsMpeg7BasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsDSType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_id);
	if (tmp->Existid())
	{
		this->Setid(XMLString::replicate(tmp->Getid()));
	}
	else
	{
		Invalidateid();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	if (tmp->ExisttimeBase())
	{
		this->SettimeBase(Mp7JrsFactory::CloneObject(tmp->GettimeBase()));
	}
	else
	{
		InvalidatetimeBase();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	if (tmp->ExisttimeUnit())
	{
		this->SettimeUnit(Mp7JrsFactory::CloneObject(tmp->GettimeUnit()));
	}
	else
	{
		InvalidatetimeUnit();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeBase); 
	if (tmp->ExistmediaTimeBase())
	{
		this->SetmediaTimeBase(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeBase()));
	}
	else
	{
		InvalidatemediaTimeBase();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_mediaTimeUnit); 
	if (tmp->ExistmediaTimeUnit())
	{
		this->SetmediaTimeUnit(Mp7JrsFactory::CloneObject(tmp->GetmediaTimeUnit()));
	}
	else
	{
		InvalidatemediaTimeUnit();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Header);
		this->SetHeader(Mp7JrsFactory::CloneObject(tmp->GetHeader()));
}

Mp7JrsNodePtr Mp7JrsDSType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsDSType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsMpeg7BaseType > Mp7JrsDSType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsDSType::Getid() const
{
	return m_id;
}

bool Mp7JrsDSType::Existid() const
{
	return m_id_Exist;
}
void Mp7JrsDSType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::Setid().");
	}
    m_id = item;
	m_id_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDSType::Invalidateid()
{
	m_id_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsDSType::GettimeBase() const
{
	return m_timeBase;
}

bool Mp7JrsDSType::ExisttimeBase() const
{
	return m_timeBase_Exist;
}
void Mp7JrsDSType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::SettimeBase().");
	}
    m_timeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeBase) m_timeBase->SetParent(m_myself.getPointer());
	m_timeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDSType::InvalidatetimeBase()
{
	m_timeBase_Exist = false;
}
Mp7JrsdurationPtr Mp7JrsDSType::GettimeUnit() const
{
	return m_timeUnit;
}

bool Mp7JrsDSType::ExisttimeUnit() const
{
	return m_timeUnit_Exist;
}
void Mp7JrsDSType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::SettimeUnit().");
	}
    m_timeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeUnit) m_timeUnit->SetParent(m_myself.getPointer());
	m_timeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDSType::InvalidatetimeUnit()
{
	m_timeUnit_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsDSType::GetmediaTimeBase() const
{
	return m_mediaTimeBase;
}

bool Mp7JrsDSType::ExistmediaTimeBase() const
{
	return m_mediaTimeBase_Exist;
}
void Mp7JrsDSType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::SetmediaTimeBase().");
	}
    m_mediaTimeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeBase) m_mediaTimeBase->SetParent(m_myself.getPointer());
	m_mediaTimeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDSType::InvalidatemediaTimeBase()
{
	m_mediaTimeBase_Exist = false;
}
Mp7JrsmediaDurationPtr Mp7JrsDSType::GetmediaTimeUnit() const
{
	return m_mediaTimeUnit;
}

bool Mp7JrsDSType::ExistmediaTimeUnit() const
{
	return m_mediaTimeUnit_Exist;
}
void Mp7JrsDSType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::SetmediaTimeUnit().");
	}
    m_mediaTimeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_mediaTimeUnit) m_mediaTimeUnit->SetParent(m_myself.getPointer());
	m_mediaTimeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDSType::InvalidatemediaTimeUnit()
{
	m_mediaTimeUnit_Exist = false;
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsDSType::GetHeader() const
{
	return m_Header;
}

void Mp7JrsDSType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDSType::SetHeader().");
	}
	if (m_Header != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Header);
		m_Header = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Header) m_Header->SetParent(m_myself.getPointer());
		if(m_Header != Mp7JrsDSType_Header_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Header->SetContentName(XMLString::transcode("Header"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsDSType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Header")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Header is item of abstract type HeaderType
		// in element collection DSType_Header_CollectionType
		
		context->Found = true;
		Mp7JrsDSType_Header_CollectionPtr coll = GetHeader();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateDSType_Header_CollectionType;
				SetHeader(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsHeaderPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for DSType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "DSType");
	}		
	
	return result;
}

XMLCh * Mp7JrsDSType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsDSType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsDSType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsDSType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DSType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_id_Exist)
	{
	// String
	if(m_id != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("id"), m_id);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeBase_Exist)
	{
	// Pattern
	if (m_timeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeBase->ToText();
		element->setAttributeNS(X(""), X("timeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeUnit_Exist)
	{
	// Pattern
	if (m_timeUnit != Mp7JrsdurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeUnit->ToText();
		element->setAttributeNS(X(""), X("timeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeBase_Exist)
	{
	// Pattern
	if (m_mediaTimeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_mediaTimeBase->ToText();
		element->setAttributeNS(X(""), X("mediaTimeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_mediaTimeUnit_Exist)
	{
	// Pattern
	if (m_mediaTimeUnit != Mp7JrsmediaDurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_mediaTimeUnit->ToText();
		element->setAttributeNS(X(""), X("mediaTimeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Header != Mp7JrsDSType_Header_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Header->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsDSType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("id"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttribute(X("id"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttribute(X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("mediaTimeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("mediaTimeBase")));
        this->SetmediaTimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("mediaTimeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("mediaTimeBase")));
        this->SetmediaTimeBase(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("mediaTimeUnit")))
	{
        Mp7JrsmediaDurationPtr tmp = CreatemediaDurationType;
        tmp->Parse(parent->getAttribute(X("mediaTimeUnit")));
        this->SetmediaTimeUnit(tmp);
        * current = parent;
    }

	// Extensionbase is Mp7JrsMpeg7BaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Header"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Header")) == 0))
        {
            // Deserialize factory type
            Mp7JrsDSType_Header_CollectionPtr tmp = CreateDSType_Header_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetHeader(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsDSType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDSType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Header")) == 0))
  {
    Mp7JrsDSType_Header_CollectionPtr tmp = CreateDSType_Header_CollectionType;
    this->SetHeader(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDSType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDSType_ExtMethodImpl.h


