 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7JrsColorSpaceType.h"
#include "Mp7JrsColorQuantizationType.h"
#include "Mp7Jrsunsigned5.h"
#include "Mp7JrsDominantColorType_Value_CollectionType.h"
#include "Mp7JrsDominantColorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsDominantColorType_Value_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Value

#include <assert.h>
IMp7JrsDominantColorType::IMp7JrsDominantColorType()
{

// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtPropInit.h

}

IMp7JrsDominantColorType::~IMp7JrsDominantColorType()
{
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtPropCleanup.h

}

Mp7JrsDominantColorType::Mp7JrsDominantColorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsDominantColorType::~Mp7JrsDominantColorType()
{
	Cleanup();
}

void Mp7JrsDominantColorType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_ColorSpace = Mp7JrsColorSpacePtr(); // Class
	m_ColorSpace_Exist = false;
	m_ColorQuantization = Mp7JrsColorQuantizationPtr(); // Class
	m_ColorQuantization_Exist = false;
	m_SpatialCoherency = Mp7Jrsunsigned5Ptr(); // Class with content 
	m_Value = Mp7JrsDominantColorType_Value_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMyPropInit.h

}

void Mp7JrsDominantColorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ColorSpace);
	// Mp7JrsFactory::DeleteObject(m_ColorQuantization);
	// Mp7JrsFactory::DeleteObject(m_SpatialCoherency);
	// Mp7JrsFactory::DeleteObject(m_Value);
}

void Mp7JrsDominantColorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use DominantColorTypePtr, since we
	// might need GetBase(), which isn't defined in IDominantColorType
    const Mp7JrsPtr< Mp7JrsDominantColorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsDominantColorType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidColorSpace())
	{
		// Mp7JrsFactory::DeleteObject(m_ColorSpace);
		this->SetColorSpace(Mp7JrsFactory::CloneObject(tmp->GetColorSpace()));
	}
	else
	{
		InvalidateColorSpace();
	}
	if (tmp->IsValidColorQuantization())
	{
		// Mp7JrsFactory::DeleteObject(m_ColorQuantization);
		this->SetColorQuantization(Mp7JrsFactory::CloneObject(tmp->GetColorQuantization()));
	}
	else
	{
		InvalidateColorQuantization();
	}
		// Mp7JrsFactory::DeleteObject(m_SpatialCoherency);
		this->SetSpatialCoherency(Mp7JrsFactory::CloneObject(tmp->GetSpatialCoherency()));
		// Mp7JrsFactory::DeleteObject(m_Value);
		this->SetValue(Mp7JrsFactory::CloneObject(tmp->GetValue()));
}

Mp7JrsNodePtr Mp7JrsDominantColorType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsDominantColorType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsDominantColorType::GetBase() const
{
	return m_Base;
}

Mp7JrsColorSpacePtr Mp7JrsDominantColorType::GetColorSpace() const
{
	return m_ColorSpace;
}

// Element is optional
bool Mp7JrsDominantColorType::IsValidColorSpace() const
{
	return m_ColorSpace_Exist;
}

Mp7JrsColorQuantizationPtr Mp7JrsDominantColorType::GetColorQuantization() const
{
	return m_ColorQuantization;
}

// Element is optional
bool Mp7JrsDominantColorType::IsValidColorQuantization() const
{
	return m_ColorQuantization_Exist;
}

Mp7Jrsunsigned5Ptr Mp7JrsDominantColorType::GetSpatialCoherency() const
{
	return m_SpatialCoherency;
}

Mp7JrsDominantColorType_Value_CollectionPtr Mp7JrsDominantColorType::GetValue() const
{
	return m_Value;
}

void Mp7JrsDominantColorType::SetColorSpace(const Mp7JrsColorSpacePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType::SetColorSpace().");
	}
	if (m_ColorSpace != item || m_ColorSpace_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorSpace);
		m_ColorSpace = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorSpace) m_ColorSpace->SetParent(m_myself.getPointer());
		if (m_ColorSpace && m_ColorSpace->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorSpaceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorSpace->UseTypeAttribute = true;
		}
		m_ColorSpace_Exist = true;
		if(m_ColorSpace != Mp7JrsColorSpacePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorSpace->SetContentName(XMLString::transcode("ColorSpace"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDominantColorType::InvalidateColorSpace()
{
	m_ColorSpace_Exist = false;
}
void Mp7JrsDominantColorType::SetColorQuantization(const Mp7JrsColorQuantizationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType::SetColorQuantization().");
	}
	if (m_ColorQuantization != item || m_ColorQuantization_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_ColorQuantization);
		m_ColorQuantization = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ColorQuantization) m_ColorQuantization->SetParent(m_myself.getPointer());
		if (m_ColorQuantization && m_ColorQuantization->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorQuantizationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_ColorQuantization->UseTypeAttribute = true;
		}
		m_ColorQuantization_Exist = true;
		if(m_ColorQuantization != Mp7JrsColorQuantizationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ColorQuantization->SetContentName(XMLString::transcode("ColorQuantization"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDominantColorType::InvalidateColorQuantization()
{
	m_ColorQuantization_Exist = false;
}
void Mp7JrsDominantColorType::SetSpatialCoherency(const Mp7Jrsunsigned5Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType::SetSpatialCoherency().");
	}
	if (m_SpatialCoherency != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpatialCoherency);
		m_SpatialCoherency = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpatialCoherency) m_SpatialCoherency->SetParent(m_myself.getPointer());
		if (m_SpatialCoherency && m_SpatialCoherency->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned5"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpatialCoherency->UseTypeAttribute = true;
		}
		if(m_SpatialCoherency != Mp7Jrsunsigned5Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpatialCoherency->SetContentName(XMLString::transcode("SpatialCoherency"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsDominantColorType::SetValue(const Mp7JrsDominantColorType_Value_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsDominantColorType::SetValue().");
	}
	if (m_Value != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Value);
		m_Value = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Value) m_Value->SetParent(m_myself.getPointer());
		if(m_Value != Mp7JrsDominantColorType_Value_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Value->SetContentName(XMLString::transcode("Value"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsDominantColorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ColorSpace")) == 0)
	{
		// ColorSpace is simple element ColorSpaceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorSpace()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorSpaceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorSpacePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorSpace(p, client);
					if((p = GetColorSpace()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("ColorQuantization")) == 0)
	{
		// ColorQuantization is simple element ColorQuantizationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetColorQuantization()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ColorQuantizationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsColorQuantizationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetColorQuantization(p, client);
					if((p = GetColorQuantization()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpatialCoherency")) == 0)
	{
		// SpatialCoherency is simple element unsigned5
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpatialCoherency()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned5")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned5Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpatialCoherency(p, client);
					if((p = GetSpatialCoherency()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Value")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Value is item of type DominantColorType_Value_LocalType
		// in element collection DominantColorType_Value_CollectionType
		
		context->Found = true;
		Mp7JrsDominantColorType_Value_CollectionPtr coll = GetValue();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateDominantColorType_Value_CollectionType;
				SetValue(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 8))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DominantColorType_Value_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDominantColorType_Value_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for DominantColorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "DominantColorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsDominantColorType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsDominantColorType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsDominantColorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("DominantColorType"));
	// Element serialization:
	// Class
	
	if (m_ColorSpace != Mp7JrsColorSpacePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorSpace->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorSpace"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorSpace->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_ColorQuantization != Mp7JrsColorQuantizationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_ColorQuantization->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("ColorQuantization"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_ColorQuantization->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_SpatialCoherency != Mp7Jrsunsigned5Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpatialCoherency->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpatialCoherency"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpatialCoherency->Serialize(doc, element, &elem);
	}
	if (m_Value != Mp7JrsDominantColorType_Value_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Value->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsDominantColorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorSpace"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorSpace")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorSpaceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorSpace(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("ColorQuantization"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("ColorQuantization")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateColorQuantizationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetColorQuantization(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpatialCoherency"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpatialCoherency")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = Createunsigned5;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSpatialCoherency(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Value"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Value")) == 0))
        {
            // Deserialize factory type
            Mp7JrsDominantColorType_Value_CollectionPtr tmp = CreateDominantColorType_Value_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetValue(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsDominantColorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("ColorSpace")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorSpaceType;
    }
    this->SetColorSpace(child);
  }
  if (XMLString::compareString(elementname, X("ColorQuantization")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateColorQuantizationType;
    }
    this->SetColorQuantization(child);
  }
  if (XMLString::compareString(elementname, X("SpatialCoherency")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = Createunsigned5;
    }
    this->SetSpatialCoherency(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Value")) == 0))
  {
    Mp7JrsDominantColorType_Value_CollectionPtr tmp = CreateDominantColorType_Value_CollectionType;
    this->SetValue(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsDominantColorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetColorSpace() != Mp7JrsNodePtr())
		result.Insert(GetColorSpace());
	if (GetColorQuantization() != Mp7JrsNodePtr())
		result.Insert(GetColorQuantization());
	if (GetSpatialCoherency() != Mp7JrsNodePtr())
		result.Insert(GetSpatialCoherency());
	if (GetValue() != Mp7JrsNodePtr())
		result.Insert(GetValue());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsDominantColorType_ExtMethodImpl.h


