 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtImplInclude.h



#include "Mp7JrsEventType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsEventType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsEventType_LocalType::IMp7JrsEventType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtPropInit.h

}

IMp7JrsEventType_LocalType::~IMp7JrsEventType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtPropCleanup.h

}

Mp7JrsEventType_LocalType::Mp7JrsEventType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsEventType_LocalType::~Mp7JrsEventType_LocalType()
{
	Cleanup();
}

void Mp7JrsEventType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_Event = Mp7JrsEventPtr(); // Class
	m_EventRef = Mp7JrsReferencePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtMyPropInit.h

}

void Mp7JrsEventType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Event);
	// Mp7JrsFactory::DeleteObject(m_EventRef);
}

void Mp7JrsEventType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use EventType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IEventType_LocalType
    const Mp7JrsPtr< Mp7JrsEventType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_Event);
		this->SetEvent(Mp7JrsFactory::CloneObject(tmp->GetEvent()));
		// Mp7JrsFactory::DeleteObject(m_EventRef);
		this->SetEventRef(Mp7JrsFactory::CloneObject(tmp->GetEventRef()));
}

Mp7JrsNodePtr Mp7JrsEventType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsEventType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsEventPtr Mp7JrsEventType_LocalType::GetEvent() const
{
	return m_Event;
}

Mp7JrsReferencePtr Mp7JrsEventType_LocalType::GetEventRef() const
{
	return m_EventRef;
}

// implementing setter for choice 
/* element */
void Mp7JrsEventType_LocalType::SetEvent(const Mp7JrsEventPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType_LocalType::SetEvent().");
	}
	if (m_Event != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Event);
		m_Event = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Event) m_Event->SetParent(m_myself.getPointer());
		if (m_Event && m_Event->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:EventType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Event->UseTypeAttribute = true;
		}
		if(m_Event != Mp7JrsEventPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Event->SetContentName(XMLString::transcode("Event"));
		}
	// Mp7JrsFactory::DeleteObject(m_EventRef);
	m_EventRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsEventType_LocalType::SetEventRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsEventType_LocalType::SetEventRef().");
	}
	if (m_EventRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_EventRef);
		m_EventRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_EventRef) m_EventRef->SetParent(m_myself.getPointer());
		if (m_EventRef && m_EventRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_EventRef->UseTypeAttribute = true;
		}
		if(m_EventRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_EventRef->SetContentName(XMLString::transcode("EventRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_Event);
	m_Event = Mp7JrsEventPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: EventType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsEventType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsEventType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsEventType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsEventType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_Event != Mp7JrsEventPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Event->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Event"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Event->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_EventRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_EventRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("EventRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_EventRef->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsEventType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Event"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Event")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateEventType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetEvent(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("EventRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("EventRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetEventRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsEventType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Event")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateEventType;
    }
    this->SetEvent(child);
  }
  if (XMLString::compareString(elementname, X("EventRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetEventRef(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsEventType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetEvent() != Mp7JrsNodePtr())
		result.Insert(GetEvent());
	if (GetEventRef() != Mp7JrsNodePtr())
		result.Insert(GetEventRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsEventType_LocalType_ExtMethodImpl.h


