 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtImplInclude.h



#include "Mp7JrsReferenceType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsGraphType_Node_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsGraphType_Node_LocalType::IMp7JrsGraphType_Node_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtPropInit.h

}

IMp7JrsGraphType_Node_LocalType::~IMp7JrsGraphType_Node_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtPropCleanup.h

}

Mp7JrsGraphType_Node_LocalType::Mp7JrsGraphType_Node_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsGraphType_Node_LocalType::~Mp7JrsGraphType_Node_LocalType()
{
	Cleanup();
}

void Mp7JrsGraphType_Node_LocalType::Init()
{
	// Init base
	m_Base = CreateReferenceType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_id = NULL; // String
	m_id_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Name = Mp7JrsTermUsePtr(); // Class
	m_Name_Exist = false;


// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtMyPropInit.h

}

void Mp7JrsGraphType_Node_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_id);
	// Mp7JrsFactory::DeleteObject(m_Name);
}

void Mp7JrsGraphType_Node_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use GraphType_Node_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IGraphType_Node_LocalType
    const Mp7JrsPtr< Mp7JrsGraphType_Node_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsReferencePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsGraphType_Node_LocalType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_id);
	if (tmp->Existid())
	{
		this->Setid(XMLString::replicate(tmp->Getid()));
	}
	else
	{
		Invalidateid();
	}
	}
	if (tmp->IsValidName())
	{
		// Mp7JrsFactory::DeleteObject(m_Name);
		this->SetName(Mp7JrsFactory::CloneObject(tmp->GetName()));
	}
	else
	{
		InvalidateName();
	}
}

Mp7JrsNodePtr Mp7JrsGraphType_Node_LocalType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsGraphType_Node_LocalType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsReferenceType > Mp7JrsGraphType_Node_LocalType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsGraphType_Node_LocalType::Getid() const
{
	return m_id;
}

bool Mp7JrsGraphType_Node_LocalType::Existid() const
{
	return m_id_Exist;
}
void Mp7JrsGraphType_Node_LocalType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_Node_LocalType::Setid().");
	}
    m_id = item;
	m_id_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType_Node_LocalType::Invalidateid()
{
	m_id_Exist = false;
}
Mp7JrsTermUsePtr Mp7JrsGraphType_Node_LocalType::GetName() const
{
	return m_Name;
}

// Element is optional
bool Mp7JrsGraphType_Node_LocalType::IsValidName() const
{
	return m_Name_Exist;
}

void Mp7JrsGraphType_Node_LocalType::SetName(const Mp7JrsTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_Node_LocalType::SetName().");
	}
	if (m_Name != item || m_Name_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Name);
		m_Name = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Name) m_Name->SetParent(m_myself.getPointer());
		if (m_Name && m_Name->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Name->UseTypeAttribute = true;
		}
		m_Name_Exist = true;
		if(m_Name != Mp7JrsTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Name->SetContentName(XMLString::transcode("Name"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType_Node_LocalType::InvalidateName()
{
	m_Name_Exist = false;
}
XMLCh * Mp7JrsGraphType_Node_LocalType::Getidref() const
{
	return GetBase()->Getidref();
}

bool Mp7JrsGraphType_Node_LocalType::Existidref() const
{
	return GetBase()->Existidref();
}
void Mp7JrsGraphType_Node_LocalType::Setidref(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_Node_LocalType::Setidref().");
	}
	GetBase()->Setidref(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType_Node_LocalType::Invalidateidref()
{
	GetBase()->Invalidateidref();
}
Mp7JrsxPathRefPtr Mp7JrsGraphType_Node_LocalType::Getxpath() const
{
	return GetBase()->Getxpath();
}

bool Mp7JrsGraphType_Node_LocalType::Existxpath() const
{
	return GetBase()->Existxpath();
}
void Mp7JrsGraphType_Node_LocalType::Setxpath(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_Node_LocalType::Setxpath().");
	}
	GetBase()->Setxpath(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType_Node_LocalType::Invalidatexpath()
{
	GetBase()->Invalidatexpath();
}
XMLCh * Mp7JrsGraphType_Node_LocalType::Gethref() const
{
	return GetBase()->Gethref();
}

bool Mp7JrsGraphType_Node_LocalType::Existhref() const
{
	return GetBase()->Existhref();
}
void Mp7JrsGraphType_Node_LocalType::Sethref(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsGraphType_Node_LocalType::Sethref().");
	}
	GetBase()->Sethref(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsGraphType_Node_LocalType::Invalidatehref()
{
	GetBase()->Invalidatehref();
}

Mp7JrsNodeEnum Mp7JrsGraphType_Node_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Name")) == 0)
	{
		// Name is simple element TermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetName()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetName(p, client);
					if((p = GetName()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for GraphType_Node_LocalType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "GraphType_Node_LocalType");
	}		
	
	return result;
}

XMLCh * Mp7JrsGraphType_Node_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsGraphType_Node_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsGraphType_Node_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("GraphType_Node_LocalType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_id_Exist)
	{
	// String
	if(m_id != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("id"), m_id);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Name != Mp7JrsTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Name->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Name"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Name->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsGraphType_Node_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("id"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttribute(X("id"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsReferenceType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Name"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Name")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetName(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsGraphType_Node_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Name")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTermUseType;
    }
    this->SetName(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsGraphType_Node_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsGraphType_Node_LocalType_ExtMethodImpl.h


