 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtImplInclude.h



#include "Mp7JrsVisualDType.h"
#include "Mp7Jrsunsigned8.h"
#include "Mp7JrstextureListType.h"
#include "Mp7JrsHomogeneousTextureType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsHomogeneousTextureType::IMp7JrsHomogeneousTextureType()
{

// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtPropInit.h

}

IMp7JrsHomogeneousTextureType::~IMp7JrsHomogeneousTextureType()
{
// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtPropCleanup.h

}

Mp7JrsHomogeneousTextureType::Mp7JrsHomogeneousTextureType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsHomogeneousTextureType::~Mp7JrsHomogeneousTextureType()
{
	Cleanup();
}

void Mp7JrsHomogeneousTextureType::Init()
{
	// Init base
	m_Base = CreateVisualDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Average = Mp7Jrsunsigned8Ptr(); // Class with content 
	m_StandardDeviation = Mp7Jrsunsigned8Ptr(); // Class with content 
	m_Energy = Mp7JrstextureListPtr(); // Class
	m_EnergyDeviation = Mp7JrstextureListPtr(); // Class
	m_EnergyDeviation_Exist = false;


// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtMyPropInit.h

}

void Mp7JrsHomogeneousTextureType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Average);
	// Mp7JrsFactory::DeleteObject(m_StandardDeviation);
	// Mp7JrsFactory::DeleteObject(m_Energy);
	// Mp7JrsFactory::DeleteObject(m_EnergyDeviation);
}

void Mp7JrsHomogeneousTextureType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use HomogeneousTextureTypePtr, since we
	// might need GetBase(), which isn't defined in IHomogeneousTextureType
    const Mp7JrsPtr< Mp7JrsHomogeneousTextureType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsVisualDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsHomogeneousTextureType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Average);
		this->SetAverage(Mp7JrsFactory::CloneObject(tmp->GetAverage()));
		// Mp7JrsFactory::DeleteObject(m_StandardDeviation);
		this->SetStandardDeviation(Mp7JrsFactory::CloneObject(tmp->GetStandardDeviation()));
		// Mp7JrsFactory::DeleteObject(m_Energy);
		this->SetEnergy(Mp7JrsFactory::CloneObject(tmp->GetEnergy()));
	if (tmp->IsValidEnergyDeviation())
	{
		// Mp7JrsFactory::DeleteObject(m_EnergyDeviation);
		this->SetEnergyDeviation(Mp7JrsFactory::CloneObject(tmp->GetEnergyDeviation()));
	}
	else
	{
		InvalidateEnergyDeviation();
	}
}

Mp7JrsNodePtr Mp7JrsHomogeneousTextureType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsHomogeneousTextureType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsVisualDType > Mp7JrsHomogeneousTextureType::GetBase() const
{
	return m_Base;
}

Mp7Jrsunsigned8Ptr Mp7JrsHomogeneousTextureType::GetAverage() const
{
	return m_Average;
}

Mp7Jrsunsigned8Ptr Mp7JrsHomogeneousTextureType::GetStandardDeviation() const
{
	return m_StandardDeviation;
}

Mp7JrstextureListPtr Mp7JrsHomogeneousTextureType::GetEnergy() const
{
	return m_Energy;
}

Mp7JrstextureListPtr Mp7JrsHomogeneousTextureType::GetEnergyDeviation() const
{
	return m_EnergyDeviation;
}

// Element is optional
bool Mp7JrsHomogeneousTextureType::IsValidEnergyDeviation() const
{
	return m_EnergyDeviation_Exist;
}

void Mp7JrsHomogeneousTextureType::SetAverage(const Mp7Jrsunsigned8Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHomogeneousTextureType::SetAverage().");
	}
	if (m_Average != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Average);
		m_Average = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Average) m_Average->SetParent(m_myself.getPointer());
		if (m_Average && m_Average->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Average->UseTypeAttribute = true;
		}
		if(m_Average != Mp7Jrsunsigned8Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Average->SetContentName(XMLString::transcode("Average"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHomogeneousTextureType::SetStandardDeviation(const Mp7Jrsunsigned8Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHomogeneousTextureType::SetStandardDeviation().");
	}
	if (m_StandardDeviation != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StandardDeviation);
		m_StandardDeviation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StandardDeviation) m_StandardDeviation->SetParent(m_myself.getPointer());
		if (m_StandardDeviation && m_StandardDeviation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_StandardDeviation->UseTypeAttribute = true;
		}
		if(m_StandardDeviation != Mp7Jrsunsigned8Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StandardDeviation->SetContentName(XMLString::transcode("StandardDeviation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHomogeneousTextureType::SetEnergy(const Mp7JrstextureListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHomogeneousTextureType::SetEnergy().");
	}
	if (m_Energy != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Energy);
		m_Energy = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Energy) m_Energy->SetParent(m_myself.getPointer());
		if (m_Energy && m_Energy->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:textureListType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Energy->UseTypeAttribute = true;
		}
		if(m_Energy != Mp7JrstextureListPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Energy->SetContentName(XMLString::transcode("Energy"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHomogeneousTextureType::SetEnergyDeviation(const Mp7JrstextureListPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsHomogeneousTextureType::SetEnergyDeviation().");
	}
	if (m_EnergyDeviation != item || m_EnergyDeviation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_EnergyDeviation);
		m_EnergyDeviation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_EnergyDeviation) m_EnergyDeviation->SetParent(m_myself.getPointer());
		if (m_EnergyDeviation && m_EnergyDeviation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:textureListType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_EnergyDeviation->UseTypeAttribute = true;
		}
		m_EnergyDeviation_Exist = true;
		if(m_EnergyDeviation != Mp7JrstextureListPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_EnergyDeviation->SetContentName(XMLString::transcode("EnergyDeviation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsHomogeneousTextureType::InvalidateEnergyDeviation()
{
	m_EnergyDeviation_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsHomogeneousTextureType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Average")) == 0)
	{
		// Average is simple element unsigned8
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetAverage()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned8Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetAverage(p, client);
					if((p = GetAverage()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("StandardDeviation")) == 0)
	{
		// StandardDeviation is simple element unsigned8
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetStandardDeviation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:unsigned8")))) != empty)
			{
				// Is type allowed
				Mp7Jrsunsigned8Ptr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetStandardDeviation(p, client);
					if((p = GetStandardDeviation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Energy")) == 0)
	{
		// Energy is simple element textureListType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetEnergy()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:textureListType")))) != empty)
			{
				// Is type allowed
				Mp7JrstextureListPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetEnergy(p, client);
					if((p = GetEnergy()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("EnergyDeviation")) == 0)
	{
		// EnergyDeviation is simple element textureListType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetEnergyDeviation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:textureListType")))) != empty)
			{
				// Is type allowed
				Mp7JrstextureListPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetEnergyDeviation(p, client);
					if((p = GetEnergyDeviation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for HomogeneousTextureType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "HomogeneousTextureType");
	}		
	
	return result;
}

XMLCh * Mp7JrsHomogeneousTextureType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsHomogeneousTextureType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsHomogeneousTextureType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("HomogeneousTextureType"));
	// Element serialization:
	// Class with content		
	
	if (m_Average != Mp7Jrsunsigned8Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Average->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Average"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Average->Serialize(doc, element, &elem);
	}
	// Class with content		
	
	if (m_StandardDeviation != Mp7Jrsunsigned8Ptr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_StandardDeviation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("StandardDeviation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_StandardDeviation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Energy != Mp7JrstextureListPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Energy->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Energy"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Energy->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_EnergyDeviation != Mp7JrstextureListPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_EnergyDeviation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("EnergyDeviation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_EnergyDeviation->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsHomogeneousTextureType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsVisualDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Average"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Average")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = Createunsigned8;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetAverage(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("StandardDeviation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("StandardDeviation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = Createunsigned8;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetStandardDeviation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Energy"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Energy")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatetextureListType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetEnergy(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("EnergyDeviation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("EnergyDeviation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatetextureListType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetEnergyDeviation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsHomogeneousTextureType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Average")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = Createunsigned8;
    }
    this->SetAverage(child);
  }
  if (XMLString::compareString(elementname, X("StandardDeviation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = Createunsigned8;
    }
    this->SetStandardDeviation(child);
  }
  if (XMLString::compareString(elementname, X("Energy")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatetextureListType;
    }
    this->SetEnergy(child);
  }
  if (XMLString::compareString(elementname, X("EnergyDeviation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatetextureListType;
    }
    this->SetEnergyDeviation(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsHomogeneousTextureType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetAverage() != Mp7JrsNodePtr())
		result.Insert(GetAverage());
	if (GetStandardDeviation() != Mp7JrsNodePtr())
		result.Insert(GetStandardDeviation());
	if (GetEnergy() != Mp7JrsNodePtr())
		result.Insert(GetEnergy());
	if (GetEnergyDeviation() != Mp7JrsNodePtr())
		result.Insert(GetEnergyDeviation());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsHomogeneousTextureType_ExtMethodImpl.h


