 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtImplInclude.h



#include "Mp7JrsAudioDSType.h"
#include "Mp7JrsLogAttackTimeType.h"
#include "Mp7JrsHarmonicSpectralCentroidType.h"
#include "Mp7JrsHarmonicSpectralDeviationType.h"
#include "Mp7JrsHarmonicSpectralSpreadType.h"
#include "Mp7JrsHarmonicSpectralVariationType.h"
#include "Mp7JrsSpectralCentroidType.h"
#include "Mp7JrsTemporalCentroidType.h"
#include "Mp7JrsintegerVector.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsInstrumentTimbreType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsInstrumentTimbreType::IMp7JrsInstrumentTimbreType()
{

// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtPropInit.h

}

IMp7JrsInstrumentTimbreType::~IMp7JrsInstrumentTimbreType()
{
// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtPropCleanup.h

}

Mp7JrsInstrumentTimbreType::Mp7JrsInstrumentTimbreType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsInstrumentTimbreType::~Mp7JrsInstrumentTimbreType()
{
	Cleanup();
}

void Mp7JrsInstrumentTimbreType::Init()
{
	// Init base
	m_Base = CreateAudioDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_LogAttackTime = Mp7JrsLogAttackTimePtr(); // Class
	m_LogAttackTime_Exist = false;
	m_HarmonicSpectralCentroid = Mp7JrsHarmonicSpectralCentroidPtr(); // Class
	m_HarmonicSpectralCentroid_Exist = false;
	m_HarmonicSpectralDeviation = Mp7JrsHarmonicSpectralDeviationPtr(); // Class
	m_HarmonicSpectralDeviation_Exist = false;
	m_HarmonicSpectralSpread = Mp7JrsHarmonicSpectralSpreadPtr(); // Class
	m_HarmonicSpectralSpread_Exist = false;
	m_HarmonicSpectralVariation = Mp7JrsHarmonicSpectralVariationPtr(); // Class
	m_HarmonicSpectralVariation_Exist = false;
	m_SpectralCentroid = Mp7JrsSpectralCentroidPtr(); // Class
	m_SpectralCentroid_Exist = false;
	m_TemporalCentroid = Mp7JrsTemporalCentroidPtr(); // Class
	m_TemporalCentroid_Exist = false;


// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtMyPropInit.h

}

void Mp7JrsInstrumentTimbreType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_LogAttackTime);
	// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralCentroid);
	// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralDeviation);
	// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralSpread);
	// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralVariation);
	// Mp7JrsFactory::DeleteObject(m_SpectralCentroid);
	// Mp7JrsFactory::DeleteObject(m_TemporalCentroid);
}

void Mp7JrsInstrumentTimbreType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use InstrumentTimbreTypePtr, since we
	// might need GetBase(), which isn't defined in IInstrumentTimbreType
    const Mp7JrsPtr< Mp7JrsInstrumentTimbreType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsAudioDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsInstrumentTimbreType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidLogAttackTime())
	{
		// Mp7JrsFactory::DeleteObject(m_LogAttackTime);
		this->SetLogAttackTime(Mp7JrsFactory::CloneObject(tmp->GetLogAttackTime()));
	}
	else
	{
		InvalidateLogAttackTime();
	}
	if (tmp->IsValidHarmonicSpectralCentroid())
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralCentroid);
		this->SetHarmonicSpectralCentroid(Mp7JrsFactory::CloneObject(tmp->GetHarmonicSpectralCentroid()));
	}
	else
	{
		InvalidateHarmonicSpectralCentroid();
	}
	if (tmp->IsValidHarmonicSpectralDeviation())
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralDeviation);
		this->SetHarmonicSpectralDeviation(Mp7JrsFactory::CloneObject(tmp->GetHarmonicSpectralDeviation()));
	}
	else
	{
		InvalidateHarmonicSpectralDeviation();
	}
	if (tmp->IsValidHarmonicSpectralSpread())
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralSpread);
		this->SetHarmonicSpectralSpread(Mp7JrsFactory::CloneObject(tmp->GetHarmonicSpectralSpread()));
	}
	else
	{
		InvalidateHarmonicSpectralSpread();
	}
	if (tmp->IsValidHarmonicSpectralVariation())
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralVariation);
		this->SetHarmonicSpectralVariation(Mp7JrsFactory::CloneObject(tmp->GetHarmonicSpectralVariation()));
	}
	else
	{
		InvalidateHarmonicSpectralVariation();
	}
	if (tmp->IsValidSpectralCentroid())
	{
		// Mp7JrsFactory::DeleteObject(m_SpectralCentroid);
		this->SetSpectralCentroid(Mp7JrsFactory::CloneObject(tmp->GetSpectralCentroid()));
	}
	else
	{
		InvalidateSpectralCentroid();
	}
	if (tmp->IsValidTemporalCentroid())
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalCentroid);
		this->SetTemporalCentroid(Mp7JrsFactory::CloneObject(tmp->GetTemporalCentroid()));
	}
	else
	{
		InvalidateTemporalCentroid();
	}
}

Mp7JrsNodePtr Mp7JrsInstrumentTimbreType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsInstrumentTimbreType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsAudioDSType > Mp7JrsInstrumentTimbreType::GetBase() const
{
	return m_Base;
}

Mp7JrsLogAttackTimePtr Mp7JrsInstrumentTimbreType::GetLogAttackTime() const
{
	return m_LogAttackTime;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidLogAttackTime() const
{
	return m_LogAttackTime_Exist;
}

Mp7JrsHarmonicSpectralCentroidPtr Mp7JrsInstrumentTimbreType::GetHarmonicSpectralCentroid() const
{
	return m_HarmonicSpectralCentroid;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidHarmonicSpectralCentroid() const
{
	return m_HarmonicSpectralCentroid_Exist;
}

Mp7JrsHarmonicSpectralDeviationPtr Mp7JrsInstrumentTimbreType::GetHarmonicSpectralDeviation() const
{
	return m_HarmonicSpectralDeviation;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidHarmonicSpectralDeviation() const
{
	return m_HarmonicSpectralDeviation_Exist;
}

Mp7JrsHarmonicSpectralSpreadPtr Mp7JrsInstrumentTimbreType::GetHarmonicSpectralSpread() const
{
	return m_HarmonicSpectralSpread;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidHarmonicSpectralSpread() const
{
	return m_HarmonicSpectralSpread_Exist;
}

Mp7JrsHarmonicSpectralVariationPtr Mp7JrsInstrumentTimbreType::GetHarmonicSpectralVariation() const
{
	return m_HarmonicSpectralVariation;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidHarmonicSpectralVariation() const
{
	return m_HarmonicSpectralVariation_Exist;
}

Mp7JrsSpectralCentroidPtr Mp7JrsInstrumentTimbreType::GetSpectralCentroid() const
{
	return m_SpectralCentroid;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidSpectralCentroid() const
{
	return m_SpectralCentroid_Exist;
}

Mp7JrsTemporalCentroidPtr Mp7JrsInstrumentTimbreType::GetTemporalCentroid() const
{
	return m_TemporalCentroid;
}

// Element is optional
bool Mp7JrsInstrumentTimbreType::IsValidTemporalCentroid() const
{
	return m_TemporalCentroid_Exist;
}

void Mp7JrsInstrumentTimbreType::SetLogAttackTime(const Mp7JrsLogAttackTimePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetLogAttackTime().");
	}
	if (m_LogAttackTime != item || m_LogAttackTime_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_LogAttackTime);
		m_LogAttackTime = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_LogAttackTime) m_LogAttackTime->SetParent(m_myself.getPointer());
		if (m_LogAttackTime && m_LogAttackTime->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:LogAttackTimeType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_LogAttackTime->UseTypeAttribute = true;
		}
		m_LogAttackTime_Exist = true;
		if(m_LogAttackTime != Mp7JrsLogAttackTimePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_LogAttackTime->SetContentName(XMLString::transcode("LogAttackTime"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateLogAttackTime()
{
	m_LogAttackTime_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetHarmonicSpectralCentroid(const Mp7JrsHarmonicSpectralCentroidPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetHarmonicSpectralCentroid().");
	}
	if (m_HarmonicSpectralCentroid != item || m_HarmonicSpectralCentroid_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralCentroid);
		m_HarmonicSpectralCentroid = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HarmonicSpectralCentroid) m_HarmonicSpectralCentroid->SetParent(m_myself.getPointer());
		if (m_HarmonicSpectralCentroid && m_HarmonicSpectralCentroid->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralCentroidType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HarmonicSpectralCentroid->UseTypeAttribute = true;
		}
		m_HarmonicSpectralCentroid_Exist = true;
		if(m_HarmonicSpectralCentroid != Mp7JrsHarmonicSpectralCentroidPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HarmonicSpectralCentroid->SetContentName(XMLString::transcode("HarmonicSpectralCentroid"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateHarmonicSpectralCentroid()
{
	m_HarmonicSpectralCentroid_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetHarmonicSpectralDeviation(const Mp7JrsHarmonicSpectralDeviationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetHarmonicSpectralDeviation().");
	}
	if (m_HarmonicSpectralDeviation != item || m_HarmonicSpectralDeviation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralDeviation);
		m_HarmonicSpectralDeviation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HarmonicSpectralDeviation) m_HarmonicSpectralDeviation->SetParent(m_myself.getPointer());
		if (m_HarmonicSpectralDeviation && m_HarmonicSpectralDeviation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralDeviationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HarmonicSpectralDeviation->UseTypeAttribute = true;
		}
		m_HarmonicSpectralDeviation_Exist = true;
		if(m_HarmonicSpectralDeviation != Mp7JrsHarmonicSpectralDeviationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HarmonicSpectralDeviation->SetContentName(XMLString::transcode("HarmonicSpectralDeviation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateHarmonicSpectralDeviation()
{
	m_HarmonicSpectralDeviation_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetHarmonicSpectralSpread(const Mp7JrsHarmonicSpectralSpreadPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetHarmonicSpectralSpread().");
	}
	if (m_HarmonicSpectralSpread != item || m_HarmonicSpectralSpread_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralSpread);
		m_HarmonicSpectralSpread = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HarmonicSpectralSpread) m_HarmonicSpectralSpread->SetParent(m_myself.getPointer());
		if (m_HarmonicSpectralSpread && m_HarmonicSpectralSpread->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralSpreadType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HarmonicSpectralSpread->UseTypeAttribute = true;
		}
		m_HarmonicSpectralSpread_Exist = true;
		if(m_HarmonicSpectralSpread != Mp7JrsHarmonicSpectralSpreadPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HarmonicSpectralSpread->SetContentName(XMLString::transcode("HarmonicSpectralSpread"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateHarmonicSpectralSpread()
{
	m_HarmonicSpectralSpread_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetHarmonicSpectralVariation(const Mp7JrsHarmonicSpectralVariationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetHarmonicSpectralVariation().");
	}
	if (m_HarmonicSpectralVariation != item || m_HarmonicSpectralVariation_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_HarmonicSpectralVariation);
		m_HarmonicSpectralVariation = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_HarmonicSpectralVariation) m_HarmonicSpectralVariation->SetParent(m_myself.getPointer());
		if (m_HarmonicSpectralVariation && m_HarmonicSpectralVariation->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralVariationType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_HarmonicSpectralVariation->UseTypeAttribute = true;
		}
		m_HarmonicSpectralVariation_Exist = true;
		if(m_HarmonicSpectralVariation != Mp7JrsHarmonicSpectralVariationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_HarmonicSpectralVariation->SetContentName(XMLString::transcode("HarmonicSpectralVariation"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateHarmonicSpectralVariation()
{
	m_HarmonicSpectralVariation_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetSpectralCentroid(const Mp7JrsSpectralCentroidPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetSpectralCentroid().");
	}
	if (m_SpectralCentroid != item || m_SpectralCentroid_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_SpectralCentroid);
		m_SpectralCentroid = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpectralCentroid) m_SpectralCentroid->SetParent(m_myself.getPointer());
		if (m_SpectralCentroid && m_SpectralCentroid->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpectralCentroidType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpectralCentroid->UseTypeAttribute = true;
		}
		m_SpectralCentroid_Exist = true;
		if(m_SpectralCentroid != Mp7JrsSpectralCentroidPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpectralCentroid->SetContentName(XMLString::transcode("SpectralCentroid"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateSpectralCentroid()
{
	m_SpectralCentroid_Exist = false;
}
void Mp7JrsInstrumentTimbreType::SetTemporalCentroid(const Mp7JrsTemporalCentroidPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetTemporalCentroid().");
	}
	if (m_TemporalCentroid != item || m_TemporalCentroid_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TemporalCentroid);
		m_TemporalCentroid = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TemporalCentroid) m_TemporalCentroid->SetParent(m_myself.getPointer());
		if (m_TemporalCentroid && m_TemporalCentroid->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalCentroidType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TemporalCentroid->UseTypeAttribute = true;
		}
		m_TemporalCentroid_Exist = true;
		if(m_TemporalCentroid != Mp7JrsTemporalCentroidPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TemporalCentroid->SetContentName(XMLString::transcode("TemporalCentroid"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidateTemporalCentroid()
{
	m_TemporalCentroid_Exist = false;
}
Mp7JrsintegerVectorPtr Mp7JrsInstrumentTimbreType::Getchannels() const
{
	return GetBase()->Getchannels();
}

bool Mp7JrsInstrumentTimbreType::Existchannels() const
{
	return GetBase()->Existchannels();
}
void Mp7JrsInstrumentTimbreType::Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::Setchannels().");
	}
	GetBase()->Setchannels(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::Invalidatechannels()
{
	GetBase()->Invalidatechannels();
}
XMLCh * Mp7JrsInstrumentTimbreType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsInstrumentTimbreType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsInstrumentTimbreType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsInstrumentTimbreType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsInstrumentTimbreType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsInstrumentTimbreType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsInstrumentTimbreType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsInstrumentTimbreType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsInstrumentTimbreType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsInstrumentTimbreType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsInstrumentTimbreType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsInstrumentTimbreType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsInstrumentTimbreType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsInstrumentTimbreType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsInstrumentTimbreType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsInstrumentTimbreType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsInstrumentTimbreType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsInstrumentTimbreType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsInstrumentTimbreType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsInstrumentTimbreType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("LogAttackTime")) == 0)
	{
		// LogAttackTime is simple element LogAttackTimeType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetLogAttackTime()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:LogAttackTimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsLogAttackTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetLogAttackTime(p, client);
					if((p = GetLogAttackTime()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HarmonicSpectralCentroid")) == 0)
	{
		// HarmonicSpectralCentroid is simple element HarmonicSpectralCentroidType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHarmonicSpectralCentroid()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralCentroidType")))) != empty)
			{
				// Is type allowed
				Mp7JrsHarmonicSpectralCentroidPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHarmonicSpectralCentroid(p, client);
					if((p = GetHarmonicSpectralCentroid()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HarmonicSpectralDeviation")) == 0)
	{
		// HarmonicSpectralDeviation is simple element HarmonicSpectralDeviationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHarmonicSpectralDeviation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralDeviationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsHarmonicSpectralDeviationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHarmonicSpectralDeviation(p, client);
					if((p = GetHarmonicSpectralDeviation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HarmonicSpectralSpread")) == 0)
	{
		// HarmonicSpectralSpread is simple element HarmonicSpectralSpreadType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHarmonicSpectralSpread()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralSpreadType")))) != empty)
			{
				// Is type allowed
				Mp7JrsHarmonicSpectralSpreadPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHarmonicSpectralSpread(p, client);
					if((p = GetHarmonicSpectralSpread()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("HarmonicSpectralVariation")) == 0)
	{
		// HarmonicSpectralVariation is simple element HarmonicSpectralVariationType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetHarmonicSpectralVariation()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:HarmonicSpectralVariationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsHarmonicSpectralVariationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetHarmonicSpectralVariation(p, client);
					if((p = GetHarmonicSpectralVariation()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpectralCentroid")) == 0)
	{
		// SpectralCentroid is simple element SpectralCentroidType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpectralCentroid()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SpectralCentroidType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSpectralCentroidPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpectralCentroid(p, client);
					if((p = GetSpectralCentroid()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TemporalCentroid")) == 0)
	{
		// TemporalCentroid is simple element TemporalCentroidType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTemporalCentroid()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalCentroidType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalCentroidPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTemporalCentroid(p, client);
					if((p = GetTemporalCentroid()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for InstrumentTimbreType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "InstrumentTimbreType");
	}		
	
	return result;
}

XMLCh * Mp7JrsInstrumentTimbreType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsInstrumentTimbreType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsInstrumentTimbreType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("InstrumentTimbreType"));
	// Element serialization:
	// Class
	
	if (m_LogAttackTime != Mp7JrsLogAttackTimePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_LogAttackTime->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("LogAttackTime"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_LogAttackTime->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_HarmonicSpectralCentroid != Mp7JrsHarmonicSpectralCentroidPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HarmonicSpectralCentroid->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HarmonicSpectralCentroid"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HarmonicSpectralCentroid->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_HarmonicSpectralDeviation != Mp7JrsHarmonicSpectralDeviationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HarmonicSpectralDeviation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HarmonicSpectralDeviation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HarmonicSpectralDeviation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_HarmonicSpectralSpread != Mp7JrsHarmonicSpectralSpreadPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HarmonicSpectralSpread->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HarmonicSpectralSpread"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HarmonicSpectralSpread->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_HarmonicSpectralVariation != Mp7JrsHarmonicSpectralVariationPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_HarmonicSpectralVariation->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("HarmonicSpectralVariation"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_HarmonicSpectralVariation->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SpectralCentroid != Mp7JrsSpectralCentroidPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpectralCentroid->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpectralCentroid"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpectralCentroid->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TemporalCentroid != Mp7JrsTemporalCentroidPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TemporalCentroid->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TemporalCentroid"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TemporalCentroid->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsInstrumentTimbreType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsAudioDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("LogAttackTime"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("LogAttackTime")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateLogAttackTimeType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetLogAttackTime(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HarmonicSpectralCentroid"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HarmonicSpectralCentroid")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateHarmonicSpectralCentroidType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHarmonicSpectralCentroid(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HarmonicSpectralDeviation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HarmonicSpectralDeviation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateHarmonicSpectralDeviationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHarmonicSpectralDeviation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HarmonicSpectralSpread"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HarmonicSpectralSpread")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateHarmonicSpectralSpreadType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHarmonicSpectralSpread(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("HarmonicSpectralVariation"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("HarmonicSpectralVariation")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateHarmonicSpectralVariationType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetHarmonicSpectralVariation(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpectralCentroid"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpectralCentroid")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSpectralCentroidType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSpectralCentroid(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TemporalCentroid"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TemporalCentroid")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalCentroidType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTemporalCentroid(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsInstrumentTimbreType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("LogAttackTime")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateLogAttackTimeType;
    }
    this->SetLogAttackTime(child);
  }
  if (XMLString::compareString(elementname, X("HarmonicSpectralCentroid")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateHarmonicSpectralCentroidType;
    }
    this->SetHarmonicSpectralCentroid(child);
  }
  if (XMLString::compareString(elementname, X("HarmonicSpectralDeviation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateHarmonicSpectralDeviationType;
    }
    this->SetHarmonicSpectralDeviation(child);
  }
  if (XMLString::compareString(elementname, X("HarmonicSpectralSpread")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateHarmonicSpectralSpreadType;
    }
    this->SetHarmonicSpectralSpread(child);
  }
  if (XMLString::compareString(elementname, X("HarmonicSpectralVariation")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateHarmonicSpectralVariationType;
    }
    this->SetHarmonicSpectralVariation(child);
  }
  if (XMLString::compareString(elementname, X("SpectralCentroid")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSpectralCentroidType;
    }
    this->SetSpectralCentroid(child);
  }
  if (XMLString::compareString(elementname, X("TemporalCentroid")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalCentroidType;
    }
    this->SetTemporalCentroid(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsInstrumentTimbreType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetLogAttackTime() != Mp7JrsNodePtr())
		result.Insert(GetLogAttackTime());
	if (GetHarmonicSpectralCentroid() != Mp7JrsNodePtr())
		result.Insert(GetHarmonicSpectralCentroid());
	if (GetHarmonicSpectralDeviation() != Mp7JrsNodePtr())
		result.Insert(GetHarmonicSpectralDeviation());
	if (GetHarmonicSpectralSpread() != Mp7JrsNodePtr())
		result.Insert(GetHarmonicSpectralSpread());
	if (GetHarmonicSpectralVariation() != Mp7JrsNodePtr())
		result.Insert(GetHarmonicSpectralVariation());
	if (GetSpectralCentroid() != Mp7JrsNodePtr())
		result.Insert(GetSpectralCentroid());
	if (GetTemporalCentroid() != Mp7JrsNodePtr())
		result.Insert(GetTemporalCentroid());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsInstrumentTimbreType_ExtMethodImpl.h


