 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtImplInclude.h



#include "Mp7JrsGraphType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsControlledTermUseType.h"
#include "Mp7JrsGraphType_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsMorphismGraphType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsGraphType_LocalType.h" // Choice collection Node
#include "Mp7JrsGraphType_Node_LocalType.h" // Choice collection element Node
#include "Mp7JrsRelationType.h" // Choice collection element Relation

#include <assert.h>
IMp7JrsMorphismGraphType::IMp7JrsMorphismGraphType()
{

// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtPropInit.h

}

IMp7JrsMorphismGraphType::~IMp7JrsMorphismGraphType()
{
// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtPropCleanup.h

}

Mp7JrsMorphismGraphType::Mp7JrsMorphismGraphType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMorphismGraphType::~Mp7JrsMorphismGraphType()
{
	Cleanup();
}

void Mp7JrsMorphismGraphType::Init()
{
	// Init base
	m_Base = CreateGraphType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_SourceGraphRef = Mp7JrsReferencePtr(); // Class
	m_SourceGraphTermRef = Mp7JrsControlledTermUsePtr(); // Class
	m_TargetGraphRef = Mp7JrsReferencePtr(); // Class
	m_TargetGraphTermRef = Mp7JrsControlledTermUsePtr(); // Class
	m_RuleTermRef = Mp7JrsControlledTermUsePtr(); // Class
	m_RuleTermRef_Exist = false;


// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtMyPropInit.h

}

void Mp7JrsMorphismGraphType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_SourceGraphRef);
	// Mp7JrsFactory::DeleteObject(m_SourceGraphTermRef);
	// Mp7JrsFactory::DeleteObject(m_TargetGraphRef);
	// Mp7JrsFactory::DeleteObject(m_TargetGraphTermRef);
	// Mp7JrsFactory::DeleteObject(m_RuleTermRef);
}

void Mp7JrsMorphismGraphType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MorphismGraphTypePtr, since we
	// might need GetBase(), which isn't defined in IMorphismGraphType
    const Mp7JrsPtr< Mp7JrsMorphismGraphType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsGraphPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMorphismGraphType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_SourceGraphRef);
		this->SetSourceGraphRef(Mp7JrsFactory::CloneObject(tmp->GetSourceGraphRef()));
		// Mp7JrsFactory::DeleteObject(m_SourceGraphTermRef);
		this->SetSourceGraphTermRef(Mp7JrsFactory::CloneObject(tmp->GetSourceGraphTermRef()));
		// Mp7JrsFactory::DeleteObject(m_TargetGraphRef);
		this->SetTargetGraphRef(Mp7JrsFactory::CloneObject(tmp->GetTargetGraphRef()));
		// Mp7JrsFactory::DeleteObject(m_TargetGraphTermRef);
		this->SetTargetGraphTermRef(Mp7JrsFactory::CloneObject(tmp->GetTargetGraphTermRef()));
	if (tmp->IsValidRuleTermRef())
	{
		// Mp7JrsFactory::DeleteObject(m_RuleTermRef);
		this->SetRuleTermRef(Mp7JrsFactory::CloneObject(tmp->GetRuleTermRef()));
	}
	else
	{
		InvalidateRuleTermRef();
	}
}

Mp7JrsNodePtr Mp7JrsMorphismGraphType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMorphismGraphType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsGraphType > Mp7JrsMorphismGraphType::GetBase() const
{
	return m_Base;
}

Mp7JrsReferencePtr Mp7JrsMorphismGraphType::GetSourceGraphRef() const
{
	return m_SourceGraphRef;
}

Mp7JrsControlledTermUsePtr Mp7JrsMorphismGraphType::GetSourceGraphTermRef() const
{
	return m_SourceGraphTermRef;
}

Mp7JrsReferencePtr Mp7JrsMorphismGraphType::GetTargetGraphRef() const
{
	return m_TargetGraphRef;
}

Mp7JrsControlledTermUsePtr Mp7JrsMorphismGraphType::GetTargetGraphTermRef() const
{
	return m_TargetGraphTermRef;
}

Mp7JrsControlledTermUsePtr Mp7JrsMorphismGraphType::GetRuleTermRef() const
{
	return m_RuleTermRef;
}

// Element is optional
bool Mp7JrsMorphismGraphType::IsValidRuleTermRef() const
{
	return m_RuleTermRef_Exist;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMorphismGraphType::SetSourceGraphRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetSourceGraphRef().");
	}
	if (m_SourceGraphRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SourceGraphRef);
		m_SourceGraphRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SourceGraphRef) m_SourceGraphRef->SetParent(m_myself.getPointer());
		if (m_SourceGraphRef && m_SourceGraphRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SourceGraphRef->UseTypeAttribute = true;
		}
		if(m_SourceGraphRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SourceGraphRef->SetContentName(XMLString::transcode("SourceGraphRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_SourceGraphTermRef);
	m_SourceGraphTermRef = Mp7JrsControlledTermUsePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMorphismGraphType::SetSourceGraphTermRef(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetSourceGraphTermRef().");
	}
	if (m_SourceGraphTermRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SourceGraphTermRef);
		m_SourceGraphTermRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SourceGraphTermRef) m_SourceGraphTermRef->SetParent(m_myself.getPointer());
		if (m_SourceGraphTermRef && m_SourceGraphTermRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SourceGraphTermRef->UseTypeAttribute = true;
		}
		if(m_SourceGraphTermRef != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SourceGraphTermRef->SetContentName(XMLString::transcode("SourceGraphTermRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_SourceGraphRef);
	m_SourceGraphRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsMorphismGraphType::SetTargetGraphRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetTargetGraphRef().");
	}
	if (m_TargetGraphRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TargetGraphRef);
		m_TargetGraphRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TargetGraphRef) m_TargetGraphRef->SetParent(m_myself.getPointer());
		if (m_TargetGraphRef && m_TargetGraphRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TargetGraphRef->UseTypeAttribute = true;
		}
		if(m_TargetGraphRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TargetGraphRef->SetContentName(XMLString::transcode("TargetGraphRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_TargetGraphTermRef);
	m_TargetGraphTermRef = Mp7JrsControlledTermUsePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMorphismGraphType::SetTargetGraphTermRef(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetTargetGraphTermRef().");
	}
	if (m_TargetGraphTermRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TargetGraphTermRef);
		m_TargetGraphTermRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TargetGraphTermRef) m_TargetGraphTermRef->SetParent(m_myself.getPointer());
		if (m_TargetGraphTermRef && m_TargetGraphTermRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TargetGraphTermRef->UseTypeAttribute = true;
		}
		if(m_TargetGraphTermRef != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TargetGraphTermRef->SetContentName(XMLString::transcode("TargetGraphTermRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_TargetGraphRef);
	m_TargetGraphRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::SetRuleTermRef(const Mp7JrsControlledTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetRuleTermRef().");
	}
	if (m_RuleTermRef != item || m_RuleTermRef_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_RuleTermRef);
		m_RuleTermRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_RuleTermRef) m_RuleTermRef->SetParent(m_myself.getPointer());
		if (m_RuleTermRef && m_RuleTermRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_RuleTermRef->UseTypeAttribute = true;
		}
		m_RuleTermRef_Exist = true;
		if(m_RuleTermRef != Mp7JrsControlledTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_RuleTermRef->SetContentName(XMLString::transcode("RuleTermRef"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::InvalidateRuleTermRef()
{
	m_RuleTermRef_Exist = false;
}
Mp7JrsGraphType_CollectionPtr Mp7JrsMorphismGraphType::GetGraphType_LocalType() const
{
	return GetBase()->GetGraphType_LocalType();
}

void Mp7JrsMorphismGraphType::SetGraphType_LocalType(const Mp7JrsGraphType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetGraphType_LocalType().");
	}
	GetBase()->SetGraphType_LocalType(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsMorphismGraphType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsMorphismGraphType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsMorphismGraphType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsMorphismGraphType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsMorphismGraphType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsMorphismGraphType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMorphismGraphType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsMorphismGraphType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsMorphismGraphType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMorphismGraphType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMorphismGraphType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMorphismGraphType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMorphismGraphType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMorphismGraphType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMorphismGraphType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMorphismGraphType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsMorphismGraphType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsMorphismGraphType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMorphismGraphType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMorphismGraphType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("SourceGraphRef")) == 0)
	{
		// SourceGraphRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSourceGraphRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSourceGraphRef(p, client);
					if((p = GetSourceGraphRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SourceGraphTermRef")) == 0)
	{
		// SourceGraphTermRef is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSourceGraphTermRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSourceGraphTermRef(p, client);
					if((p = GetSourceGraphTermRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TargetGraphRef")) == 0)
	{
		// TargetGraphRef is simple element ReferenceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTargetGraphRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsReferencePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTargetGraphRef(p, client);
					if((p = GetTargetGraphRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TargetGraphTermRef")) == 0)
	{
		// TargetGraphTermRef is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTargetGraphTermRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTargetGraphTermRef(p, client);
					if((p = GetTargetGraphTermRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("RuleTermRef")) == 0)
	{
		// RuleTermRef is simple element ControlledTermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRuleTermRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRuleTermRef(p, client);
					if((p = GetRuleTermRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MorphismGraphType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MorphismGraphType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMorphismGraphType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMorphismGraphType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMorphismGraphType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MorphismGraphType"));
	// Element serialization:
	// Class
	
	if (m_SourceGraphRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SourceGraphRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SourceGraphRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SourceGraphRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SourceGraphTermRef != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SourceGraphTermRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SourceGraphTermRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SourceGraphTermRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TargetGraphRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TargetGraphRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TargetGraphRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TargetGraphRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TargetGraphTermRef != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TargetGraphTermRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TargetGraphTermRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TargetGraphTermRef->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_RuleTermRef != Mp7JrsControlledTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_RuleTermRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("RuleTermRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_RuleTermRef->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsMorphismGraphType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsGraphType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SourceGraphRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SourceGraphRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSourceGraphRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SourceGraphTermRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SourceGraphTermRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSourceGraphTermRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TargetGraphRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TargetGraphRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTargetGraphRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TargetGraphTermRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TargetGraphTermRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTargetGraphTermRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("RuleTermRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("RuleTermRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateControlledTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRuleTermRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMorphismGraphType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("SourceGraphRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetSourceGraphRef(child);
  }
  if (XMLString::compareString(elementname, X("SourceGraphTermRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetSourceGraphTermRef(child);
  }
  if (XMLString::compareString(elementname, X("TargetGraphRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetTargetGraphRef(child);
  }
  if (XMLString::compareString(elementname, X("TargetGraphTermRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetTargetGraphTermRef(child);
  }
  if (XMLString::compareString(elementname, X("RuleTermRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateControlledTermUseType;
    }
    this->SetRuleTermRef(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMorphismGraphType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRuleTermRef() != Mp7JrsNodePtr())
		result.Insert(GetRuleTermRef());
	if (GetGraphType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetGraphType_LocalType());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetSourceGraphRef() != Mp7JrsNodePtr())
		result.Insert(GetSourceGraphRef());
	if (GetSourceGraphTermRef() != Mp7JrsNodePtr())
		result.Insert(GetSourceGraphTermRef());
	if (GetTargetGraphRef() != Mp7JrsNodePtr())
		result.Insert(GetTargetGraphRef());
	if (GetTargetGraphTermRef() != Mp7JrsNodePtr())
		result.Insert(GetTargetGraphTermRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMorphismGraphType_ExtMethodImpl.h


