 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtImplInclude.h



#include "Mp7JrsMpeg7Type.h"
#include "Mp7JrsMpeg7BaseType.h"
#include "Mp7JrsMpeg7_Description_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDescriptionMetadataType.h"
#include "Mp7JrsMpeg7_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsCompleteDescriptionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Description

#include <assert.h>
IMp7JrsMpeg7_LocalType::IMp7JrsMpeg7_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtPropInit.h

}

IMp7JrsMpeg7_LocalType::~IMp7JrsMpeg7_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtPropCleanup.h

}

Mp7JrsMpeg7_LocalType::Mp7JrsMpeg7_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMpeg7_LocalType::~Mp7JrsMpeg7_LocalType()
{
	Cleanup();
}

void Mp7JrsMpeg7_LocalType::Init()
{
	// Init base
	m_Base = CreateMpeg7Type;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_DescriptionUnit = Mp7JrsPtr< Mp7JrsNode >(); // Class
	m_Description = Mp7JrsMpeg7_Description_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtMyPropInit.h

}

void Mp7JrsMpeg7_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_DescriptionUnit);
	// Mp7JrsFactory::DeleteObject(m_Description);
}

void Mp7JrsMpeg7_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use Mpeg7_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in IMpeg7_LocalType
    const Mp7JrsPtr< Mp7JrsMpeg7_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsMpeg7Ptr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMpeg7_LocalType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_DescriptionUnit);
		this->SetDescriptionUnit(Mp7JrsFactory::CloneObject(tmp->GetDescriptionUnit()));
		// Mp7JrsFactory::DeleteObject(m_Description);
		this->SetDescription(Mp7JrsFactory::CloneObject(tmp->GetDescription()));
}

Mp7JrsNodePtr Mp7JrsMpeg7_LocalType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsMpeg7_LocalType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsMpeg7Type > Mp7JrsMpeg7_LocalType::GetBase() const
{
	return m_Base;
}

Mp7JrsPtr< Mp7JrsNode > Mp7JrsMpeg7_LocalType::GetDescriptionUnit() const
{
	return m_DescriptionUnit;
}

Mp7JrsMpeg7_Description_CollectionPtr Mp7JrsMpeg7_LocalType::GetDescription() const
{
	return m_Description;
}

// implementing setter for choice 
/* element */
void Mp7JrsMpeg7_LocalType::SetDescriptionUnit(const Mp7JrsPtr< Mp7JrsNode > & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SetDescriptionUnit().");
	}
	if (m_DescriptionUnit != item)
	{
		// Mp7JrsFactory::DeleteObject(m_DescriptionUnit);
		m_DescriptionUnit = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DescriptionUnit) m_DescriptionUnit->SetParent(m_myself.getPointer());
		// Element is abstract, so set UseTypeAttribute to true
		if (m_DescriptionUnit) m_DescriptionUnit->UseTypeAttribute = true;
		if(m_DescriptionUnit != Mp7JrsPtr< Mp7JrsNode >()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DescriptionUnit->SetContentName(XMLString::transcode("DescriptionUnit"));
		}
	// Mp7JrsFactory::DeleteObject(m_Description);
	m_Description = Mp7JrsMpeg7_Description_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMpeg7_LocalType::SetDescription(const Mp7JrsMpeg7_Description_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SetDescription().");
	}
	if (m_Description != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Description);
		m_Description = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Description) m_Description->SetParent(m_myself.getPointer());
		if(m_Description != Mp7JrsMpeg7_Description_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Description->SetContentName(XMLString::transcode("Description"));
		}
	// Mp7JrsFactory::DeleteObject(m_DescriptionUnit);
	m_DescriptionUnit = Mp7JrsPtr< Mp7JrsNode >();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsMpeg7_LocalType::Getlang() const
{
	return GetBase()->Getlang();
}

bool Mp7JrsMpeg7_LocalType::Existlang() const
{
	return GetBase()->Existlang();
}
void Mp7JrsMpeg7_LocalType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::Setlang().");
	}
	GetBase()->Setlang(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::Invalidatelang()
{
	GetBase()->Invalidatelang();
}
Mp7JrsxPathRefPtr Mp7JrsMpeg7_LocalType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsMpeg7_LocalType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsMpeg7_LocalType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMpeg7_LocalType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsMpeg7_LocalType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsMpeg7_LocalType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMpeg7_LocalType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMpeg7_LocalType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMpeg7_LocalType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMpeg7_LocalType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMpeg7_LocalType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMpeg7_LocalType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDescriptionMetadataPtr Mp7JrsMpeg7_LocalType::GetDescriptionMetadata() const
{
	return GetBase()->GetDescriptionMetadata();
}

// Element is optional
bool Mp7JrsMpeg7_LocalType::IsValidDescriptionMetadata() const
{
	return GetBase()->IsValidDescriptionMetadata();
}

void Mp7JrsMpeg7_LocalType::SetDescriptionMetadata(const Mp7JrsDescriptionMetadataPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMpeg7_LocalType::SetDescriptionMetadata().");
	}
	GetBase()->SetDescriptionMetadata(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMpeg7_LocalType::InvalidateDescriptionMetadata()
{
	GetBase()->InvalidateDescriptionMetadata();
}

Mp7JrsNodeEnum Mp7JrsMpeg7_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("DescriptionUnit")) == 0)
	{
		// DescriptionUnit is simple abstract element Mpeg7BaseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDescriptionUnit()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsMpeg7BasePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDescriptionUnit(p, client);
					if((p = GetDescriptionUnit()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Description")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Description is item of abstract type CompleteDescriptionType
		// in element collection Mpeg7_Description_CollectionType
		
		context->Found = true;
		Mp7JrsMpeg7_Description_CollectionPtr coll = GetDescription();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMpeg7_Description_CollectionType;
				SetDescription(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsCompleteDescriptionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for Mpeg7_LocalType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "Mpeg7_LocalType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMpeg7_LocalType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMpeg7_LocalType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMpeg7_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("Mpeg7_LocalType"));
	// Element serialization:
	// Class
	
	if (m_DescriptionUnit != Mp7JrsPtr< Mp7JrsNode >())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DescriptionUnit->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DescriptionUnit"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
			// Abstract type, so use xsitype
			m_DescriptionUnit->UseTypeAttribute = true;
		}
		m_DescriptionUnit->Serialize(doc, element, &elem);
	}
	if (m_Description != Mp7JrsMpeg7_Description_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Description->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMpeg7_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsMpeg7Type
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DescriptionUnit"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DescriptionUnit")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateMpeg7BaseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDescriptionUnit(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Description"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Description")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMpeg7_Description_CollectionPtr tmp = CreateMpeg7_Description_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetDescription(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMpeg7_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("DescriptionUnit")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateMpeg7BaseType;
    }
    this->SetDescriptionUnit(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Description")) == 0))
  {
    Mp7JrsMpeg7_Description_CollectionPtr tmp = CreateMpeg7_Description_CollectionType;
    this->SetDescription(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMpeg7_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDescriptionUnit() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionUnit());
	if (GetDescription() != Mp7JrsNodePtr())
		result.Insert(GetDescription());
	if (GetDescriptionMetadata() != Mp7JrsNodePtr())
		result.Insert(GetDescriptionMetadata());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMpeg7_LocalType_ExtMethodImpl.h


