 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtImplInclude.h



#include "Mp7JrsMultimediaContentType.h"
#include "Mp7JrsMultimediaCollectionType_Collection_CollectionType.h"
#include "Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsMultimediaCollectionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsCollectionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Collection
#include "Mp7JrsStructuredCollectionType.h" // Element collection urn:mpeg:mpeg7:schema:2004:StructuredCollection

#include <assert.h>
IMp7JrsMultimediaCollectionType::IMp7JrsMultimediaCollectionType()
{

// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtPropInit.h

}

IMp7JrsMultimediaCollectionType::~IMp7JrsMultimediaCollectionType()
{
// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtPropCleanup.h

}

Mp7JrsMultimediaCollectionType::Mp7JrsMultimediaCollectionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsMultimediaCollectionType::~Mp7JrsMultimediaCollectionType()
{
	Cleanup();
}

void Mp7JrsMultimediaCollectionType::Init()
{
	// Init base
	m_Base = CreateMultimediaContentType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Collection = Mp7JrsMultimediaCollectionType_Collection_CollectionPtr(); // Collection
	m_StructuredCollection = Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtMyPropInit.h

}

void Mp7JrsMultimediaCollectionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Collection);
	// Mp7JrsFactory::DeleteObject(m_StructuredCollection);
}

void Mp7JrsMultimediaCollectionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use MultimediaCollectionTypePtr, since we
	// might need GetBase(), which isn't defined in IMultimediaCollectionType
    const Mp7JrsPtr< Mp7JrsMultimediaCollectionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsMultimediaContentPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsMultimediaCollectionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
		// Mp7JrsFactory::DeleteObject(m_Collection);
		this->SetCollection(Mp7JrsFactory::CloneObject(tmp->GetCollection()));
		// Mp7JrsFactory::DeleteObject(m_StructuredCollection);
		this->SetStructuredCollection(Mp7JrsFactory::CloneObject(tmp->GetStructuredCollection()));
}

Mp7JrsNodePtr Mp7JrsMultimediaCollectionType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsMultimediaCollectionType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsMultimediaContentType > Mp7JrsMultimediaCollectionType::GetBase() const
{
	return m_Base;
}

Mp7JrsMultimediaCollectionType_Collection_CollectionPtr Mp7JrsMultimediaCollectionType::GetCollection() const
{
	return m_Collection;
}

Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr Mp7JrsMultimediaCollectionType::GetStructuredCollection() const
{
	return m_StructuredCollection;
}

// implementing setter for choice 
/* element */
void Mp7JrsMultimediaCollectionType::SetCollection(const Mp7JrsMultimediaCollectionType_Collection_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SetCollection().");
	}
	if (m_Collection != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Collection);
		m_Collection = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Collection) m_Collection->SetParent(m_myself.getPointer());
		if(m_Collection != Mp7JrsMultimediaCollectionType_Collection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Collection->SetContentName(XMLString::transcode("Collection"));
		}
	// Mp7JrsFactory::DeleteObject(m_StructuredCollection);
	m_StructuredCollection = Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsMultimediaCollectionType::SetStructuredCollection(const Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SetStructuredCollection().");
	}
	if (m_StructuredCollection != item)
	{
		// Mp7JrsFactory::DeleteObject(m_StructuredCollection);
		m_StructuredCollection = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_StructuredCollection) m_StructuredCollection->SetParent(m_myself.getPointer());
		if(m_StructuredCollection != Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_StructuredCollection->SetContentName(XMLString::transcode("StructuredCollection"));
		}
	// Mp7JrsFactory::DeleteObject(m_Collection);
	m_Collection = Mp7JrsMultimediaCollectionType_Collection_CollectionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsMultimediaCollectionType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsMultimediaCollectionType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsMultimediaCollectionType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMultimediaCollectionType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsMultimediaCollectionType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsMultimediaCollectionType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsMultimediaCollectionType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMultimediaCollectionType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsMultimediaCollectionType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsMultimediaCollectionType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsMultimediaCollectionType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMultimediaCollectionType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsMultimediaCollectionType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsMultimediaCollectionType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsMultimediaCollectionType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMultimediaCollectionType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsMultimediaCollectionType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsMultimediaCollectionType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsMultimediaCollectionType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsMultimediaCollectionType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsMultimediaCollectionType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsMultimediaCollectionType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsMultimediaCollectionType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsMultimediaCollectionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Collection")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Collection is item of abstract type CollectionType
		// in element collection MultimediaCollectionType_Collection_CollectionType
		
		context->Found = true;
		Mp7JrsMultimediaCollectionType_Collection_CollectionPtr coll = GetCollection();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMultimediaCollectionType_Collection_CollectionType;
				SetCollection(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, 0)) != empty)
			{
				// Is type allowed
				Mp7JrsCollectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("StructuredCollection")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:StructuredCollection is item of type StructuredCollectionType
		// in element collection MultimediaCollectionType_StructuredCollection_CollectionType
		
		context->Found = true;
		Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr coll = GetStructuredCollection();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateMultimediaCollectionType_StructuredCollection_CollectionType;
				SetStructuredCollection(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:StructuredCollectionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsStructuredCollectionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for MultimediaCollectionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "MultimediaCollectionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsMultimediaCollectionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsMultimediaCollectionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsMultimediaCollectionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("MultimediaCollectionType"));
	// Element serialization:
	if (m_Collection != Mp7JrsMultimediaCollectionType_Collection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Collection->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_StructuredCollection != Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_StructuredCollection->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsMultimediaCollectionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsMultimediaContentType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Collection"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Collection")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMultimediaCollectionType_Collection_CollectionPtr tmp = CreateMultimediaCollectionType_Collection_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetCollection(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("StructuredCollection"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("StructuredCollection")) == 0))
        {
            // Deserialize factory type
            Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr tmp = CreateMultimediaCollectionType_StructuredCollection_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetStructuredCollection(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsMultimediaCollectionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Collection")) == 0))
  {
    Mp7JrsMultimediaCollectionType_Collection_CollectionPtr tmp = CreateMultimediaCollectionType_Collection_CollectionType;
    this->SetCollection(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("StructuredCollection")) == 0))
  {
    Mp7JrsMultimediaCollectionType_StructuredCollection_CollectionPtr tmp = CreateMultimediaCollectionType_StructuredCollection_CollectionType;
    this->SetStructuredCollection(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsMultimediaCollectionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetCollection() != Mp7JrsNodePtr())
		result.Insert(GetCollection());
	if (GetStructuredCollection() != Mp7JrsNodePtr())
		result.Insert(GetStructuredCollection());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsMultimediaCollectionType_ExtMethodImpl.h


