 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtImplInclude.h



#include "Mp7JrsLexiconType.h"
#include "Mp7JrsPhoneLexiconType_Token_CollectionType.h"
#include "Mp7JrsPhoneLexiconType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsPhoneType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Token

#include <assert.h>
IMp7JrsPhoneLexiconType::IMp7JrsPhoneLexiconType()
{

// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtPropInit.h

}

IMp7JrsPhoneLexiconType::~IMp7JrsPhoneLexiconType()
{
// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtPropCleanup.h

}

Mp7JrsPhoneLexiconType::Mp7JrsPhoneLexiconType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPhoneLexiconType::~Mp7JrsPhoneLexiconType()
{
	Cleanup();
}

void Mp7JrsPhoneLexiconType::Init()
{
	// Init base
	m_Base = CreateLexiconType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_phoneticAlphabet = Mp7JrsphoneticAlphabetType::UninitializedEnumeration;
// old	m_phoneticAlphabet = Mp7JrsphoneticAlphabetType::Enumeration::;
// old	m_phoneticAlphabet_Default = Mp7JrsphoneticAlphabetType::Enumeration::sampa;
	m_phoneticAlphabet_Default = Mp7JrsphoneticAlphabetType::sampa;
	m_phoneticAlphabet_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Token = Mp7JrsPhoneLexiconType_Token_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtMyPropInit.h

}

void Mp7JrsPhoneLexiconType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Token);
}

void Mp7JrsPhoneLexiconType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PhoneLexiconTypePtr, since we
	// might need GetBase(), which isn't defined in IPhoneLexiconType
    const Mp7JrsPtr< Mp7JrsPhoneLexiconType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsLexiconPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsPhoneLexiconType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistphoneticAlphabet())
	{
		this->SetphoneticAlphabet(tmp->GetphoneticAlphabet());
	}
	else
	{
		InvalidatephoneticAlphabet();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Token);
		this->SetToken(Mp7JrsFactory::CloneObject(tmp->GetToken()));
}

Mp7JrsNodePtr Mp7JrsPhoneLexiconType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsPhoneLexiconType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsLexiconType > Mp7JrsPhoneLexiconType::GetBase() const
{
	return m_Base;
}

Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsPhoneLexiconType::GetphoneticAlphabet() const
{
	if (this->ExistphoneticAlphabet()) {
		return m_phoneticAlphabet;
	} else {
		return m_phoneticAlphabet_Default;
	}
}

bool Mp7JrsPhoneLexiconType::ExistphoneticAlphabet() const
{
	return m_phoneticAlphabet_Exist;
}
void Mp7JrsPhoneLexiconType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPhoneLexiconType::SetphoneticAlphabet().");
	}
    m_phoneticAlphabet = item;
	m_phoneticAlphabet_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPhoneLexiconType::InvalidatephoneticAlphabet()
{
	m_phoneticAlphabet_Exist = false;
}
Mp7JrsPhoneLexiconType_Token_CollectionPtr Mp7JrsPhoneLexiconType::GetToken() const
{
	return m_Token;
}

void Mp7JrsPhoneLexiconType::SetToken(const Mp7JrsPhoneLexiconType_Token_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPhoneLexiconType::SetToken().");
	}
	if (m_Token != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Token);
		m_Token = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Token) m_Token->SetParent(m_myself.getPointer());
		if(m_Token != Mp7JrsPhoneLexiconType_Token_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Token->SetContentName(XMLString::transcode("Token"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

unsigned Mp7JrsPhoneLexiconType::GetnumOfOriginalEntries() const
{
	return GetBase()->GetnumOfOriginalEntries();
}

bool Mp7JrsPhoneLexiconType::ExistnumOfOriginalEntries() const
{
	return GetBase()->ExistnumOfOriginalEntries();
}
void Mp7JrsPhoneLexiconType::SetnumOfOriginalEntries(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPhoneLexiconType::SetnumOfOriginalEntries().");
	}
	GetBase()->SetnumOfOriginalEntries(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPhoneLexiconType::InvalidatenumOfOriginalEntries()
{
	GetBase()->InvalidatenumOfOriginalEntries();
}
XMLCh * Mp7JrsPhoneLexiconType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsPhoneLexiconType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsPhoneLexiconType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPhoneLexiconType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPhoneLexiconType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsPhoneLexiconType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Token")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Token is item of type PhoneType
		// in element collection PhoneLexiconType_Token_CollectionType
		
		context->Found = true;
		Mp7JrsPhoneLexiconType_Token_CollectionPtr coll = GetToken();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePhoneLexiconType_Token_CollectionType;
				SetToken(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), 65536))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PhoneType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPhonePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PhoneLexiconType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PhoneLexiconType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPhoneLexiconType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPhoneLexiconType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPhoneLexiconType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PhoneLexiconType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_phoneticAlphabet_Exist)
	{
	// Enumeration
	if(m_phoneticAlphabet != Mp7JrsphoneticAlphabetType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsphoneticAlphabetType::ToText(m_phoneticAlphabet);
		element->setAttributeNS(X(""), X("phoneticAlphabet"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Token != Mp7JrsPhoneLexiconType_Token_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Token->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsPhoneLexiconType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("phoneticAlphabet")))
	{
        this->SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Parse(parent->getAttributeNS(X(""), X("phoneticAlphabet"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("phoneticAlphabet")))
	{
        this->SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Parse(parent->getAttribute(X("phoneticAlphabet"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsLexiconType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Token"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Token")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPhoneLexiconType_Token_CollectionPtr tmp = CreatePhoneLexiconType_Token_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetToken(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPhoneLexiconType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Token")) == 0))
  {
    Mp7JrsPhoneLexiconType_Token_CollectionPtr tmp = CreatePhoneLexiconType_Token_CollectionType;
    this->SetToken(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPhoneLexiconType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetToken() != Mp7JrsNodePtr())
		result.Insert(GetToken());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPhoneLexiconType_ExtMethodImpl.h


