 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsPlaceType_Name_CollectionType.h"
#include "Mp7JrsPlaceType_NameTerm_CollectionType.h"
#include "Mp7JrsTermUseType.h"
#include "Mp7JrsPlaceType_GeographicPosition_LocalType.h"
#include "Mp7JrsPlaceType_AstronomicalBody_CollectionType.h"
#include "Mp7JrsPlaceType_Region_CollectionType.h"
#include "Mp7JrsPlaceType_AdministrativeUnit_CollectionType.h"
#include "Mp7JrsPlaceType_PostalAddress_LocalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsPlaceType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTextualType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Name
#include "Mp7JrsControlledTermUseType.h" // Element collection urn:mpeg:mpeg7:schema:2004:NameTerm
#include "Mp7JrsregionCode.h" // Element collection urn:mpeg:mpeg7:schema:2004:Region
#include "Mp7JrsPlaceType_AdministrativeUnit_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:AdministrativeUnit

#include <assert.h>
IMp7JrsPlaceType::IMp7JrsPlaceType()
{

// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtPropInit.h

}

IMp7JrsPlaceType::~IMp7JrsPlaceType()
{
// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtPropCleanup.h

}

Mp7JrsPlaceType::Mp7JrsPlaceType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPlaceType::~Mp7JrsPlaceType()
{
	Cleanup();
}

void Mp7JrsPlaceType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_lang = NULL; // String
	m_lang_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Name = Mp7JrsPlaceType_Name_CollectionPtr(); // Collection
	m_NameTerm = Mp7JrsPlaceType_NameTerm_CollectionPtr(); // Collection
	m_Role = Mp7JrsTermUsePtr(); // Class
	m_Role_Exist = false;
	m_GeographicPosition = Mp7JrsPlaceType_GeographicPosition_LocalPtr(); // Class
	m_GeographicPosition_Exist = false;
	m_AstronomicalBody = Mp7JrsPlaceType_AstronomicalBody_CollectionPtr(); // Collection
	m_Region = Mp7JrsPlaceType_Region_CollectionPtr(); // Collection
	m_AdministrativeUnit = Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr(); // Collection
	m_PostalAddress = Mp7JrsPlaceType_PostalAddress_LocalPtr(); // Class
	m_PostalAddress_Exist = false;
	m_InternalCoordinates = NULL; // Optional String
	m_InternalCoordinates_Exist = false;


// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtMyPropInit.h

}

void Mp7JrsPlaceType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_Name);
	// Mp7JrsFactory::DeleteObject(m_NameTerm);
	// Mp7JrsFactory::DeleteObject(m_Role);
	// Mp7JrsFactory::DeleteObject(m_GeographicPosition);
	// Mp7JrsFactory::DeleteObject(m_AstronomicalBody);
	// Mp7JrsFactory::DeleteObject(m_Region);
	// Mp7JrsFactory::DeleteObject(m_AdministrativeUnit);
	// Mp7JrsFactory::DeleteObject(m_PostalAddress);
	XMLString::release(&this->m_InternalCoordinates);
	this->m_InternalCoordinates = NULL;
}

void Mp7JrsPlaceType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PlaceTypePtr, since we
	// might need GetBase(), which isn't defined in IPlaceType
    const Mp7JrsPtr< Mp7JrsPlaceType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsPlaceType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_Name);
		this->SetName(Mp7JrsFactory::CloneObject(tmp->GetName()));
		// Mp7JrsFactory::DeleteObject(m_NameTerm);
		this->SetNameTerm(Mp7JrsFactory::CloneObject(tmp->GetNameTerm()));
	if (tmp->IsValidRole())
	{
		// Mp7JrsFactory::DeleteObject(m_Role);
		this->SetRole(Mp7JrsFactory::CloneObject(tmp->GetRole()));
	}
	else
	{
		InvalidateRole();
	}
	if (tmp->IsValidGeographicPosition())
	{
		// Mp7JrsFactory::DeleteObject(m_GeographicPosition);
		this->SetGeographicPosition(Mp7JrsFactory::CloneObject(tmp->GetGeographicPosition()));
	}
	else
	{
		InvalidateGeographicPosition();
	}
		// Mp7JrsFactory::DeleteObject(m_AstronomicalBody);
		this->SetAstronomicalBody(Mp7JrsFactory::CloneObject(tmp->GetAstronomicalBody()));
		// Mp7JrsFactory::DeleteObject(m_Region);
		this->SetRegion(Mp7JrsFactory::CloneObject(tmp->GetRegion()));
		// Mp7JrsFactory::DeleteObject(m_AdministrativeUnit);
		this->SetAdministrativeUnit(Mp7JrsFactory::CloneObject(tmp->GetAdministrativeUnit()));
	if (tmp->IsValidPostalAddress())
	{
		// Mp7JrsFactory::DeleteObject(m_PostalAddress);
		this->SetPostalAddress(Mp7JrsFactory::CloneObject(tmp->GetPostalAddress()));
	}
	else
	{
		InvalidatePostalAddress();
	}
	if (tmp->IsValidInternalCoordinates())
	{
		this->SetInternalCoordinates(XMLString::replicate(tmp->GetInternalCoordinates()));
	}
	else
	{
		InvalidateInternalCoordinates();
	}
}

Mp7JrsNodePtr Mp7JrsPlaceType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsPlaceType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsPlaceType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsPlaceType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsPlaceType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsPlaceType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsPlaceType_Name_CollectionPtr Mp7JrsPlaceType::GetName() const
{
	return m_Name;
}

Mp7JrsPlaceType_NameTerm_CollectionPtr Mp7JrsPlaceType::GetNameTerm() const
{
	return m_NameTerm;
}

Mp7JrsTermUsePtr Mp7JrsPlaceType::GetRole() const
{
	return m_Role;
}

// Element is optional
bool Mp7JrsPlaceType::IsValidRole() const
{
	return m_Role_Exist;
}

Mp7JrsPlaceType_GeographicPosition_LocalPtr Mp7JrsPlaceType::GetGeographicPosition() const
{
	return m_GeographicPosition;
}

// Element is optional
bool Mp7JrsPlaceType::IsValidGeographicPosition() const
{
	return m_GeographicPosition_Exist;
}

Mp7JrsPlaceType_AstronomicalBody_CollectionPtr Mp7JrsPlaceType::GetAstronomicalBody() const
{
	return m_AstronomicalBody;
}

Mp7JrsPlaceType_Region_CollectionPtr Mp7JrsPlaceType::GetRegion() const
{
	return m_Region;
}

Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr Mp7JrsPlaceType::GetAdministrativeUnit() const
{
	return m_AdministrativeUnit;
}

Mp7JrsPlaceType_PostalAddress_LocalPtr Mp7JrsPlaceType::GetPostalAddress() const
{
	return m_PostalAddress;
}

// Element is optional
bool Mp7JrsPlaceType::IsValidPostalAddress() const
{
	return m_PostalAddress_Exist;
}

XMLCh * Mp7JrsPlaceType::GetInternalCoordinates() const
{
	return m_InternalCoordinates;
}

// Element is optional
bool Mp7JrsPlaceType::IsValidInternalCoordinates() const
{
	return m_InternalCoordinates_Exist;
}

void Mp7JrsPlaceType::SetName(const Mp7JrsPlaceType_Name_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetName().");
	}
	if (m_Name != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Name);
		m_Name = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Name) m_Name->SetParent(m_myself.getPointer());
		if(m_Name != Mp7JrsPlaceType_Name_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Name->SetContentName(XMLString::transcode("Name"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::SetNameTerm(const Mp7JrsPlaceType_NameTerm_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetNameTerm().");
	}
	if (m_NameTerm != item)
	{
		// Mp7JrsFactory::DeleteObject(m_NameTerm);
		m_NameTerm = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_NameTerm) m_NameTerm->SetParent(m_myself.getPointer());
		if(m_NameTerm != Mp7JrsPlaceType_NameTerm_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_NameTerm->SetContentName(XMLString::transcode("NameTerm"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::SetRole(const Mp7JrsTermUsePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetRole().");
	}
	if (m_Role != item || m_Role_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Role);
		m_Role = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Role) m_Role->SetParent(m_myself.getPointer());
		if (m_Role && m_Role->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Role->UseTypeAttribute = true;
		}
		m_Role_Exist = true;
		if(m_Role != Mp7JrsTermUsePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Role->SetContentName(XMLString::transcode("Role"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidateRole()
{
	m_Role_Exist = false;
}
void Mp7JrsPlaceType::SetGeographicPosition(const Mp7JrsPlaceType_GeographicPosition_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetGeographicPosition().");
	}
	if (m_GeographicPosition != item || m_GeographicPosition_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_GeographicPosition);
		m_GeographicPosition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_GeographicPosition) m_GeographicPosition->SetParent(m_myself.getPointer());
		if (m_GeographicPosition && m_GeographicPosition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType_GeographicPosition_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_GeographicPosition->UseTypeAttribute = true;
		}
		m_GeographicPosition_Exist = true;
		if(m_GeographicPosition != Mp7JrsPlaceType_GeographicPosition_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_GeographicPosition->SetContentName(XMLString::transcode("GeographicPosition"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidateGeographicPosition()
{
	m_GeographicPosition_Exist = false;
}
void Mp7JrsPlaceType::SetAstronomicalBody(const Mp7JrsPlaceType_AstronomicalBody_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetAstronomicalBody().");
	}
	if (m_AstronomicalBody != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AstronomicalBody);
		m_AstronomicalBody = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AstronomicalBody) m_AstronomicalBody->SetParent(m_myself.getPointer());
		if(m_AstronomicalBody != Mp7JrsPlaceType_AstronomicalBody_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AstronomicalBody->SetContentName(XMLString::transcode("AstronomicalBody"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::SetRegion(const Mp7JrsPlaceType_Region_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetRegion().");
	}
	if (m_Region != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Region);
		m_Region = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Region) m_Region->SetParent(m_myself.getPointer());
		if(m_Region != Mp7JrsPlaceType_Region_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Region->SetContentName(XMLString::transcode("Region"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::SetAdministrativeUnit(const Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetAdministrativeUnit().");
	}
	if (m_AdministrativeUnit != item)
	{
		// Mp7JrsFactory::DeleteObject(m_AdministrativeUnit);
		m_AdministrativeUnit = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_AdministrativeUnit) m_AdministrativeUnit->SetParent(m_myself.getPointer());
		if(m_AdministrativeUnit != Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_AdministrativeUnit->SetContentName(XMLString::transcode("AdministrativeUnit"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::SetPostalAddress(const Mp7JrsPlaceType_PostalAddress_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetPostalAddress().");
	}
	if (m_PostalAddress != item || m_PostalAddress_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_PostalAddress);
		m_PostalAddress = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PostalAddress) m_PostalAddress->SetParent(m_myself.getPointer());
		if (m_PostalAddress && m_PostalAddress->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType_PostalAddress_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PostalAddress->UseTypeAttribute = true;
		}
		m_PostalAddress_Exist = true;
		if(m_PostalAddress != Mp7JrsPlaceType_PostalAddress_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PostalAddress->SetContentName(XMLString::transcode("PostalAddress"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidatePostalAddress()
{
	m_PostalAddress_Exist = false;
}
void Mp7JrsPlaceType::SetInternalCoordinates(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetInternalCoordinates().");
	}
	if (m_InternalCoordinates != item || m_InternalCoordinates_Exist == false)
	{
		XMLString::release(&m_InternalCoordinates);
		m_InternalCoordinates = item;
		m_InternalCoordinates_Exist = true;
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidateInternalCoordinates()
{
	m_InternalCoordinates_Exist = false;
}
XMLCh * Mp7JrsPlaceType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsPlaceType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsPlaceType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsPlaceType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsPlaceType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsPlaceType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsPlaceType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsPlaceType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsPlaceType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsPlaceType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsPlaceType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsPlaceType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsPlaceType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsPlaceType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsPlaceType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPlaceType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsPlaceType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsPlaceType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPlaceType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsPlaceType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Name")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Name is item of type TextualType
		// in element collection PlaceType_Name_CollectionType
		
		context->Found = true;
		Mp7JrsPlaceType_Name_CollectionPtr coll = GetName();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePlaceType_Name_CollectionType;
				SetName(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("NameTerm")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:NameTerm is item of type ControlledTermUseType
		// in element collection PlaceType_NameTerm_CollectionType
		
		context->Found = true;
		Mp7JrsPlaceType_NameTerm_CollectionPtr coll = GetNameTerm();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePlaceType_NameTerm_CollectionType;
				SetNameTerm(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ControlledTermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsControlledTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Role")) == 0)
	{
		// Role is simple element TermUseType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRole()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRole(p, client);
					if((p = GetRole()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("GeographicPosition")) == 0)
	{
		// GeographicPosition is simple element PlaceType_GeographicPosition_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetGeographicPosition()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType_GeographicPosition_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPlaceType_GeographicPosition_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetGeographicPosition(p, client);
					if((p = GetGeographicPosition()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AstronomicalBody")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AstronomicalBody is item of type TermUseType
		// in element collection PlaceType_AstronomicalBody_CollectionType
		
		context->Found = true;
		Mp7JrsPlaceType_AstronomicalBody_CollectionPtr coll = GetAstronomicalBody();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePlaceType_AstronomicalBody_CollectionType;
				SetAstronomicalBody(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TermUseType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTermUsePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Region")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Region is item of type regionCode
		// in element collection PlaceType_Region_CollectionType
		
		context->Found = true;
		Mp7JrsPlaceType_Region_CollectionPtr coll = GetRegion();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePlaceType_Region_CollectionType;
				SetRegion(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:regionCode")))) != empty)
			{
				// Is type allowed
				Mp7JrsregionCodePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("AdministrativeUnit")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:AdministrativeUnit is item of type PlaceType_AdministrativeUnit_LocalType
		// in element collection PlaceType_AdministrativeUnit_CollectionType
		
		context->Found = true;
		Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr coll = GetAdministrativeUnit();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePlaceType_AdministrativeUnit_CollectionType;
				SetAdministrativeUnit(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType_AdministrativeUnit_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPlaceType_AdministrativeUnit_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("PostalAddress")) == 0)
	{
		// PostalAddress is simple element PlaceType_PostalAddress_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPostalAddress()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType_PostalAddress_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPlaceType_PostalAddress_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPostalAddress(p, client);
					if((p = GetPostalAddress()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PlaceType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PlaceType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPlaceType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPlaceType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPlaceType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PlaceType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_Name != Mp7JrsPlaceType_Name_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Name->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_NameTerm != Mp7JrsPlaceType_NameTerm_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_NameTerm->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_Role != Mp7JrsTermUsePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Role->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Role"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Role->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_GeographicPosition != Mp7JrsPlaceType_GeographicPosition_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_GeographicPosition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("GeographicPosition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_GeographicPosition->Serialize(doc, element, &elem);
	}
	if (m_AstronomicalBody != Mp7JrsPlaceType_AstronomicalBody_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AstronomicalBody->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Region != Mp7JrsPlaceType_Region_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Region->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_AdministrativeUnit != Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_AdministrativeUnit->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	// Class
	
	if (m_PostalAddress != Mp7JrsPlaceType_PostalAddress_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PostalAddress->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PostalAddress"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PostalAddress->Serialize(doc, element, &elem);
	}
	 // String
	if(m_InternalCoordinates != (XMLCh*)NULL) Mp7JrsUtil::SerializeTextNode(element, doc, m_InternalCoordinates, X("urn:mpeg:mpeg7:schema:2004"), X("InternalCoordinates"), true);

}

bool Mp7JrsPlaceType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Name"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Name")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPlaceType_Name_CollectionPtr tmp = CreatePlaceType_Name_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetName(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("NameTerm"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("NameTerm")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPlaceType_NameTerm_CollectionPtr tmp = CreatePlaceType_NameTerm_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetNameTerm(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Role"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Role")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTermUseType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRole(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("GeographicPosition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("GeographicPosition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePlaceType_GeographicPosition_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetGeographicPosition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("AstronomicalBody"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("AstronomicalBody")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPlaceType_AstronomicalBody_CollectionPtr tmp = CreatePlaceType_AstronomicalBody_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAstronomicalBody(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Region"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Region")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPlaceType_Region_CollectionPtr tmp = CreatePlaceType_Region_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetRegion(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("AdministrativeUnit"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("AdministrativeUnit")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr tmp = CreatePlaceType_AdministrativeUnit_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAdministrativeUnit(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PostalAddress"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PostalAddress")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePlaceType_PostalAddress_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPostalAddress(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
	// Deserialize string (XMLCh*) element
	if(Mp7JrsUtil::HasNodeName(parent, X("InternalCoordinates"), X("urn:mpeg:mpeg7:schema:2004")))
	{
		// FTT memleaking		this->SetInternalCoordinates(Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent)));
		this->SetInternalCoordinates(Mp7JrsUtil::GetElementText(parent));
		found = true;
		* current = parent;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPlaceType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Name")) == 0))
  {
    Mp7JrsPlaceType_Name_CollectionPtr tmp = CreatePlaceType_Name_CollectionType;
    this->SetName(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("NameTerm")) == 0))
  {
    Mp7JrsPlaceType_NameTerm_CollectionPtr tmp = CreatePlaceType_NameTerm_CollectionType;
    this->SetNameTerm(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("Role")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTermUseType;
    }
    this->SetRole(child);
  }
  if (XMLString::compareString(elementname, X("GeographicPosition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePlaceType_GeographicPosition_LocalType;
    }
    this->SetGeographicPosition(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AstronomicalBody")) == 0))
  {
    Mp7JrsPlaceType_AstronomicalBody_CollectionPtr tmp = CreatePlaceType_AstronomicalBody_CollectionType;
    this->SetAstronomicalBody(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Region")) == 0))
  {
    Mp7JrsPlaceType_Region_CollectionPtr tmp = CreatePlaceType_Region_CollectionType;
    this->SetRegion(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("AdministrativeUnit")) == 0))
  {
    Mp7JrsPlaceType_AdministrativeUnit_CollectionPtr tmp = CreatePlaceType_AdministrativeUnit_CollectionType;
    this->SetAdministrativeUnit(tmp);
    child = tmp;
  }
  if (XMLString::compareString(elementname, X("PostalAddress")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePlaceType_PostalAddress_LocalType;
    }
    this->SetPostalAddress(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPlaceType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetName() != Mp7JrsNodePtr())
		result.Insert(GetName());
	if (GetNameTerm() != Mp7JrsNodePtr())
		result.Insert(GetNameTerm());
	if (GetRole() != Mp7JrsNodePtr())
		result.Insert(GetRole());
	if (GetGeographicPosition() != Mp7JrsNodePtr())
		result.Insert(GetGeographicPosition());
	if (GetAstronomicalBody() != Mp7JrsNodePtr())
		result.Insert(GetAstronomicalBody());
	if (GetRegion() != Mp7JrsNodePtr())
		result.Insert(GetRegion());
	if (GetAdministrativeUnit() != Mp7JrsNodePtr())
		result.Insert(GetAdministrativeUnit());
	if (GetPostalAddress() != Mp7JrsNodePtr())
		result.Insert(GetPostalAddress());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPlaceType_ExtMethodImpl.h


