 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPositionType_ExtImplInclude.h



#include "Mp7JrsExtentType.h"
#include "Mp7JrsPositionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsPositionType::IMp7JrsPositionType()
{

// no includefile for extension defined 
// file Mp7JrsPositionType_ExtPropInit.h

}

IMp7JrsPositionType::~IMp7JrsPositionType()
{
// no includefile for extension defined 
// file Mp7JrsPositionType_ExtPropCleanup.h

}

Mp7JrsPositionType::Mp7JrsPositionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPositionType::~Mp7JrsPositionType()
{
	Cleanup();
}

void Mp7JrsPositionType::Init()
{

	// Init attributes
	m_origin = NULL; // String
	m_origin_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Displacement = Mp7JrsExtentPtr(); // Class
	m_Displacement_Exist = false;
	m_Direction = Mp7JrsExtentPtr(); // Class
	m_Direction_Exist = false;


// no includefile for extension defined 
// file Mp7JrsPositionType_ExtMyPropInit.h

}

void Mp7JrsPositionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPositionType_ExtMyPropCleanup.h


	XMLString::release(&m_origin);
	// Mp7JrsFactory::DeleteObject(m_Displacement);
	// Mp7JrsFactory::DeleteObject(m_Direction);
}

void Mp7JrsPositionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PositionTypePtr, since we
	// might need GetBase(), which isn't defined in IPositionType
    const Mp7JrsPtr< Mp7JrsPositionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	XMLString::release(&m_origin);
	if (tmp->Existorigin())
	{
		this->Setorigin(XMLString::replicate(tmp->Getorigin()));
	}
	else
	{
		Invalidateorigin();
	}
	}
	if (tmp->IsValidDisplacement())
	{
		// Mp7JrsFactory::DeleteObject(m_Displacement);
		this->SetDisplacement(Mp7JrsFactory::CloneObject(tmp->GetDisplacement()));
	}
	else
	{
		InvalidateDisplacement();
	}
	if (tmp->IsValidDirection())
	{
		// Mp7JrsFactory::DeleteObject(m_Direction);
		this->SetDirection(Mp7JrsFactory::CloneObject(tmp->GetDirection()));
	}
	else
	{
		InvalidateDirection();
	}
}

Mp7JrsNodePtr Mp7JrsPositionType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsPositionType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


XMLCh * Mp7JrsPositionType::Getorigin() const
{
	return m_origin;
}

bool Mp7JrsPositionType::Existorigin() const
{
	return m_origin_Exist;
}
void Mp7JrsPositionType::Setorigin(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPositionType::Setorigin().");
	}
    m_origin = item;
	m_origin_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPositionType::Invalidateorigin()
{
	m_origin_Exist = false;
}
Mp7JrsExtentPtr Mp7JrsPositionType::GetDisplacement() const
{
	return m_Displacement;
}

// Element is optional
bool Mp7JrsPositionType::IsValidDisplacement() const
{
	return m_Displacement_Exist;
}

Mp7JrsExtentPtr Mp7JrsPositionType::GetDirection() const
{
	return m_Direction;
}

// Element is optional
bool Mp7JrsPositionType::IsValidDirection() const
{
	return m_Direction_Exist;
}

void Mp7JrsPositionType::SetDisplacement(const Mp7JrsExtentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPositionType::SetDisplacement().");
	}
	if (m_Displacement != item || m_Displacement_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Displacement);
		m_Displacement = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Displacement) m_Displacement->SetParent(m_myself.getPointer());
		if (m_Displacement && m_Displacement->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ExtentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Displacement->UseTypeAttribute = true;
		}
		m_Displacement_Exist = true;
		if(m_Displacement != Mp7JrsExtentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Displacement->SetContentName(XMLString::transcode("Displacement"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPositionType::InvalidateDisplacement()
{
	m_Displacement_Exist = false;
}
void Mp7JrsPositionType::SetDirection(const Mp7JrsExtentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPositionType::SetDirection().");
	}
	if (m_Direction != item || m_Direction_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Direction);
		m_Direction = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Direction) m_Direction->SetParent(m_myself.getPointer());
		if (m_Direction && m_Direction->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ExtentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Direction->UseTypeAttribute = true;
		}
		m_Direction_Exist = true;
		if(m_Direction != Mp7JrsExtentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Direction->SetContentName(XMLString::transcode("Direction"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPositionType::InvalidateDirection()
{
	m_Direction_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsPositionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Displacement")) == 0)
	{
		// Displacement is simple element ExtentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDisplacement()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ExtentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsExtentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDisplacement(p, client);
					if((p = GetDisplacement()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Direction")) == 0)
	{
		// Direction is simple element ExtentType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDirection()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ExtentType")))) != empty)
			{
				// Is type allowed
				Mp7JrsExtentPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDirection(p, client);
					if((p = GetDirection()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PositionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PositionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPositionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsPositionType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsPositionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PositionType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_origin_Exist)
	{
	// String
	if(m_origin != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("origin"), m_origin);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Displacement != Mp7JrsExtentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Displacement->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Displacement"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Displacement->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Direction != Mp7JrsExtentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Direction->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Direction"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Direction->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsPositionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("origin")))
	{
        // Deserialize string type
        this->Setorigin(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("origin"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("origin")))
	{
        // Deserialize string type
        this->Setorigin(Mp7JrsConvert::TextToString(parent->getAttribute(X("origin"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Displacement"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Displacement")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateExtentType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDisplacement(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Direction"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Direction")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateExtentType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDirection(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsPositionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPositionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("Displacement")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateExtentType;
    }
    this->SetDisplacement(child);
  }
  if (XMLString::compareString(elementname, X("Direction")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateExtentType;
    }
    this->SetDirection(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPositionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetDisplacement() != Mp7JrsNodePtr())
		result.Insert(GetDisplacement());
	if (GetDirection() != Mp7JrsNodePtr())
		result.Insert(GetDirection());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPositionType_ExtMethodImpl.h


