 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtImplInclude.h



#include "Mp7JrsDType.h"
#include "Mp7JrsPlaceType.h"
#include "Mp7JrsPreferenceConditionType_Time_CollectionType.h"
#include "Mp7JrsPreferenceConditionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsPreferenceConditionType_Time_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Time

#include <assert.h>
IMp7JrsPreferenceConditionType::IMp7JrsPreferenceConditionType()
{

// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtPropInit.h

}

IMp7JrsPreferenceConditionType::~IMp7JrsPreferenceConditionType()
{
// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtPropCleanup.h

}

Mp7JrsPreferenceConditionType::Mp7JrsPreferenceConditionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsPreferenceConditionType::~Mp7JrsPreferenceConditionType()
{
	Cleanup();
}

void Mp7JrsPreferenceConditionType::Init()
{
	// Init base
	m_Base = CreateDType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Place = Mp7JrsPlacePtr(); // Class
	m_Place_Exist = false;
	m_Time = Mp7JrsPreferenceConditionType_Time_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtMyPropInit.h

}

void Mp7JrsPreferenceConditionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Place);
	// Mp7JrsFactory::DeleteObject(m_Time);
}

void Mp7JrsPreferenceConditionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use PreferenceConditionTypePtr, since we
	// might need GetBase(), which isn't defined in IPreferenceConditionType
    const Mp7JrsPtr< Mp7JrsPreferenceConditionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsPreferenceConditionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidPlace())
	{
		// Mp7JrsFactory::DeleteObject(m_Place);
		this->SetPlace(Mp7JrsFactory::CloneObject(tmp->GetPlace()));
	}
	else
	{
		InvalidatePlace();
	}
		// Mp7JrsFactory::DeleteObject(m_Time);
		this->SetTime(Mp7JrsFactory::CloneObject(tmp->GetTime()));
}

Mp7JrsNodePtr Mp7JrsPreferenceConditionType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsPreferenceConditionType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDType > Mp7JrsPreferenceConditionType::GetBase() const
{
	return m_Base;
}

Mp7JrsPlacePtr Mp7JrsPreferenceConditionType::GetPlace() const
{
	return m_Place;
}

// Element is optional
bool Mp7JrsPreferenceConditionType::IsValidPlace() const
{
	return m_Place_Exist;
}

Mp7JrsPreferenceConditionType_Time_CollectionPtr Mp7JrsPreferenceConditionType::GetTime() const
{
	return m_Time;
}

void Mp7JrsPreferenceConditionType::SetPlace(const Mp7JrsPlacePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPreferenceConditionType::SetPlace().");
	}
	if (m_Place != item || m_Place_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Place);
		m_Place = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Place) m_Place->SetParent(m_myself.getPointer());
		if (m_Place && m_Place->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Place->UseTypeAttribute = true;
		}
		m_Place_Exist = true;
		if(m_Place != Mp7JrsPlacePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Place->SetContentName(XMLString::transcode("Place"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsPreferenceConditionType::InvalidatePlace()
{
	m_Place_Exist = false;
}
void Mp7JrsPreferenceConditionType::SetTime(const Mp7JrsPreferenceConditionType_Time_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsPreferenceConditionType::SetTime().");
	}
	if (m_Time != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Time);
		m_Time = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Time) m_Time->SetParent(m_myself.getPointer());
		if(m_Time != Mp7JrsPreferenceConditionType_Time_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Time->SetContentName(XMLString::transcode("Time"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsPreferenceConditionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Place")) == 0)
	{
		// Place is simple element PlaceType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetPlace()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PlaceType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPlacePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetPlace(p, client);
					if((p = GetPlace()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Time")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Time is item of type PreferenceConditionType_Time_LocalType
		// in element collection PreferenceConditionType_Time_CollectionType
		
		context->Found = true;
		Mp7JrsPreferenceConditionType_Time_CollectionPtr coll = GetTime();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreatePreferenceConditionType_Time_CollectionType;
				SetTime(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PreferenceConditionType_Time_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPreferenceConditionType_Time_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for PreferenceConditionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "PreferenceConditionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsPreferenceConditionType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsPreferenceConditionType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsPreferenceConditionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("PreferenceConditionType"));
	// Element serialization:
	// Class
	
	if (m_Place != Mp7JrsPlacePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Place->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Place"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Place->Serialize(doc, element, &elem);
	}
	if (m_Time != Mp7JrsPreferenceConditionType_Time_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Time->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsPreferenceConditionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Place"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Place")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePlaceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPlace(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Time"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Time")) == 0))
        {
            // Deserialize factory type
            Mp7JrsPreferenceConditionType_Time_CollectionPtr tmp = CreatePreferenceConditionType_Time_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTime(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsPreferenceConditionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Place")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePlaceType;
    }
    this->SetPlace(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Time")) == 0))
  {
    Mp7JrsPreferenceConditionType_Time_CollectionPtr tmp = CreatePreferenceConditionType_Time_CollectionType;
    this->SetTime(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsPreferenceConditionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetPlace() != Mp7JrsNodePtr())
		result.Insert(GetPlace());
	if (GetTime() != Mp7JrsNodePtr())
		result.Insert(GetTime());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsPreferenceConditionType_ExtMethodImpl.h


