 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtImplInclude.h



#include "Mp7JrsRegionLocatorType_CoordRef_LocalType.h"
#include "Mp7JrsRegionLocatorType_Box_CollectionType.h"
#include "Mp7JrsRegionLocatorType_Polygon_CollectionType.h"
#include "Mp7JrsRegionLocatorType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsRegionLocatorType_Box_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Box
#include "Mp7JrsRegionLocatorType_Polygon_LocalType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Polygon

#include <assert.h>
IMp7JrsRegionLocatorType::IMp7JrsRegionLocatorType()
{

// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtPropInit.h

}

IMp7JrsRegionLocatorType::~IMp7JrsRegionLocatorType()
{
// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtPropCleanup.h

}

Mp7JrsRegionLocatorType::Mp7JrsRegionLocatorType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRegionLocatorType::~Mp7JrsRegionLocatorType()
{
	Cleanup();
}

void Mp7JrsRegionLocatorType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_CoordRef = Mp7JrsRegionLocatorType_CoordRef_LocalPtr(); // Class
	m_CoordRef_Exist = false;
	m_Box = Mp7JrsRegionLocatorType_Box_CollectionPtr(); // Collection
	m_Polygon = Mp7JrsRegionLocatorType_Polygon_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtMyPropInit.h

}

void Mp7JrsRegionLocatorType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_CoordRef);
	// Mp7JrsFactory::DeleteObject(m_Box);
	// Mp7JrsFactory::DeleteObject(m_Polygon);
}

void Mp7JrsRegionLocatorType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RegionLocatorTypePtr, since we
	// might need GetBase(), which isn't defined in IRegionLocatorType
    const Mp7JrsPtr< Mp7JrsRegionLocatorType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	if (tmp->IsValidCoordRef())
	{
		// Mp7JrsFactory::DeleteObject(m_CoordRef);
		this->SetCoordRef(Mp7JrsFactory::CloneObject(tmp->GetCoordRef()));
	}
	else
	{
		InvalidateCoordRef();
	}
		// Mp7JrsFactory::DeleteObject(m_Box);
		this->SetBox(Mp7JrsFactory::CloneObject(tmp->GetBox()));
		// Mp7JrsFactory::DeleteObject(m_Polygon);
		this->SetPolygon(Mp7JrsFactory::CloneObject(tmp->GetPolygon()));
}

Mp7JrsNodePtr Mp7JrsRegionLocatorType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsRegionLocatorType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsRegionLocatorType_CoordRef_LocalPtr Mp7JrsRegionLocatorType::GetCoordRef() const
{
	return m_CoordRef;
}

// Element is optional
bool Mp7JrsRegionLocatorType::IsValidCoordRef() const
{
	return m_CoordRef_Exist;
}

Mp7JrsRegionLocatorType_Box_CollectionPtr Mp7JrsRegionLocatorType::GetBox() const
{
	return m_Box;
}

Mp7JrsRegionLocatorType_Polygon_CollectionPtr Mp7JrsRegionLocatorType::GetPolygon() const
{
	return m_Polygon;
}

void Mp7JrsRegionLocatorType::SetCoordRef(const Mp7JrsRegionLocatorType_CoordRef_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRegionLocatorType::SetCoordRef().");
	}
	if (m_CoordRef != item || m_CoordRef_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_CoordRef);
		m_CoordRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_CoordRef) m_CoordRef->SetParent(m_myself.getPointer());
		if (m_CoordRef && m_CoordRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType_CoordRef_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_CoordRef->UseTypeAttribute = true;
		}
		m_CoordRef_Exist = true;
		if(m_CoordRef != Mp7JrsRegionLocatorType_CoordRef_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_CoordRef->SetContentName(XMLString::transcode("CoordRef"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRegionLocatorType::InvalidateCoordRef()
{
	m_CoordRef_Exist = false;
}
void Mp7JrsRegionLocatorType::SetBox(const Mp7JrsRegionLocatorType_Box_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRegionLocatorType::SetBox().");
	}
	if (m_Box != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Box);
		m_Box = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Box) m_Box->SetParent(m_myself.getPointer());
		if(m_Box != Mp7JrsRegionLocatorType_Box_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Box->SetContentName(XMLString::transcode("Box"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRegionLocatorType::SetPolygon(const Mp7JrsRegionLocatorType_Polygon_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRegionLocatorType::SetPolygon().");
	}
	if (m_Polygon != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Polygon);
		m_Polygon = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Polygon) m_Polygon->SetParent(m_myself.getPointer());
		if(m_Polygon != Mp7JrsRegionLocatorType_Polygon_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Polygon->SetContentName(XMLString::transcode("Polygon"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsRegionLocatorType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("CoordRef")) == 0)
	{
		// CoordRef is simple element RegionLocatorType_CoordRef_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetCoordRef()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType_CoordRef_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionLocatorType_CoordRef_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetCoordRef(p, client);
					if((p = GetCoordRef()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Box")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Box is item of type RegionLocatorType_Box_LocalType
		// in element collection RegionLocatorType_Box_CollectionType
		
		context->Found = true;
		Mp7JrsRegionLocatorType_Box_CollectionPtr coll = GetBox();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateRegionLocatorType_Box_CollectionType;
				SetBox(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType_Box_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionLocatorType_Box_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Polygon")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Polygon is item of type RegionLocatorType_Polygon_LocalType
		// in element collection RegionLocatorType_Polygon_CollectionType
		
		context->Found = true;
		Mp7JrsRegionLocatorType_Polygon_CollectionPtr coll = GetPolygon();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateRegionLocatorType_Polygon_CollectionType;
				SetPolygon(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType_Polygon_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionLocatorType_Polygon_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for RegionLocatorType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "RegionLocatorType");
	}		
	
	return result;
}

XMLCh * Mp7JrsRegionLocatorType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRegionLocatorType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsRegionLocatorType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RegionLocatorType"));
	// Element serialization:
	// Class
	
	if (m_CoordRef != Mp7JrsRegionLocatorType_CoordRef_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_CoordRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("CoordRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_CoordRef->Serialize(doc, element, &elem);
	}
	if (m_Box != Mp7JrsRegionLocatorType_Box_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Box->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_Polygon != Mp7JrsRegionLocatorType_Polygon_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Polygon->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsRegionLocatorType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("CoordRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("CoordRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRegionLocatorType_CoordRef_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetCoordRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Box"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Box")) == 0))
        {
            // Deserialize factory type
            Mp7JrsRegionLocatorType_Box_CollectionPtr tmp = CreateRegionLocatorType_Box_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetBox(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Polygon"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Polygon")) == 0))
        {
            // Deserialize factory type
            Mp7JrsRegionLocatorType_Polygon_CollectionPtr tmp = CreateRegionLocatorType_Polygon_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetPolygon(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRegionLocatorType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("CoordRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRegionLocatorType_CoordRef_LocalType;
    }
    this->SetCoordRef(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Box")) == 0))
  {
    Mp7JrsRegionLocatorType_Box_CollectionPtr tmp = CreateRegionLocatorType_Box_CollectionType;
    this->SetBox(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Polygon")) == 0))
  {
    Mp7JrsRegionLocatorType_Polygon_CollectionPtr tmp = CreateRegionLocatorType_Polygon_CollectionType;
    this->SetPolygon(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRegionLocatorType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetCoordRef() != Mp7JrsNodePtr())
		result.Insert(GetCoordRef());
	if (GetBox() != Mp7JrsNodePtr())
		result.Insert(GetBox());
	if (GetPolygon() != Mp7JrsNodePtr())
		result.Insert(GetPolygon());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRegionLocatorType_ExtMethodImpl.h


