 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtImplInclude.h



#include "Mp7JrsdurationType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsRelIncrTimePointType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsRelIncrTimePointType::IMp7JrsRelIncrTimePointType()
{

// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtPropInit.h

}

IMp7JrsRelIncrTimePointType::~IMp7JrsRelIncrTimePointType()
{
// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtPropCleanup.h

}

Mp7JrsRelIncrTimePointType::Mp7JrsRelIncrTimePointType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRelIncrTimePointType::~Mp7JrsRelIncrTimePointType()
{
	Cleanup();
}

void Mp7JrsRelIncrTimePointType::Init()
{
	// Init base
	m_Base = 0;

	// Init attributes
	m_timeUnit = Mp7JrsdurationPtr(); // Pattern
	m_timeUnit_Exist = false;
	m_timeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_timeBase_Exist = false;



// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtMyPropInit.h

}

void Mp7JrsRelIncrTimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
}

void Mp7JrsRelIncrTimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RelIncrTimePointTypePtr, since we
	// might need GetBase(), which isn't defined in IRelIncrTimePointType
    const Mp7JrsPtr< Mp7JrsRelIncrTimePointType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	m_Base = tmp->GetContent();
	{
	// Mp7JrsFactory::DeleteObject(m_timeUnit); 
	if (tmp->ExisttimeUnit())
	{
		this->SettimeUnit(Mp7JrsFactory::CloneObject(tmp->GettimeUnit()));
	}
	else
	{
		InvalidatetimeUnit();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	if (tmp->ExisttimeBase())
	{
		this->SettimeBase(Mp7JrsFactory::CloneObject(tmp->GettimeBase()));
	}
	else
	{
		InvalidatetimeBase();
	}
	}
}

Mp7JrsNodePtr Mp7JrsRelIncrTimePointType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsRelIncrTimePointType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsRelIncrTimePointType::SetContent(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelIncrTimePointType::SetContent().");
	}
    if (m_Base != item)
    {
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelIncrTimePointType::GetContent() const
{
	return m_Base;
}
Mp7JrsdurationPtr Mp7JrsRelIncrTimePointType::GettimeUnit() const
{
	return m_timeUnit;
}

bool Mp7JrsRelIncrTimePointType::ExisttimeUnit() const
{
	return m_timeUnit_Exist;
}
void Mp7JrsRelIncrTimePointType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelIncrTimePointType::SettimeUnit().");
	}
    m_timeUnit = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeUnit) m_timeUnit->SetParent(m_myself.getPointer());
	m_timeUnit_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRelIncrTimePointType::InvalidatetimeUnit()
{
	m_timeUnit_Exist = false;
}
Mp7JrsxPathRefPtr Mp7JrsRelIncrTimePointType::GettimeBase() const
{
	return m_timeBase;
}

bool Mp7JrsRelIncrTimePointType::ExisttimeBase() const
{
	return m_timeBase_Exist;
}
void Mp7JrsRelIncrTimePointType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelIncrTimePointType::SettimeBase().");
	}
    m_timeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeBase) m_timeBase->SetParent(m_myself.getPointer());
	m_timeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRelIncrTimePointType::InvalidatetimeBase()
{
	m_timeBase_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsRelIncrTimePointType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for RelIncrTimePointType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "RelIncrTimePointType");
	}
	
	return result;
}

XMLCh * Mp7JrsRelIncrTimePointType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRelIncrTimePointType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsRelIncrTimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsRelIncrTimePointType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsRelIncrTimePointType::ContentToString() const
{
	// Mp7JrsRelIncrTimePointType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsRelIncrTimePointType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


    {
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_Base);
		element->appendChild(doc->createTextNode(tmp));
		XMLString::release(&tmp);
    }

	assert(element);
// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RelIncrTimePointType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeUnit_Exist)
	{
	// Pattern
	if (m_timeUnit != Mp7JrsdurationPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeUnit->ToText();
		element->setAttributeNS(X(""), X("timeUnit"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeBase_Exist)
	{
	// Pattern
	if (m_timeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeBase->ToText();
		element->setAttributeNS(X(""), X("timeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsRelIncrTimePointType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeUnit")))
	{
        Mp7JrsdurationPtr tmp = CreatedurationType;
        tmp->Parse(parent->getAttribute(X("timeUnit")));
        this->SettimeUnit(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
    }

  m_Base = Mp7JrsConvert::TextToInt(Mp7JrsUtil::GetElementText(parent));
  found = true;
// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRelIncrTimePointType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRelIncrTimePointType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRelIncrTimePointType_ExtMethodImpl.h


