 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtImplInclude.h



#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsRelTimePointType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsRelTimePointType::IMp7JrsRelTimePointType()
{

// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtPropInit.h

}

IMp7JrsRelTimePointType::~IMp7JrsRelTimePointType()
{
// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtPropCleanup.h

}

Mp7JrsRelTimePointType::Mp7JrsRelTimePointType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsRelTimePointType::~Mp7JrsRelTimePointType()
{
	Cleanup();
}

void Mp7JrsRelTimePointType::Init()
{
	// Init base
	m_Base = CreatetimeOffsetType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_timeBase = Mp7JrsxPathRefPtr(); // Pattern
	m_timeBase_Exist = false;



// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtMyPropInit.h

}

void Mp7JrsRelTimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
}

void Mp7JrsRelTimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use RelTimePointTypePtr, since we
	// might need GetBase(), which isn't defined in IRelTimePointType
    const Mp7JrsPtr< Mp7JrsRelTimePointType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrstimeOffsetPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsRelTimePointType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_timeBase); 
	if (tmp->ExisttimeBase())
	{
		this->SettimeBase(Mp7JrsFactory::CloneObject(tmp->GettimeBase()));
	}
	else
	{
		InvalidatetimeBase();
	}
	}
}

Mp7JrsNodePtr Mp7JrsRelTimePointType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsRelTimePointType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrstimeOffsetType > Mp7JrsRelTimePointType::GetBase() const
{
	return m_Base;
}

Mp7JrsxPathRefPtr Mp7JrsRelTimePointType::GettimeBase() const
{
	return m_timeBase;
}

bool Mp7JrsRelTimePointType::ExisttimeBase() const
{
	return m_timeBase_Exist;
}
void Mp7JrsRelTimePointType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SettimeBase().");
	}
    m_timeBase = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_timeBase) m_timeBase->SetParent(m_myself.getPointer());
	m_timeBase_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsRelTimePointType::InvalidatetimeBase()
{
	m_timeBase_Exist = false;
}
int Mp7JrsRelTimePointType::GetOffsetSign() const
{
	return GetBase()->GetOffsetSign();
}

void Mp7JrsRelTimePointType::SetOffsetSign(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetOffsetSign().");
	}
	GetBase()->SetOffsetSign(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetDays() const
{
	return GetBase()->GetDays();
}

void Mp7JrsRelTimePointType::SetDays(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetDays().");
	}
	GetBase()->SetDays(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetHours() const
{
	return GetBase()->GetHours();
}

void Mp7JrsRelTimePointType::SetHours(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetHours().");
	}
	GetBase()->SetHours(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetMinutes() const
{
	return GetBase()->GetMinutes();
}

void Mp7JrsRelTimePointType::SetMinutes(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetMinutes().");
	}
	GetBase()->SetMinutes(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetSeconds() const
{
	return GetBase()->GetSeconds();
}

void Mp7JrsRelTimePointType::SetSeconds(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetSeconds().");
	}
	GetBase()->SetSeconds(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetNumberOfFractions() const
{
	return GetBase()->GetNumberOfFractions();
}

void Mp7JrsRelTimePointType::SetNumberOfFractions(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetNumberOfFractions().");
	}
	GetBase()->SetNumberOfFractions(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsRelTimePointType::GetTime_Exist() const
{
	return GetBase()->GetTime_Exist();
}

void Mp7JrsRelTimePointType::SetTime_Exist(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetTime_Exist().");
	}
	GetBase()->SetTime_Exist(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetFractionsPerSecond() const
{
	return GetBase()->GetFractionsPerSecond();
}

void Mp7JrsRelTimePointType::SetFractionsPerSecond(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetFractionsPerSecond().");
	}
	GetBase()->SetFractionsPerSecond(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetTimezoneSign() const
{
	return GetBase()->GetTimezoneSign();
}

void Mp7JrsRelTimePointType::SetTimezoneSign(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetTimezoneSign().");
	}
	GetBase()->SetTimezoneSign(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetTimezoneHours() const
{
	return GetBase()->GetTimezoneHours();
}

void Mp7JrsRelTimePointType::SetTimezoneHours(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetTimezoneHours().");
	}
	GetBase()->SetTimezoneHours(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

int Mp7JrsRelTimePointType::GetTimezoneMinutes() const
{
	return GetBase()->GetTimezoneMinutes();
}

void Mp7JrsRelTimePointType::SetTimezoneMinutes(int item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetTimezoneMinutes().");
	}
	GetBase()->SetTimezoneMinutes(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

bool Mp7JrsRelTimePointType::GetTimezone_Exist() const
{
	return GetBase()->GetTimezone_Exist();
}

void Mp7JrsRelTimePointType::SetTimezone_Exist(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsRelTimePointType::SetTimezone_Exist().");
	}
	GetBase()->SetTimezone_Exist(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsRelTimePointType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for RelTimePointType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "RelTimePointType");
	}
	
	return result;
}

XMLCh * Mp7JrsRelTimePointType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsRelTimePointType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsRelTimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsRelTimePointType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsRelTimePointType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    m_Base->Serialize(doc, element, newElem);

	assert(element);
// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("RelTimePointType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_timeBase_Exist)
	{
	// Pattern
	if (m_timeBase != Mp7JrsxPathRefPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_timeBase->ToText();
		element->setAttributeNS(X(""), X("timeBase"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsRelTimePointType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttributeNS(X(""), X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("timeBase")))
	{
        Mp7JrsxPathRefPtr tmp = CreatexPathRefType;
        tmp->Parse(parent->getAttribute(X("timeBase")));
        this->SettimeBase(tmp);
        * current = parent;
    }

  // Extensionbase is Mp7JrstimeOffsetType
  // Deserialize cce factory type
  if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
  }
// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsRelTimePointType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsRelTimePointType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsRelTimePointType_ExtMethodImpl.h


