 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSegmentDecompositionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsSegmentDecompositionType::IMp7JrsSegmentDecompositionType()
{

// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtPropInit.h

}

IMp7JrsSegmentDecompositionType::~IMp7JrsSegmentDecompositionType()
{
// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtPropCleanup.h

}

Mp7JrsSegmentDecompositionType::Mp7JrsSegmentDecompositionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSegmentDecompositionType::~Mp7JrsSegmentDecompositionType()
{
	Cleanup();
}

void Mp7JrsSegmentDecompositionType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_criteria = NULL; // String
	m_criteria_Exist = false;
	m_overlap = false; // Value
	m_overlap_Default = false;
	m_overlap_Exist = false;
	m_gap = false; // Value
	m_gap_Default = false;
	m_gap_Exist = false;



// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtMyPropInit.h

}

void Mp7JrsSegmentDecompositionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	XMLString::release(&m_criteria);
}

void Mp7JrsSegmentDecompositionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SegmentDecompositionTypePtr, since we
	// might need GetBase(), which isn't defined in ISegmentDecompositionType
    const Mp7JrsPtr< Mp7JrsSegmentDecompositionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSegmentDecompositionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	XMLString::release(&m_criteria);
	if (tmp->Existcriteria())
	{
		this->Setcriteria(XMLString::replicate(tmp->Getcriteria()));
	}
	else
	{
		Invalidatecriteria();
	}
	}
	{
	if (tmp->Existoverlap())
	{
		this->Setoverlap(tmp->Getoverlap());
	}
	else
	{
		Invalidateoverlap();
	}
	}
	{
	if (tmp->Existgap())
	{
		this->Setgap(tmp->Getgap());
	}
	else
	{
		Invalidategap();
	}
	}
}

Mp7JrsNodePtr Mp7JrsSegmentDecompositionType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSegmentDecompositionType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsSegmentDecompositionType::GetBase() const
{
	return m_Base;
}

XMLCh * Mp7JrsSegmentDecompositionType::Getcriteria() const
{
	return m_criteria;
}

bool Mp7JrsSegmentDecompositionType::Existcriteria() const
{
	return m_criteria_Exist;
}
void Mp7JrsSegmentDecompositionType::Setcriteria(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::Setcriteria().");
	}
    m_criteria = item;
	m_criteria_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::Invalidatecriteria()
{
	m_criteria_Exist = false;
}
bool Mp7JrsSegmentDecompositionType::Getoverlap() const
{
	if (this->Existoverlap()) {
		return m_overlap;
	} else {
		return m_overlap_Default;
	}
}

bool Mp7JrsSegmentDecompositionType::Existoverlap() const
{
	return m_overlap_Exist;
}
void Mp7JrsSegmentDecompositionType::Setoverlap(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::Setoverlap().");
	}
    m_overlap = item;
	m_overlap_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::Invalidateoverlap()
{
	m_overlap_Exist = false;
}
bool Mp7JrsSegmentDecompositionType::Getgap() const
{
	if (this->Existgap()) {
		return m_gap;
	} else {
		return m_gap_Default;
	}
}

bool Mp7JrsSegmentDecompositionType::Existgap() const
{
	return m_gap_Exist;
}
void Mp7JrsSegmentDecompositionType::Setgap(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::Setgap().");
	}
    m_gap = item;
	m_gap_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::Invalidategap()
{
	m_gap_Exist = false;
}
XMLCh * Mp7JrsSegmentDecompositionType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsSegmentDecompositionType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsSegmentDecompositionType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSegmentDecompositionType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsSegmentDecompositionType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsSegmentDecompositionType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSegmentDecompositionType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsSegmentDecompositionType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsSegmentDecompositionType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSegmentDecompositionType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSegmentDecompositionType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSegmentDecompositionType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSegmentDecompositionType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSegmentDecompositionType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSegmentDecompositionType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSegmentDecompositionType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSegmentDecompositionType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsSegmentDecompositionType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSegmentDecompositionType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSegmentDecompositionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SegmentDecompositionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SegmentDecompositionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSegmentDecompositionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSegmentDecompositionType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsSegmentDecompositionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SegmentDecompositionType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_criteria_Exist)
	{
	// String
	if(m_criteria != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("criteria"), m_criteria);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_overlap_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_overlap);
		element->setAttributeNS(X(""), X("overlap"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_gap_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_gap);
		element->setAttributeNS(X(""), X("gap"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsSegmentDecompositionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("criteria")))
	{
        // Deserialize string type
        this->Setcriteria(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("criteria"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("criteria")))
	{
        // Deserialize string type
        this->Setcriteria(Mp7JrsConvert::TextToString(parent->getAttribute(X("criteria"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("overlap")))
	{
        // deserialize value type
        this->Setoverlap(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("overlap"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("overlap")))
	{
        // deserialize value type
        this->Setoverlap(Mp7JrsConvert::TextToBool(parent->getAttribute(X("overlap"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("gap")))
	{
        // deserialize value type
        this->Setgap(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("gap"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("gap")))
	{
        // deserialize value type
        this->Setgap(Mp7JrsConvert::TextToBool(parent->getAttribute(X("gap"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSegmentDecompositionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSegmentDecompositionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSegmentDecompositionType_ExtMethodImpl.h


