 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtImplInclude.h



#include "Mp7JrsViewType.h"
#include "Mp7JrsPartitionType.h"
#include "Mp7JrsRegionLocatorType.h"
#include "Mp7JrsFilteringType.h"
#include "Mp7JrsSignalPlaneSampleType.h"
#include "Mp7JrsSignalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsSpaceFrequencyViewType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsSpaceFrequencyViewType::IMp7JrsSpaceFrequencyViewType()
{

// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtPropInit.h

}

IMp7JrsSpaceFrequencyViewType::~IMp7JrsSpaceFrequencyViewType()
{
// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtPropCleanup.h

}

Mp7JrsSpaceFrequencyViewType::Mp7JrsSpaceFrequencyViewType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpaceFrequencyViewType::~Mp7JrsSpaceFrequencyViewType()
{
	Cleanup();
}

void Mp7JrsSpaceFrequencyViewType::Init()
{
	// Init base
	m_Base = CreateViewType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_order = Mp7JrsSpaceFrequencyViewType_order_LocalType::UninitializedEnumeration;
// old	m_order = Mp7JrsSpaceFrequencyViewType_order_LocalType::Enumeration::;
// old	m_order_Default = Mp7JrsSpaceFrequencyViewType_order_LocalType::Enumeration::spaceFrequency;
	m_order_Default = Mp7JrsSpaceFrequencyViewType_order_LocalType::spaceFrequency;
	m_order_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_SpacePartition = Mp7JrsPartitionPtr(); // Class
	m_SpaceRegion = Mp7JrsRegionLocatorPtr(); // Class
	m_FrequencyPartition = Mp7JrsPartitionPtr(); // Class
	m_Filtering = Mp7JrsFilteringPtr(); // Class
	m_Filtering_Exist = false;
	m_DownSamplingFactor = Mp7JrsSignalPlaneSamplePtr(); // Class
	m_DownSamplingFactor_Exist = false;


// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtMyPropInit.h

}

void Mp7JrsSpaceFrequencyViewType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_SpacePartition);
	// Mp7JrsFactory::DeleteObject(m_SpaceRegion);
	// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
	// Mp7JrsFactory::DeleteObject(m_Filtering);
	// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
}

void Mp7JrsSpaceFrequencyViewType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpaceFrequencyViewTypePtr, since we
	// might need GetBase(), which isn't defined in ISpaceFrequencyViewType
    const Mp7JrsPtr< Mp7JrsSpaceFrequencyViewType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsViewPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsSpaceFrequencyViewType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existorder())
	{
		this->Setorder(tmp->Getorder());
	}
	else
	{
		Invalidateorder();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_SpacePartition);
		this->SetSpacePartition(Mp7JrsFactory::CloneObject(tmp->GetSpacePartition()));
		// Mp7JrsFactory::DeleteObject(m_SpaceRegion);
		this->SetSpaceRegion(Mp7JrsFactory::CloneObject(tmp->GetSpaceRegion()));
		// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
		this->SetFrequencyPartition(Mp7JrsFactory::CloneObject(tmp->GetFrequencyPartition()));
	if (tmp->IsValidFiltering())
	{
		// Mp7JrsFactory::DeleteObject(m_Filtering);
		this->SetFiltering(Mp7JrsFactory::CloneObject(tmp->GetFiltering()));
	}
	else
	{
		InvalidateFiltering();
	}
	if (tmp->IsValidDownSamplingFactor())
	{
		// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
		this->SetDownSamplingFactor(Mp7JrsFactory::CloneObject(tmp->GetDownSamplingFactor()));
	}
	else
	{
		InvalidateDownSamplingFactor();
	}
}

Mp7JrsNodePtr Mp7JrsSpaceFrequencyViewType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsSpaceFrequencyViewType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsViewType > Mp7JrsSpaceFrequencyViewType::GetBase() const
{
	return m_Base;
}

Mp7JrsSpaceFrequencyViewType_order_LocalType::Enumeration Mp7JrsSpaceFrequencyViewType::Getorder() const
{
	if (this->Existorder()) {
		return m_order;
	} else {
		return m_order_Default;
	}
}

bool Mp7JrsSpaceFrequencyViewType::Existorder() const
{
	return m_order_Exist;
}
void Mp7JrsSpaceFrequencyViewType::Setorder(Mp7JrsSpaceFrequencyViewType_order_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::Setorder().");
	}
    m_order = item;
	m_order_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::Invalidateorder()
{
	m_order_Exist = false;
}
Mp7JrsPartitionPtr Mp7JrsSpaceFrequencyViewType::GetSpacePartition() const
{
	return m_SpacePartition;
}

Mp7JrsRegionLocatorPtr Mp7JrsSpaceFrequencyViewType::GetSpaceRegion() const
{
	return m_SpaceRegion;
}

Mp7JrsPartitionPtr Mp7JrsSpaceFrequencyViewType::GetFrequencyPartition() const
{
	return m_FrequencyPartition;
}

Mp7JrsFilteringPtr Mp7JrsSpaceFrequencyViewType::GetFiltering() const
{
	return m_Filtering;
}

// Element is optional
bool Mp7JrsSpaceFrequencyViewType::IsValidFiltering() const
{
	return m_Filtering_Exist;
}

Mp7JrsSignalPlaneSamplePtr Mp7JrsSpaceFrequencyViewType::GetDownSamplingFactor() const
{
	return m_DownSamplingFactor;
}

// Element is optional
bool Mp7JrsSpaceFrequencyViewType::IsValidDownSamplingFactor() const
{
	return m_DownSamplingFactor_Exist;
}

	// TODO Implement proper cleanup for nested choices (e.g. CompositionShotEditinbTemporalDecompositionType_LocalType)
// implementing setter for choice 
/* element */
void Mp7JrsSpaceFrequencyViewType::SetSpacePartition(const Mp7JrsPartitionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetSpacePartition().");
	}
	if (m_SpacePartition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpacePartition);
		m_SpacePartition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpacePartition) m_SpacePartition->SetParent(m_myself.getPointer());
		if (m_SpacePartition && m_SpacePartition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpacePartition->UseTypeAttribute = true;
		}
		if(m_SpacePartition != Mp7JrsPartitionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpacePartition->SetContentName(XMLString::transcode("SpacePartition"));
		}
	// Mp7JrsFactory::DeleteObject(m_SpaceRegion);
	m_SpaceRegion = Mp7JrsRegionLocatorPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSpaceFrequencyViewType::SetSpaceRegion(const Mp7JrsRegionLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetSpaceRegion().");
	}
	if (m_SpaceRegion != item)
	{
		// Mp7JrsFactory::DeleteObject(m_SpaceRegion);
		m_SpaceRegion = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_SpaceRegion) m_SpaceRegion->SetParent(m_myself.getPointer());
		if (m_SpaceRegion && m_SpaceRegion->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_SpaceRegion->UseTypeAttribute = true;
		}
		if(m_SpaceRegion != Mp7JrsRegionLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_SpaceRegion->SetContentName(XMLString::transcode("SpaceRegion"));
		}
	// Mp7JrsFactory::DeleteObject(m_SpacePartition);
	m_SpacePartition = Mp7JrsPartitionPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::SetFrequencyPartition(const Mp7JrsPartitionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetFrequencyPartition().");
	}
	if (m_FrequencyPartition != item)
	{
		// Mp7JrsFactory::DeleteObject(m_FrequencyPartition);
		m_FrequencyPartition = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FrequencyPartition) m_FrequencyPartition->SetParent(m_myself.getPointer());
		if (m_FrequencyPartition && m_FrequencyPartition->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FrequencyPartition->UseTypeAttribute = true;
		}
		if(m_FrequencyPartition != Mp7JrsPartitionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FrequencyPartition->SetContentName(XMLString::transcode("FrequencyPartition"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::SetFiltering(const Mp7JrsFilteringPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetFiltering().");
	}
	if (m_Filtering != item || m_Filtering_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Filtering);
		m_Filtering = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Filtering) m_Filtering->SetParent(m_myself.getPointer());
		if (m_Filtering && m_Filtering->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FilteringType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Filtering->UseTypeAttribute = true;
		}
		m_Filtering_Exist = true;
		if(m_Filtering != Mp7JrsFilteringPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Filtering->SetContentName(XMLString::transcode("Filtering"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidateFiltering()
{
	m_Filtering_Exist = false;
}
void Mp7JrsSpaceFrequencyViewType::SetDownSamplingFactor(const Mp7JrsSignalPlaneSamplePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetDownSamplingFactor().");
	}
	if (m_DownSamplingFactor != item || m_DownSamplingFactor_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_DownSamplingFactor);
		m_DownSamplingFactor = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_DownSamplingFactor) m_DownSamplingFactor->SetParent(m_myself.getPointer());
		if (m_DownSamplingFactor && m_DownSamplingFactor->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_DownSamplingFactor->UseTypeAttribute = true;
		}
		m_DownSamplingFactor_Exist = true;
		if(m_DownSamplingFactor != Mp7JrsSignalPlaneSamplePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_DownSamplingFactor->SetContentName(XMLString::transcode("DownSamplingFactor"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidateDownSamplingFactor()
{
	m_DownSamplingFactor_Exist = false;
}
Mp7JrsSignalPtr Mp7JrsSpaceFrequencyViewType::GetTarget() const
{
	return GetBase()->GetTarget();
}

Mp7JrsSignalPtr Mp7JrsSpaceFrequencyViewType::GetSource() const
{
	return GetBase()->GetSource();
}

// Element is optional
bool Mp7JrsSpaceFrequencyViewType::IsValidSource() const
{
	return GetBase()->IsValidSource();
}

void Mp7JrsSpaceFrequencyViewType::SetTarget(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetTarget().");
	}
	GetBase()->SetTarget(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::SetSource(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetSource().");
	}
	GetBase()->SetSource(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidateSource()
{
	GetBase()->InvalidateSource();
}
XMLCh * Mp7JrsSpaceFrequencyViewType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsSpaceFrequencyViewType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsSpaceFrequencyViewType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsSpaceFrequencyViewType::GettimeBase() const
{
	return GetBase()->GetBase()->GettimeBase();
}

bool Mp7JrsSpaceFrequencyViewType::ExisttimeBase() const
{
	return GetBase()->GetBase()->ExisttimeBase();
}
void Mp7JrsSpaceFrequencyViewType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SettimeBase().");
	}
	GetBase()->GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidatetimeBase()
{
	GetBase()->GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsSpaceFrequencyViewType::GettimeUnit() const
{
	return GetBase()->GetBase()->GettimeUnit();
}

bool Mp7JrsSpaceFrequencyViewType::ExisttimeUnit() const
{
	return GetBase()->GetBase()->ExisttimeUnit();
}
void Mp7JrsSpaceFrequencyViewType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SettimeUnit().");
	}
	GetBase()->GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidatetimeUnit()
{
	GetBase()->GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsSpaceFrequencyViewType::GetmediaTimeBase() const
{
	return GetBase()->GetBase()->GetmediaTimeBase();
}

bool Mp7JrsSpaceFrequencyViewType::ExistmediaTimeBase() const
{
	return GetBase()->GetBase()->ExistmediaTimeBase();
}
void Mp7JrsSpaceFrequencyViewType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetmediaTimeBase().");
	}
	GetBase()->GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidatemediaTimeBase()
{
	GetBase()->GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsSpaceFrequencyViewType::GetmediaTimeUnit() const
{
	return GetBase()->GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsSpaceFrequencyViewType::ExistmediaTimeUnit() const
{
	return GetBase()->GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsSpaceFrequencyViewType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetmediaTimeUnit().");
	}
	GetBase()->GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpaceFrequencyViewType::InvalidatemediaTimeUnit()
{
	GetBase()->GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsSpaceFrequencyViewType::GetHeader() const
{
	return GetBase()->GetBase()->GetHeader();
}

void Mp7JrsSpaceFrequencyViewType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpaceFrequencyViewType::SetHeader().");
	}
	GetBase()->GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsSpaceFrequencyViewType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("SpacePartition")) == 0)
	{
		// SpacePartition is simple element PartitionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpacePartition()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPartitionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpacePartition(p, client);
					if((p = GetSpacePartition()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("SpaceRegion")) == 0)
	{
		// SpaceRegion is simple element RegionLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSpaceRegion()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RegionLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRegionLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSpaceRegion(p, client);
					if((p = GetSpaceRegion()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FrequencyPartition")) == 0)
	{
		// FrequencyPartition is simple element PartitionType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFrequencyPartition()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PartitionType")))) != empty)
			{
				// Is type allowed
				Mp7JrsPartitionPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFrequencyPartition(p, client);
					if((p = GetFrequencyPartition()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Filtering")) == 0)
	{
		// Filtering is simple element FilteringType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFiltering()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FilteringType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFilteringPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFiltering(p, client);
					if((p = GetFiltering()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DownSamplingFactor")) == 0)
	{
		// DownSamplingFactor is simple element SignalPlaneSampleType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetDownSamplingFactor()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalPlaneSampleType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPlaneSamplePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetDownSamplingFactor(p, client);
					if((p = GetDownSamplingFactor()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for SpaceFrequencyViewType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "SpaceFrequencyViewType");
	}		
	
	return result;
}

XMLCh * Mp7JrsSpaceFrequencyViewType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSpaceFrequencyViewType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSpaceFrequencyViewType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SpaceFrequencyViewType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_order_Exist)
	{
	// Enumeration
	if(m_order != Mp7JrsSpaceFrequencyViewType_order_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsSpaceFrequencyViewType_order_LocalType::ToText(m_order);
		element->setAttributeNS(X(""), X("order"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_SpacePartition != Mp7JrsPartitionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpacePartition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpacePartition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpacePartition->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_SpaceRegion != Mp7JrsRegionLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_SpaceRegion->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("SpaceRegion"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_SpaceRegion->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FrequencyPartition != Mp7JrsPartitionPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FrequencyPartition->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FrequencyPartition"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FrequencyPartition->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_Filtering != Mp7JrsFilteringPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Filtering->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Filtering"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Filtering->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_DownSamplingFactor != Mp7JrsSignalPlaneSamplePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_DownSamplingFactor->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("DownSamplingFactor"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_DownSamplingFactor->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSpaceFrequencyViewType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("order")))
	{
        this->Setorder(Mp7JrsSpaceFrequencyViewType_order_LocalType::Parse(parent->getAttributeNS(X(""), X("order"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("order")))
	{
        this->Setorder(Mp7JrsSpaceFrequencyViewType_order_LocalType::Parse(parent->getAttribute(X("order"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsViewType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpacePartition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpacePartition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePartitionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSpacePartition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("SpaceRegion"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("SpaceRegion")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRegionLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSpaceRegion(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FrequencyPartition"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FrequencyPartition")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePartitionType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFrequencyPartition(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Filtering"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Filtering")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFilteringType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFiltering(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("DownSamplingFactor"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("DownSamplingFactor")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalPlaneSampleType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetDownSamplingFactor(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpaceFrequencyViewType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("SpacePartition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePartitionType;
    }
    this->SetSpacePartition(child);
  }
  if (XMLString::compareString(elementname, X("SpaceRegion")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRegionLocatorType;
    }
    this->SetSpaceRegion(child);
  }
  if (XMLString::compareString(elementname, X("FrequencyPartition")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePartitionType;
    }
    this->SetFrequencyPartition(child);
  }
  if (XMLString::compareString(elementname, X("Filtering")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFilteringType;
    }
    this->SetFiltering(child);
  }
  if (XMLString::compareString(elementname, X("DownSamplingFactor")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalPlaneSampleType;
    }
    this->SetDownSamplingFactor(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpaceFrequencyViewType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetFrequencyPartition() != Mp7JrsNodePtr())
		result.Insert(GetFrequencyPartition());
	if (GetFiltering() != Mp7JrsNodePtr())
		result.Insert(GetFiltering());
	if (GetDownSamplingFactor() != Mp7JrsNodePtr())
		result.Insert(GetDownSamplingFactor());
	if (GetTarget() != Mp7JrsNodePtr())
		result.Insert(GetTarget());
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	if (GetSpacePartition() != Mp7JrsNodePtr())
		result.Insert(GetSpacePartition());
	if (GetSpaceRegion() != Mp7JrsNodePtr())
		result.Insert(GetSpaceRegion());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpaceFrequencyViewType_ExtMethodImpl.h


