 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtImplInclude.h



#include "Mp7JrsWordLexiconType.h"
#include "Mp7JrsPhoneLexiconType.h"
#include "Mp7JrsSpokenContentHeaderType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsSpokenContentHeaderType_LocalType::IMp7JrsSpokenContentHeaderType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtPropInit.h

}

IMp7JrsSpokenContentHeaderType_LocalType::~IMp7JrsSpokenContentHeaderType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtPropCleanup.h

}

Mp7JrsSpokenContentHeaderType_LocalType::Mp7JrsSpokenContentHeaderType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpokenContentHeaderType_LocalType::~Mp7JrsSpokenContentHeaderType_LocalType()
{
	Cleanup();
}

void Mp7JrsSpokenContentHeaderType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_WordLexicon = Mp7JrsWordLexiconPtr(); // Class
	m_PhoneLexicon = Mp7JrsPhoneLexiconPtr(); // Class


// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtMyPropInit.h

}

void Mp7JrsSpokenContentHeaderType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_WordLexicon);
	// Mp7JrsFactory::DeleteObject(m_PhoneLexicon);
}

void Mp7JrsSpokenContentHeaderType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpokenContentHeaderType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in ISpokenContentHeaderType_LocalType
    const Mp7JrsPtr< Mp7JrsSpokenContentHeaderType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_WordLexicon);
		this->SetWordLexicon(Mp7JrsFactory::CloneObject(tmp->GetWordLexicon()));
		// Mp7JrsFactory::DeleteObject(m_PhoneLexicon);
		this->SetPhoneLexicon(Mp7JrsFactory::CloneObject(tmp->GetPhoneLexicon()));
}

Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsWordLexiconPtr Mp7JrsSpokenContentHeaderType_LocalType::GetWordLexicon() const
{
	return m_WordLexicon;
}

Mp7JrsPhoneLexiconPtr Mp7JrsSpokenContentHeaderType_LocalType::GetPhoneLexicon() const
{
	return m_PhoneLexicon;
}

// implementing setter for choice 
/* element */
void Mp7JrsSpokenContentHeaderType_LocalType::SetWordLexicon(const Mp7JrsWordLexiconPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType_LocalType::SetWordLexicon().");
	}
	if (m_WordLexicon != item)
	{
		// Mp7JrsFactory::DeleteObject(m_WordLexicon);
		m_WordLexicon = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_WordLexicon) m_WordLexicon->SetParent(m_myself.getPointer());
		if (m_WordLexicon && m_WordLexicon->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:WordLexiconType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_WordLexicon->UseTypeAttribute = true;
		}
		if(m_WordLexicon != Mp7JrsWordLexiconPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_WordLexicon->SetContentName(XMLString::transcode("WordLexicon"));
		}
	// Mp7JrsFactory::DeleteObject(m_PhoneLexicon);
	m_PhoneLexicon = Mp7JrsPhoneLexiconPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsSpokenContentHeaderType_LocalType::SetPhoneLexicon(const Mp7JrsPhoneLexiconPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentHeaderType_LocalType::SetPhoneLexicon().");
	}
	if (m_PhoneLexicon != item)
	{
		// Mp7JrsFactory::DeleteObject(m_PhoneLexicon);
		m_PhoneLexicon = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_PhoneLexicon) m_PhoneLexicon->SetParent(m_myself.getPointer());
		if (m_PhoneLexicon && m_PhoneLexicon->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:PhoneLexiconType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_PhoneLexicon->UseTypeAttribute = true;
		}
		if(m_PhoneLexicon != Mp7JrsPhoneLexiconPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_PhoneLexicon->SetContentName(XMLString::transcode("PhoneLexicon"));
		}
	// Mp7JrsFactory::DeleteObject(m_WordLexicon);
	m_WordLexicon = Mp7JrsWordLexiconPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: SpokenContentHeaderType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsSpokenContentHeaderType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsSpokenContentHeaderType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSpokenContentHeaderType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsSpokenContentHeaderType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_WordLexicon != Mp7JrsWordLexiconPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_WordLexicon->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("WordLexicon"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_WordLexicon->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_PhoneLexicon != Mp7JrsPhoneLexiconPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_PhoneLexicon->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("PhoneLexicon"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_PhoneLexicon->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsSpokenContentHeaderType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("WordLexicon"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("WordLexicon")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateWordLexiconType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetWordLexicon(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("PhoneLexicon"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("PhoneLexicon")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreatePhoneLexiconType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetPhoneLexicon(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpokenContentHeaderType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("WordLexicon")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateWordLexiconType;
    }
    this->SetWordLexicon(child);
  }
  if (XMLString::compareString(elementname, X("PhoneLexicon")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreatePhoneLexiconType;
    }
    this->SetPhoneLexicon(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpokenContentHeaderType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetWordLexicon() != Mp7JrsNodePtr())
		result.Insert(GetWordLexicon());
	if (GetPhoneLexicon() != Mp7JrsNodePtr())
		result.Insert(GetPhoneLexicon());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpokenContentHeaderType_LocalType_ExtMethodImpl.h


