 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtImplInclude.h



#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsnonNegativeReal.h"
#include "Mp7Jrsunsigned16.h"
#include "Mp7JrsSpokenContentLinkType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsSpokenContentLinkType::IMp7JrsSpokenContentLinkType()
{

// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtPropInit.h

}

IMp7JrsSpokenContentLinkType::~IMp7JrsSpokenContentLinkType()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtPropCleanup.h

}

Mp7JrsSpokenContentLinkType::Mp7JrsSpokenContentLinkType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsSpokenContentLinkType::~Mp7JrsSpokenContentLinkType()
{
	Cleanup();
}

void Mp7JrsSpokenContentLinkType::Init()
{

	// Init attributes
	m_probability = CreatezeroToOneType; // Create a valid class
	m_probability->SetContent(0.0); // Use Value initialiser (xxx2)
	m_probability_Exist = false;
	m_acousticScore = CreatenonNegativeReal; // Create a valid class
	m_acousticScore->SetContent(0.0); // Use Value initialiser (xxx2)
	m_acousticScore_Exist = false;
	m_nodeOffset = Createunsigned16; // Create a valid class
	m_nodeOffset->SetContent(0); // Use Value initialiser (xxx2)
	m_nodeOffset_Exist = false;



// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMyPropInit.h

}

void Mp7JrsSpokenContentLinkType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_probability);
	// Mp7JrsFactory::DeleteObject(m_acousticScore);
	// Mp7JrsFactory::DeleteObject(m_nodeOffset);
}

void Mp7JrsSpokenContentLinkType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use SpokenContentLinkTypePtr, since we
	// might need GetBase(), which isn't defined in ISpokenContentLinkType
    const Mp7JrsPtr< Mp7JrsSpokenContentLinkType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_probability);
	if (tmp->Existprobability())
	{
		this->Setprobability(Mp7JrsFactory::CloneObject(tmp->Getprobability()));
	}
	else
	{
		Invalidateprobability();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_acousticScore);
	if (tmp->ExistacousticScore())
	{
		this->SetacousticScore(Mp7JrsFactory::CloneObject(tmp->GetacousticScore()));
	}
	else
	{
		InvalidateacousticScore();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_nodeOffset);
	if (tmp->ExistnodeOffset())
	{
		this->SetnodeOffset(Mp7JrsFactory::CloneObject(tmp->GetnodeOffset()));
	}
	else
	{
		InvalidatenodeOffset();
	}
	}
}

Mp7JrsNodePtr Mp7JrsSpokenContentLinkType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsSpokenContentLinkType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrszeroToOnePtr Mp7JrsSpokenContentLinkType::Getprobability() const
{
	if (this->Existprobability()) {
		return m_probability;
	} else {
		return m_probability_Default;
	}
}

bool Mp7JrsSpokenContentLinkType::Existprobability() const
{
	return m_probability_Exist;
}
void Mp7JrsSpokenContentLinkType::Setprobability(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentLinkType::Setprobability().");
	}
    m_probability = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_probability) m_probability->SetParent(m_myself.getPointer());
	m_probability_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentLinkType::Invalidateprobability()
{
	m_probability_Exist = false;
}
Mp7JrsnonNegativeRealPtr Mp7JrsSpokenContentLinkType::GetacousticScore() const
{
	return m_acousticScore;
}

bool Mp7JrsSpokenContentLinkType::ExistacousticScore() const
{
	return m_acousticScore_Exist;
}
void Mp7JrsSpokenContentLinkType::SetacousticScore(const Mp7JrsnonNegativeRealPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentLinkType::SetacousticScore().");
	}
    m_acousticScore = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_acousticScore) m_acousticScore->SetParent(m_myself.getPointer());
	m_acousticScore_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentLinkType::InvalidateacousticScore()
{
	m_acousticScore_Exist = false;
}
Mp7Jrsunsigned16Ptr Mp7JrsSpokenContentLinkType::GetnodeOffset() const
{
	if (this->ExistnodeOffset()) {
		return m_nodeOffset;
	} else {
		return m_nodeOffset_Default;
	}
}

bool Mp7JrsSpokenContentLinkType::ExistnodeOffset() const
{
	return m_nodeOffset_Exist;
}
void Mp7JrsSpokenContentLinkType::SetnodeOffset(const Mp7Jrsunsigned16Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsSpokenContentLinkType::SetnodeOffset().");
	}
    m_nodeOffset = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_nodeOffset) m_nodeOffset->SetParent(m_myself.getPointer());
	m_nodeOffset_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsSpokenContentLinkType::InvalidatenodeOffset()
{
	m_nodeOffset_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsSpokenContentLinkType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	// Only rudimentary expressions allowed for this type with no child elements
	return result;
}	

XMLCh * Mp7JrsSpokenContentLinkType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsSpokenContentLinkType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}


void Mp7JrsSpokenContentLinkType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("SpokenContentLinkType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_probability_Exist)
	{
	// Class
	if (m_probability != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_probability->ToText();
		element->setAttributeNS(X(""), X("probability"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_acousticScore_Exist)
	{
	// Class
	if (m_acousticScore != Mp7JrsnonNegativeRealPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_acousticScore->ToText();
		element->setAttributeNS(X(""), X("acousticScore"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_nodeOffset_Exist)
	{
	// Class
	if (m_nodeOffset != Mp7Jrsunsigned16Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_nodeOffset->ToText();
		element->setAttributeNS(X(""), X("nodeOffset"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsSpokenContentLinkType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("probability")))
	{
        // Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttributeNS(X(""), X("probability")));
        this->Setprobability(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("probability")))
	{
		// Deserialize class type
        Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
        tmp->Parse(parent->getAttribute(X("probability")));
        this->Setprobability(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("acousticScore")))
	{
        // Deserialize class type
		Mp7JrsnonNegativeRealPtr tmp = CreatenonNegativeReal;
		tmp->Parse(parent->getAttributeNS(X(""), X("acousticScore")));
        this->SetacousticScore(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("acousticScore")))
	{
		// Deserialize class type
        Mp7JrsnonNegativeRealPtr tmp = CreatenonNegativeReal;
        tmp->Parse(parent->getAttribute(X("acousticScore")));
        this->SetacousticScore(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("nodeOffset")))
	{
        // Deserialize class type
		Mp7Jrsunsigned16Ptr tmp = Createunsigned16;
		tmp->Parse(parent->getAttributeNS(X(""), X("nodeOffset")));
        this->SetnodeOffset(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("nodeOffset")))
	{
		// Deserialize class type
        Mp7Jrsunsigned16Ptr tmp = Createunsigned16;
        tmp->Parse(parent->getAttribute(X("nodeOffset")));
        this->SetnodeOffset(tmp);
        * current = parent;
    }

// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsSpokenContentLinkType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsSpokenContentLinkType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsSpokenContentLinkType_ExtMethodImpl.h


