 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtImplInclude.h



#include "Mp7JrszeroToOneType.h"
#include "Mp7JrsTextAnnotationType_CollectionType.h"
#include "Mp7JrsTextAnnotationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsTextAnnotationType_LocalType.h" // Choice collection FreeTextAnnotation
#include "Mp7JrsTextualType.h" // Choice collection element FreeTextAnnotation
#include "Mp7JrsStructuredAnnotationType.h" // Choice collection element StructuredAnnotation
#include "Mp7JrsDependencyStructureType.h" // Choice collection element DependencyStructure
#include "Mp7JrsKeywordAnnotationType.h" // Choice collection element KeywordAnnotation

#include <assert.h>
IMp7JrsTextAnnotationType::IMp7JrsTextAnnotationType()
{

// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtPropInit.h

}

IMp7JrsTextAnnotationType::~IMp7JrsTextAnnotationType()
{
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtPropCleanup.h

}

Mp7JrsTextAnnotationType::Mp7JrsTextAnnotationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextAnnotationType::~Mp7JrsTextAnnotationType()
{
	Cleanup();
}

void Mp7JrsTextAnnotationType::Init()
{

	// Init attributes
	m_relevance = CreatezeroToOneType; // Create a valid class
	m_relevance->SetContent(0.0); // Use Value initialiser (xxx2)
	m_relevance_Exist = false;
	m_confidence = CreatezeroToOneType; // Create a valid class
	m_confidence->SetContent(0.0); // Use Value initialiser (xxx2)
	m_confidence_Exist = false;
	m_lang = NULL; // String
	m_lang_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_TextAnnotationType_LocalType = Mp7JrsTextAnnotationType_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtMyPropInit.h

}

void Mp7JrsTextAnnotationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_relevance);
	// Mp7JrsFactory::DeleteObject(m_confidence);
	XMLString::release(&m_lang);
	// Mp7JrsFactory::DeleteObject(m_TextAnnotationType_LocalType);
}

void Mp7JrsTextAnnotationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextAnnotationTypePtr, since we
	// might need GetBase(), which isn't defined in ITextAnnotationType
    const Mp7JrsPtr< Mp7JrsTextAnnotationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	{
	// Mp7JrsFactory::DeleteObject(m_relevance);
	if (tmp->Existrelevance())
	{
		this->Setrelevance(Mp7JrsFactory::CloneObject(tmp->Getrelevance()));
	}
	else
	{
		Invalidaterelevance();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_confidence);
	if (tmp->Existconfidence())
	{
		this->Setconfidence(Mp7JrsFactory::CloneObject(tmp->Getconfidence()));
	}
	else
	{
		Invalidateconfidence();
	}
	}
	{
	XMLString::release(&m_lang);
	if (tmp->Existlang())
	{
		this->Setlang(XMLString::replicate(tmp->Getlang()));
	}
	else
	{
		Invalidatelang();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_TextAnnotationType_LocalType);
		this->SetTextAnnotationType_LocalType(Mp7JrsFactory::CloneObject(tmp->GetTextAnnotationType_LocalType()));
}

Mp7JrsNodePtr Mp7JrsTextAnnotationType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTextAnnotationType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrszeroToOnePtr Mp7JrsTextAnnotationType::Getrelevance() const
{
	return m_relevance;
}

bool Mp7JrsTextAnnotationType::Existrelevance() const
{
	return m_relevance_Exist;
}
void Mp7JrsTextAnnotationType::Setrelevance(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType::Setrelevance().");
	}
    m_relevance = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_relevance) m_relevance->SetParent(m_myself.getPointer());
	m_relevance_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextAnnotationType::Invalidaterelevance()
{
	m_relevance_Exist = false;
}
Mp7JrszeroToOnePtr Mp7JrsTextAnnotationType::Getconfidence() const
{
	return m_confidence;
}

bool Mp7JrsTextAnnotationType::Existconfidence() const
{
	return m_confidence_Exist;
}
void Mp7JrsTextAnnotationType::Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType::Setconfidence().");
	}
    m_confidence = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_confidence) m_confidence->SetParent(m_myself.getPointer());
	m_confidence_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextAnnotationType::Invalidateconfidence()
{
	m_confidence_Exist = false;
}
XMLCh * Mp7JrsTextAnnotationType::Getlang() const
{
	return m_lang;
}

bool Mp7JrsTextAnnotationType::Existlang() const
{
	return m_lang_Exist;
}
void Mp7JrsTextAnnotationType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType::Setlang().");
	}
    m_lang = item;
	m_lang_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextAnnotationType::Invalidatelang()
{
	m_lang_Exist = false;
}
Mp7JrsTextAnnotationType_CollectionPtr Mp7JrsTextAnnotationType::GetTextAnnotationType_LocalType() const
{
	return m_TextAnnotationType_LocalType;
}

void Mp7JrsTextAnnotationType::SetTextAnnotationType_LocalType(const Mp7JrsTextAnnotationType_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextAnnotationType::SetTextAnnotationType_LocalType().");
	}
	if (m_TextAnnotationType_LocalType != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextAnnotationType_LocalType);
		m_TextAnnotationType_LocalType = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextAnnotationType_LocalType) m_TextAnnotationType_LocalType->SetParent(m_myself.getPointer());
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsTextAnnotationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("FreeTextAnnotation")) == 0)
	{
		// FreeTextAnnotation is contained in itemtype TextAnnotationType_LocalType
		// in choice collection TextAnnotationType_CollectionType
		
		context->Found = true;
		Mp7JrsTextAnnotationType_CollectionPtr coll = GetTextAnnotationType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTextAnnotationType_CollectionType;
				SetTextAnnotationType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetFreeTextAnnotation()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsTextAnnotationType_LocalPtr item = CreateTextAnnotationType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextualType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTextualPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->SetFreeTextAnnotation(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetFreeTextAnnotation()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("StructuredAnnotation")) == 0)
	{
		// StructuredAnnotation is contained in itemtype TextAnnotationType_LocalType
		// in choice collection TextAnnotationType_CollectionType
		
		context->Found = true;
		Mp7JrsTextAnnotationType_CollectionPtr coll = GetTextAnnotationType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTextAnnotationType_CollectionType;
				SetTextAnnotationType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetStructuredAnnotation()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsTextAnnotationType_LocalPtr item = CreateTextAnnotationType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:StructuredAnnotationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsStructuredAnnotationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->SetStructuredAnnotation(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetStructuredAnnotation()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("DependencyStructure")) == 0)
	{
		// DependencyStructure is contained in itemtype TextAnnotationType_LocalType
		// in choice collection TextAnnotationType_CollectionType
		
		context->Found = true;
		Mp7JrsTextAnnotationType_CollectionPtr coll = GetTextAnnotationType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTextAnnotationType_CollectionType;
				SetTextAnnotationType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetDependencyStructure()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsTextAnnotationType_LocalPtr item = CreateTextAnnotationType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:DependencyStructureType")))) != empty)
			{
				// Is type allowed
				Mp7JrsDependencyStructurePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->SetDependencyStructure(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetDependencyStructure()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("KeywordAnnotation")) == 0)
	{
		// KeywordAnnotation is contained in itemtype TextAnnotationType_LocalType
		// in choice collection TextAnnotationType_CollectionType
		
		context->Found = true;
		Mp7JrsTextAnnotationType_CollectionPtr coll = GetTextAnnotationType_LocalType();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateTextAnnotationType_CollectionType;
				SetTextAnnotationType_LocalType(coll, client);
			}
			else
			{
				return result; // No collection, silently return emtpy enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetKeywordAnnotation()) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				++i;
			}
		}
		if(Mp7JrsXPathParseContext::PostProcessChoiceCollection(context, i, result))
		{
			if(i >= 0)
			{
				Mp7JrsTextAnnotationType_LocalPtr item = CreateTextAnnotationType_LocalType;
				coll->insertElementAt(item, i, client);
			}
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:KeywordAnnotationType")))) != empty)
			{
				// Is type allowed
				Mp7JrsKeywordAnnotationPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->SetKeywordAnnotation(p, client);
					// Maybe some other restrictions are waiting...				
					if((p = ((Mp7JrsTextAnnotationType_LocalPtr)coll->elementAt(i))->GetKeywordAnnotation()) != empty)
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;

	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TextAnnotationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TextAnnotationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsTextAnnotationType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTextAnnotationType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTextAnnotationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TextAnnotationType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_relevance_Exist)
	{
	// Class
	if (m_relevance != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_relevance->ToText();
		element->setAttributeNS(X(""), X("relevance"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_confidence_Exist)
	{
	// Class
	if (m_confidence != Mp7JrszeroToOnePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_confidence->ToText();
		element->setAttributeNS(X(""), X("confidence"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_lang_Exist)
	{
	// String
	if(m_lang != (XMLCh *)NULL)
	{
		element->setAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("xml:lang"), m_lang);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_TextAnnotationType_LocalType != Mp7JrsTextAnnotationType_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_TextAnnotationType_LocalType->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsTextAnnotationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("relevance")))
	{
        // Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttributeNS(X(""), X("relevance")));
        this->Setrelevance(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("relevance")))
	{
		// Deserialize class type
        Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
        tmp->Parse(parent->getAttribute(X("relevance")));
        this->Setrelevance(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("confidence")))
	{
        // Deserialize class type
		Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
		tmp->Parse(parent->getAttributeNS(X(""), X("confidence")));
        this->Setconfidence(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("confidence")))
	{
		// Deserialize class type
        Mp7JrszeroToOnePtr tmp = CreatezeroToOneType;
        tmp->Parse(parent->getAttribute(X("confidence")));
        this->Setconfidence(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttributeNS(X("http://www.w3.org/XML/1998/namespace"), X("lang"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("lang")))
	{
        // Deserialize string type
        this->Setlang(Mp7JrsConvert::TextToString(parent->getAttribute(X("lang"))));
        * current = parent;
    }

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialise Choice Collection
        if((parent != NULL) && (
            // urn:mpeg:mpeg7:schema:2004:FreeTextAnnotation
            Mp7JrsUtil::HasNodeName(parent, X("FreeTextAnnotation"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:FreeTextAnnotation")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:StructuredAnnotation
            Mp7JrsUtil::HasNodeName(parent, X("StructuredAnnotation"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:StructuredAnnotation")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:DependencyStructure
            Mp7JrsUtil::HasNodeName(parent, X("DependencyStructure"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:DependencyStructure")) == 0)
                ||
            // urn:mpeg:mpeg7:schema:2004:KeywordAnnotation
            Mp7JrsUtil::HasNodeName(parent, X("KeywordAnnotation"), X("urn:mpeg:mpeg7:schema:2004"))
//OLD            (XMLString::compareString(parent->getNodeName(), X("urn:mpeg:mpeg7:schema:2004:KeywordAnnotation")) == 0)
                ))
        {
            // Deserialize factory type
            Mp7JrsTextAnnotationType_CollectionPtr tmp = CreateTextAnnotationType_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetTextAnnotationType_LocalType(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextAnnotationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  // Choice Collection
  if(
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("FreeTextAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("FreeTextAnnotation")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("StructuredAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("StructuredAnnotation")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("DependencyStructure"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("DependencyStructure")) == 0)
    ||
// TODO better use this    
//  (Mp7JrsUtil::HasNodeName(parent, X("KeywordAnnotation"), X("urn:mpeg:mpeg7:schema:2004")))

// TODO Remove the old way
    (XMLString::compareString(elementname, X("KeywordAnnotation")) == 0)
    )
  {
    Mp7JrsTextAnnotationType_CollectionPtr tmp = CreateTextAnnotationType_CollectionType;
    this->SetTextAnnotationType_LocalType(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextAnnotationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTextAnnotationType_LocalType() != Mp7JrsNodePtr())
		result.Insert(GetTextAnnotationType_LocalType());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextAnnotationType_ExtMethodImpl.h


