 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtImplInclude.h



#include "Mp7JrsTextSegmentType.h"
#include "Mp7JrsReferenceType.h"
#include "Mp7JrsTextDecompositionType_LocalType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

#include <assert.h>
IMp7JrsTextDecompositionType_LocalType::IMp7JrsTextDecompositionType_LocalType()
{

// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtPropInit.h

}

IMp7JrsTextDecompositionType_LocalType::~IMp7JrsTextDecompositionType_LocalType()
{
// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtPropCleanup.h

}

Mp7JrsTextDecompositionType_LocalType::Mp7JrsTextDecompositionType_LocalType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextDecompositionType_LocalType::~Mp7JrsTextDecompositionType_LocalType()
{
	Cleanup();
}

void Mp7JrsTextDecompositionType_LocalType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_TextSegment = Mp7JrsTextSegmentPtr(); // Class
	m_TextSegmentRef = Mp7JrsReferencePtr(); // Class


// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtMyPropInit.h

}

void Mp7JrsTextDecompositionType_LocalType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_TextSegment);
	// Mp7JrsFactory::DeleteObject(m_TextSegmentRef);
}

void Mp7JrsTextDecompositionType_LocalType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextDecompositionType_LocalTypePtr, since we
	// might need GetBase(), which isn't defined in ITextDecompositionType_LocalType
    const Mp7JrsPtr< Mp7JrsTextDecompositionType_LocalType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
		// Mp7JrsFactory::DeleteObject(m_TextSegment);
		this->SetTextSegment(Mp7JrsFactory::CloneObject(tmp->GetTextSegment()));
		// Mp7JrsFactory::DeleteObject(m_TextSegmentRef);
		this->SetTextSegmentRef(Mp7JrsFactory::CloneObject(tmp->GetTextSegmentRef()));
}

Mp7JrsNodePtr Mp7JrsTextDecompositionType_LocalType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTextDecompositionType_LocalType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsTextSegmentPtr Mp7JrsTextDecompositionType_LocalType::GetTextSegment() const
{
	return m_TextSegment;
}

Mp7JrsReferencePtr Mp7JrsTextDecompositionType_LocalType::GetTextSegmentRef() const
{
	return m_TextSegmentRef;
}

// implementing setter for choice 
/* element */
void Mp7JrsTextDecompositionType_LocalType::SetTextSegment(const Mp7JrsTextSegmentPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextDecompositionType_LocalType::SetTextSegment().");
	}
	if (m_TextSegment != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextSegment);
		m_TextSegment = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextSegment) m_TextSegment->SetParent(m_myself.getPointer());
		if (m_TextSegment && m_TextSegment->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TextSegmentType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TextSegment->UseTypeAttribute = true;
		}
		if(m_TextSegment != Mp7JrsTextSegmentPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TextSegment->SetContentName(XMLString::transcode("TextSegment"));
		}
	// Mp7JrsFactory::DeleteObject(m_TextSegmentRef);
	m_TextSegmentRef = Mp7JrsReferencePtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

/* element */
void Mp7JrsTextDecompositionType_LocalType::SetTextSegmentRef(const Mp7JrsReferencePtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextDecompositionType_LocalType::SetTextSegmentRef().");
	}
	if (m_TextSegmentRef != item)
	{
		// Mp7JrsFactory::DeleteObject(m_TextSegmentRef);
		m_TextSegmentRef = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TextSegmentRef) m_TextSegmentRef->SetParent(m_myself.getPointer());
		if (m_TextSegmentRef && m_TextSegmentRef->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ReferenceType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TextSegmentRef->UseTypeAttribute = true;
		}
		if(m_TextSegmentRef != Mp7JrsReferencePtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TextSegmentRef->SetContentName(XMLString::transcode("TextSegmentRef"));
		}
	// Mp7JrsFactory::DeleteObject(m_TextSegment);
	m_TextSegment = Mp7JrsTextSegmentPtr();
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

// Type: TextDecompositionType_LocalType: Only types with an element name can have GetOrCreate()
// Mp7JrsNodeEnum Mp7JrsTextDecompositionType_LocalType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
//

XMLCh * Mp7JrsTextDecompositionType_LocalType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTextDecompositionType_LocalType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTextDecompositionType_LocalType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// Serialize Choice of Sequence collection item
	element = parent;


	// Element serialization:
	// Class
	
	if (m_TextSegment != Mp7JrsTextSegmentPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TextSegment->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TextSegment"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TextSegment->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TextSegmentRef != Mp7JrsReferencePtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TextSegmentRef->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TextSegmentRef"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TextSegmentRef->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsTextDecompositionType_LocalType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	do // Deserialize choice just one time!
    {
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TextSegment"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TextSegment")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTextSegmentType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTextSegment(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TextSegmentRef"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TextSegmentRef")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateReferenceType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTextSegmentRef(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
            continue;       // For choices
        }
	} while(false);
// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextDecompositionType_LocalType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("TextSegment")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTextSegmentType;
    }
    this->SetTextSegment(child);
  }
  if (XMLString::compareString(elementname, X("TextSegmentRef")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateReferenceType;
    }
    this->SetTextSegmentRef(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextDecompositionType_LocalType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTextSegment() != Mp7JrsNodePtr())
		result.Insert(GetTextSegment());
	if (GetTextSegmentRef() != Mp7JrsNodePtr())
		result.Insert(GetTextSegmentRef());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextDecompositionType_LocalType_ExtMethodImpl.h


