 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTextualType_ExtImplInclude.h



#include "Mp7JrsTextualBaseType.h"
#include "Mp7JrsTextualBaseType_phoneticTranscription_CollectionType.h"
#include "Mp7JrsTextualType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsTextualType::IMp7JrsTextualType()
{

// no includefile for extension defined 
// file Mp7JrsTextualType_ExtPropInit.h

}

IMp7JrsTextualType::~IMp7JrsTextualType()
{
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtPropCleanup.h

}

Mp7JrsTextualType::Mp7JrsTextualType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTextualType::~Mp7JrsTextualType()
{
	Cleanup();
}

void Mp7JrsTextualType::Init()
{
	// Init base
	m_Base = CreateTextualBaseType;
	m_Base->m_myself = m_myself.getPointer();
	




// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMyPropInit.h

}

void Mp7JrsTextualType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
}

void Mp7JrsTextualType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TextualTypePtr, since we
	// might need GetBase(), which isn't defined in ITextualType
    const Mp7JrsPtr< Mp7JrsTextualType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsTextualBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsTextualType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
}

Mp7JrsNodePtr Mp7JrsTextualType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsTextualType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsTextualBaseType > Mp7JrsTextualType::GetBase() const
{
	return m_Base;
}

void Mp7JrsTextualType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualType::SetContent().");
	}
	GetBase()->SetContent(item);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsTextualType::GetContent() const
{
	return GetBase()->GetContent();
}
XMLCh * Mp7JrsTextualType::Getlang() const
{
	return GetBase()->Getlang();
}

bool Mp7JrsTextualType::Existlang() const
{
	return GetBase()->Existlang();
}
void Mp7JrsTextualType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualType::Setlang().");
	}
	GetBase()->Setlang(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualType::Invalidatelang()
{
	GetBase()->Invalidatelang();
}
Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr Mp7JrsTextualType::GetphoneticTranscription() const
{
	return GetBase()->GetphoneticTranscription();
}

bool Mp7JrsTextualType::ExistphoneticTranscription() const
{
	return GetBase()->ExistphoneticTranscription();
}
void Mp7JrsTextualType::SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualType::SetphoneticTranscription().");
	}
	GetBase()->SetphoneticTranscription(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualType::InvalidatephoneticTranscription()
{
	GetBase()->InvalidatephoneticTranscription();
}
Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsTextualType::GetphoneticAlphabet() const
{
	return GetBase()->GetphoneticAlphabet();
}

bool Mp7JrsTextualType::ExistphoneticAlphabet() const
{
	return GetBase()->ExistphoneticAlphabet();
}
void Mp7JrsTextualType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTextualType::SetphoneticAlphabet().");
	}
	GetBase()->SetphoneticAlphabet(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTextualType::InvalidatephoneticAlphabet()
{
	GetBase()->InvalidatephoneticAlphabet();
}

Mp7JrsNodeEnum Mp7JrsTextualType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for TextualType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "TextualType");
	}
	
	return result;
}

XMLCh * Mp7JrsTextualType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsTextualType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}

bool Mp7JrsTextualType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsTextualType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsTextualType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TextualType"));
	// Element serialization:

}

bool Mp7JrsTextualType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsTextualBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsTextualType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTextualType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTextualType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTextualType_ExtMethodImpl.h


