 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtImplInclude.h



#include "Mp7JrsImageLocatorType.h"
#include "Mp7JrsTemporalSegmentLocatorType.h"
#include "Mp7JrsTitleMediaType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsTitleMediaType::IMp7JrsTitleMediaType()
{

// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtPropInit.h

}

IMp7JrsTitleMediaType::~IMp7JrsTitleMediaType()
{
// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtPropCleanup.h

}

Mp7JrsTitleMediaType::Mp7JrsTitleMediaType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTitleMediaType::~Mp7JrsTitleMediaType()
{
	Cleanup();
}

void Mp7JrsTitleMediaType::Init()
{


	// Init elements (element, union sequence choice all any)
	
	m_TitleImage = Mp7JrsImageLocatorPtr(); // Class
	m_TitleImage_Exist = false;
	m_TitleVideo = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_TitleVideo_Exist = false;
	m_TitleAudio = Mp7JrsTemporalSegmentLocatorPtr(); // Class
	m_TitleAudio_Exist = false;


// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtMyPropInit.h

}

void Mp7JrsTitleMediaType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_TitleImage);
	// Mp7JrsFactory::DeleteObject(m_TitleVideo);
	// Mp7JrsFactory::DeleteObject(m_TitleAudio);
}

void Mp7JrsTitleMediaType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TitleMediaTypePtr, since we
	// might need GetBase(), which isn't defined in ITitleMediaType
    const Mp7JrsPtr< Mp7JrsTitleMediaType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	if (tmp->IsValidTitleImage())
	{
		// Mp7JrsFactory::DeleteObject(m_TitleImage);
		this->SetTitleImage(Mp7JrsFactory::CloneObject(tmp->GetTitleImage()));
	}
	else
	{
		InvalidateTitleImage();
	}
	if (tmp->IsValidTitleVideo())
	{
		// Mp7JrsFactory::DeleteObject(m_TitleVideo);
		this->SetTitleVideo(Mp7JrsFactory::CloneObject(tmp->GetTitleVideo()));
	}
	else
	{
		InvalidateTitleVideo();
	}
	if (tmp->IsValidTitleAudio())
	{
		// Mp7JrsFactory::DeleteObject(m_TitleAudio);
		this->SetTitleAudio(Mp7JrsFactory::CloneObject(tmp->GetTitleAudio()));
	}
	else
	{
		InvalidateTitleAudio();
	}
}

Mp7JrsNodePtr Mp7JrsTitleMediaType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsTitleMediaType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


Mp7JrsImageLocatorPtr Mp7JrsTitleMediaType::GetTitleImage() const
{
	return m_TitleImage;
}

// Element is optional
bool Mp7JrsTitleMediaType::IsValidTitleImage() const
{
	return m_TitleImage_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsTitleMediaType::GetTitleVideo() const
{
	return m_TitleVideo;
}

// Element is optional
bool Mp7JrsTitleMediaType::IsValidTitleVideo() const
{
	return m_TitleVideo_Exist;
}

Mp7JrsTemporalSegmentLocatorPtr Mp7JrsTitleMediaType::GetTitleAudio() const
{
	return m_TitleAudio;
}

// Element is optional
bool Mp7JrsTitleMediaType::IsValidTitleAudio() const
{
	return m_TitleAudio_Exist;
}

void Mp7JrsTitleMediaType::SetTitleImage(const Mp7JrsImageLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleMediaType::SetTitleImage().");
	}
	if (m_TitleImage != item || m_TitleImage_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TitleImage);
		m_TitleImage = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TitleImage) m_TitleImage->SetParent(m_myself.getPointer());
		if (m_TitleImage && m_TitleImage->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TitleImage->UseTypeAttribute = true;
		}
		m_TitleImage_Exist = true;
		if(m_TitleImage != Mp7JrsImageLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TitleImage->SetContentName(XMLString::transcode("TitleImage"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleMediaType::InvalidateTitleImage()
{
	m_TitleImage_Exist = false;
}
void Mp7JrsTitleMediaType::SetTitleVideo(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleMediaType::SetTitleVideo().");
	}
	if (m_TitleVideo != item || m_TitleVideo_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TitleVideo);
		m_TitleVideo = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TitleVideo) m_TitleVideo->SetParent(m_myself.getPointer());
		if (m_TitleVideo && m_TitleVideo->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TitleVideo->UseTypeAttribute = true;
		}
		m_TitleVideo_Exist = true;
		if(m_TitleVideo != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TitleVideo->SetContentName(XMLString::transcode("TitleVideo"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleMediaType::InvalidateTitleVideo()
{
	m_TitleVideo_Exist = false;
}
void Mp7JrsTitleMediaType::SetTitleAudio(const Mp7JrsTemporalSegmentLocatorPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleMediaType::SetTitleAudio().");
	}
	if (m_TitleAudio != item || m_TitleAudio_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_TitleAudio);
		m_TitleAudio = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_TitleAudio) m_TitleAudio->SetParent(m_myself.getPointer());
		if (m_TitleAudio && m_TitleAudio->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_TitleAudio->UseTypeAttribute = true;
		}
		m_TitleAudio_Exist = true;
		if(m_TitleAudio != Mp7JrsTemporalSegmentLocatorPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_TitleAudio->SetContentName(XMLString::transcode("TitleAudio"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleMediaType::InvalidateTitleAudio()
{
	m_TitleAudio_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsTitleMediaType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("TitleImage")) == 0)
	{
		// TitleImage is simple element ImageLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTitleImage()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:ImageLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsImageLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTitleImage(p, client);
					if((p = GetTitleImage()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TitleVideo")) == 0)
	{
		// TitleVideo is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTitleVideo()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTitleVideo(p, client);
					if((p = GetTitleVideo()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("TitleAudio")) == 0)
	{
		// TitleAudio is simple element TemporalSegmentLocatorType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetTitleAudio()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TemporalSegmentLocatorType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTemporalSegmentLocatorPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetTitleAudio(p, client);
					if((p = GetTitleAudio()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for TitleMediaType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "TitleMediaType");
	}		
	
	return result;
}

XMLCh * Mp7JrsTitleMediaType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTitleMediaType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsTitleMediaType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TitleMediaType"));
	// Element serialization:
	// Class
	
	if (m_TitleImage != Mp7JrsImageLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TitleImage->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TitleImage"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TitleImage->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TitleVideo != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TitleVideo->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TitleVideo"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TitleVideo->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_TitleAudio != Mp7JrsTemporalSegmentLocatorPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_TitleAudio->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("TitleAudio"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_TitleAudio->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsTitleMediaType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	parent = Mp7JrsUtil::GetNextChild(parent);
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TitleImage"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TitleImage")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateImageLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTitleImage(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TitleVideo"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TitleVideo")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTitleVideo(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("TitleAudio"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("TitleAudio")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateTemporalSegmentLocatorType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetTitleAudio(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTitleMediaType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  if (XMLString::compareString(elementname, X("TitleImage")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateImageLocatorType;
    }
    this->SetTitleImage(child);
  }
  if (XMLString::compareString(elementname, X("TitleVideo")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetTitleVideo(child);
  }
  if (XMLString::compareString(elementname, X("TitleAudio")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateTemporalSegmentLocatorType;
    }
    this->SetTitleAudio(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTitleMediaType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetTitleImage() != Mp7JrsNodePtr())
		result.Insert(GetTitleImage());
	if (GetTitleVideo() != Mp7JrsNodePtr())
		result.Insert(GetTitleVideo());
	if (GetTitleAudio() != Mp7JrsNodePtr())
		result.Insert(GetTitleAudio());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTitleMediaType_ExtMethodImpl.h


