 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsTitleType_ExtImplInclude.h



#include "Mp7JrsTextualBaseType.h"
#include "Mp7JrsTitleType_type_LocalType2.h"
#include "Mp7JrsTextualBaseType_phoneticTranscription_CollectionType.h"
#include "Mp7JrsTitleType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsTitleType::IMp7JrsTitleType()
{

// no includefile for extension defined 
// file Mp7JrsTitleType_ExtPropInit.h

}

IMp7JrsTitleType::~IMp7JrsTitleType()
{
// no includefile for extension defined 
// file Mp7JrsTitleType_ExtPropCleanup.h

}

Mp7JrsTitleType::Mp7JrsTitleType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsTitleType::~Mp7JrsTitleType()
{
	Cleanup();
}

void Mp7JrsTitleType::Init()
{
	// Init base
	m_Base = CreateTextualBaseType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_type = Mp7JrsTitleType_type_Local2Ptr(); // Create emtpy class ptr
	m_type_Exist = false;



// no includefile for extension defined 
// file Mp7JrsTitleType_ExtMyPropInit.h

}

void Mp7JrsTitleType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsTitleType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_type);
}

void Mp7JrsTitleType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use TitleTypePtr, since we
	// might need GetBase(), which isn't defined in ITitleType
    const Mp7JrsPtr< Mp7JrsTitleType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsTextualBasePtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsTitleType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	// Mp7JrsFactory::DeleteObject(m_type);
	if (tmp->Existtype())
	{
		this->Settype(Mp7JrsFactory::CloneObject(tmp->Gettype()));
	}
	else
	{
		Invalidatetype();
	}
	}
}

Mp7JrsNodePtr Mp7JrsTitleType::GetBaseRoot()
{
	return GetBase();
}

const Mp7JrsNodePtr Mp7JrsTitleType::GetBaseRoot() const
{
	return GetBase();
}


Mp7JrsPtr< Mp7JrsTextualBaseType > Mp7JrsTitleType::GetBase() const
{
	return m_Base;
}

void Mp7JrsTitleType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleType::SetContent().");
	}
	GetBase()->SetContent(item);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsTitleType::GetContent() const
{
	return GetBase()->GetContent();
}
Mp7JrsTitleType_type_Local2Ptr Mp7JrsTitleType::Gettype() const
{
	if (this->Existtype()) {
		return m_type;
	} else {
		return m_type_Default;
	}
}

bool Mp7JrsTitleType::Existtype() const
{
	return m_type_Exist;
}
void Mp7JrsTitleType::Settype(const Mp7JrsTitleType_type_Local2Ptr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleType::Settype().");
	}
    m_type = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_type) m_type->SetParent(m_myself.getPointer());
	m_type_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleType::Invalidatetype()
{
	m_type_Exist = false;
}
XMLCh * Mp7JrsTitleType::Getlang() const
{
	return GetBase()->Getlang();
}

bool Mp7JrsTitleType::Existlang() const
{
	return GetBase()->Existlang();
}
void Mp7JrsTitleType::Setlang(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleType::Setlang().");
	}
	GetBase()->Setlang(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleType::Invalidatelang()
{
	GetBase()->Invalidatelang();
}
Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr Mp7JrsTitleType::GetphoneticTranscription() const
{
	return GetBase()->GetphoneticTranscription();
}

bool Mp7JrsTitleType::ExistphoneticTranscription() const
{
	return GetBase()->ExistphoneticTranscription();
}
void Mp7JrsTitleType::SetphoneticTranscription(const Mp7JrsTextualBaseType_phoneticTranscription_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleType::SetphoneticTranscription().");
	}
	GetBase()->SetphoneticTranscription(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleType::InvalidatephoneticTranscription()
{
	GetBase()->InvalidatephoneticTranscription();
}
Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsTitleType::GetphoneticAlphabet() const
{
	return GetBase()->GetphoneticAlphabet();
}

bool Mp7JrsTitleType::ExistphoneticAlphabet() const
{
	return GetBase()->ExistphoneticAlphabet();
}
void Mp7JrsTitleType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsTitleType::SetphoneticAlphabet().");
	}
	GetBase()->SetphoneticAlphabet(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsTitleType::InvalidatephoneticAlphabet()
{
	GetBase()->InvalidatephoneticAlphabet();
}

Mp7JrsNodeEnum Mp7JrsTitleType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for TitleType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "TitleType");
	}
	
	return result;
}

XMLCh * Mp7JrsTitleType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsTitleType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsTitleType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	if(m_Base)
	{
		return m_Base->ContentFromString(txt);
	}
	return false;
}
XMLCh* Mp7JrsTitleType::ContentToString() const
{
	if(m_Base)
	{
		return m_Base->ContentToString();
	}
	return (XMLCh*)NULL;
}

void Mp7JrsTitleType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsTitleType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("TitleType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_type_Exist)
	{
	// Class
	if (m_type != Mp7JrsTitleType_type_Local2Ptr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		XMLCh * tmp = m_type->ToText();
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsTitleType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        // Deserialize class type
		Mp7JrsTitleType_type_Local2Ptr tmp = CreateTitleType_type_LocalType2;
		tmp->Parse(parent->getAttributeNS(X(""), X("type")));
        this->Settype(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
		// Deserialize class type
        Mp7JrsTitleType_type_Local2Ptr tmp = CreateTitleType_type_LocalType2;
        tmp->Parse(parent->getAttribute(X("type")));
        this->Settype(tmp);
        * current = parent;
    }

	// Extensionbase is Mp7JrsTextualBaseType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
// no includefile for extension defined 
// file Mp7JrsTitleType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsTitleType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsTitleType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsTitleType_ExtMethodImpl.h


