 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtImplInclude.h



#include "Mp7JrsUniqueIDType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsUniqueIDType::IMp7JrsUniqueIDType()
{

// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtPropInit.h

}

IMp7JrsUniqueIDType::~IMp7JrsUniqueIDType()
{
// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtPropCleanup.h

}

Mp7JrsUniqueIDType::Mp7JrsUniqueIDType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUniqueIDType::~Mp7JrsUniqueIDType()
{
	Cleanup();
}

void Mp7JrsUniqueIDType::Init()
{
	// Init base
	m_Base = XMLString::transcode("");

	// Init attributes
	m_type = NULL; // String
	m_type_Default = XMLString::transcode("URI");
	m_type_Exist = false;
	m_organization = NULL; // String
	m_organization_Exist = false;
	m_authority = NULL; // String
	m_authority_Exist = false;
	m_encoding = Mp7JrsUniqueIDType_encoding_LocalType::UninitializedEnumeration;
// old	m_encoding = Mp7JrsUniqueIDType_encoding_LocalType::Enumeration::;
// old	m_encoding_Default = Mp7JrsUniqueIDType_encoding_LocalType::Enumeration::text;
	m_encoding_Default = Mp7JrsUniqueIDType_encoding_LocalType::text;
	m_encoding_Exist = false;



// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtMyPropInit.h

}

void Mp7JrsUniqueIDType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtMyPropCleanup.h


	XMLString::release(&m_Base);
	XMLString::release(&m_type);
	XMLString::release(&m_organization);
	XMLString::release(&m_authority);
}

void Mp7JrsUniqueIDType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UniqueIDTypePtr, since we
	// might need GetBase(), which isn't defined in IUniqueIDType
    const Mp7JrsPtr< Mp7JrsUniqueIDType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(tmp->GetContent());
	{
	XMLString::release(&m_type);
	if (tmp->Existtype())
	{
		this->Settype(XMLString::replicate(tmp->Gettype()));
	}
	else
	{
		Invalidatetype();
	}
	}
	{
	XMLString::release(&m_organization);
	if (tmp->Existorganization())
	{
		this->Setorganization(XMLString::replicate(tmp->Getorganization()));
	}
	else
	{
		Invalidateorganization();
	}
	}
	{
	XMLString::release(&m_authority);
	if (tmp->Existauthority())
	{
		this->Setauthority(XMLString::replicate(tmp->Getauthority()));
	}
	else
	{
		Invalidateauthority();
	}
	}
	{
	if (tmp->Existencoding())
	{
		this->Setencoding(tmp->Getencoding());
	}
	else
	{
		Invalidateencoding();
	}
	}
}

Mp7JrsNodePtr Mp7JrsUniqueIDType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsUniqueIDType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsUniqueIDType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUniqueIDType::SetContent().");
	}
    if (m_Base != item)
    {
		XMLString::release(&m_Base);
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsUniqueIDType::GetContent() const
{
	return m_Base;
}
XMLCh * Mp7JrsUniqueIDType::Gettype() const
{
	if (this->Existtype()) {
		return m_type;
	} else {
		return m_type_Default;
	}
}

bool Mp7JrsUniqueIDType::Existtype() const
{
	return m_type_Exist;
}
void Mp7JrsUniqueIDType::Settype(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUniqueIDType::Settype().");
	}
    m_type = item;
	m_type_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUniqueIDType::Invalidatetype()
{
	m_type_Exist = false;
}
XMLCh * Mp7JrsUniqueIDType::Getorganization() const
{
	return m_organization;
}

bool Mp7JrsUniqueIDType::Existorganization() const
{
	return m_organization_Exist;
}
void Mp7JrsUniqueIDType::Setorganization(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUniqueIDType::Setorganization().");
	}
    m_organization = item;
	m_organization_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUniqueIDType::Invalidateorganization()
{
	m_organization_Exist = false;
}
XMLCh * Mp7JrsUniqueIDType::Getauthority() const
{
	return m_authority;
}

bool Mp7JrsUniqueIDType::Existauthority() const
{
	return m_authority_Exist;
}
void Mp7JrsUniqueIDType::Setauthority(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUniqueIDType::Setauthority().");
	}
    m_authority = item;
	m_authority_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUniqueIDType::Invalidateauthority()
{
	m_authority_Exist = false;
}
Mp7JrsUniqueIDType_encoding_LocalType::Enumeration Mp7JrsUniqueIDType::Getencoding() const
{
	if (this->Existencoding()) {
		return m_encoding;
	} else {
		return m_encoding_Default;
	}
}

bool Mp7JrsUniqueIDType::Existencoding() const
{
	return m_encoding_Exist;
}
void Mp7JrsUniqueIDType::Setencoding(Mp7JrsUniqueIDType_encoding_LocalType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUniqueIDType::Setencoding().");
	}
    m_encoding = item;
	m_encoding_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUniqueIDType::Invalidateencoding()
{
	m_encoding_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsUniqueIDType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for UniqueIDType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "UniqueIDType");
	}
	
	return result;
}

XMLCh * Mp7JrsUniqueIDType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsUniqueIDType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsUniqueIDType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsUniqueIDType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsUniqueIDType::ContentToString() const
{
	// Mp7JrsUniqueIDType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsUniqueIDType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


    element->appendChild(doc->createTextNode(m_Base));

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UniqueIDType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_type_Exist)
	{
	// String
	if(m_type != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("type"), m_type);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_organization_Exist)
	{
	// String
	if(m_organization != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("organization"), m_organization);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_authority_Exist)
	{
	// String
	if(m_authority != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("authority"), m_authority);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_encoding_Exist)
	{
	// Enumeration
	if(m_encoding != Mp7JrsUniqueIDType_encoding_LocalType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsUniqueIDType_encoding_LocalType::ToText(m_encoding);
		element->setAttributeNS(X(""), X("encoding"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsUniqueIDType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        // Deserialize string type
        this->Settype(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("type"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
        // Deserialize string type
        this->Settype(Mp7JrsConvert::TextToString(parent->getAttribute(X("type"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("organization")))
	{
        // Deserialize string type
        this->Setorganization(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("organization"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("organization")))
	{
        // Deserialize string type
        this->Setorganization(Mp7JrsConvert::TextToString(parent->getAttribute(X("organization"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("authority")))
	{
        // Deserialize string type
        this->Setauthority(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("authority"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("authority")))
	{
        // Deserialize string type
        this->Setauthority(Mp7JrsConvert::TextToString(parent->getAttribute(X("authority"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("encoding")))
	{
        this->Setencoding(Mp7JrsUniqueIDType_encoding_LocalType::Parse(parent->getAttributeNS(X(""), X("encoding"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("encoding")))
	{
        this->Setencoding(Mp7JrsUniqueIDType_encoding_LocalType::Parse(parent->getAttribute(X("encoding"))));
        * current = parent;
    }

	XMLCh * tmp = Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		XMLString::release(&m_Base);
		m_Base = tmp;
		found = true;
	}
// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUniqueIDType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUniqueIDType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUniqueIDType_ExtMethodImpl.h


