
// Based on Unknown_cpp.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include "Mp7JrsUnknown.h"

Mp7JrsUnknown::Mp7JrsUnknown()
{
	Init();
}

Mp7JrsUnknown::~Mp7JrsUnknown()
{
	Cleanup();
}

void Mp7JrsUnknown::Init()
{
	XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementation * impl = XERCES_CPP_NAMESPACE_QUALIFIER DOMImplementationRegistry::getDOMImplementation(X("Core"));
	m_Document = impl->createDocument(0,X("dummyroot"),0);
	m_DocumentFragment = m_Document->createDocumentFragment();
}

void Mp7JrsUnknown::Cleanup()
{
	if(m_DocumentFragment != NULL)
	{
		m_DocumentFragment->release();
		m_DocumentFragment = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocumentFragment *)NULL;
	}
	if(m_Document != NULL)
	{
		m_Document->release();
		m_Document = (XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *)NULL;
	}
}

void Mp7JrsUnknown::DeepCopy(const Mp7JrsNodePtr &original)
{
	const Mp7JrsUnknown* tmp = dynamic_cast<const Mp7JrsUnknown *> (original.Ptr());
	if (!tmp) return; // EXIT: wrong argument
	DOMNode * unknownxml = (DOMNode *)tmp->GetUnknownXml();
	if(unknownxml != NULL)
	{
		if(m_DocumentFragment->getFirstChild() != NULL)
		{
			m_DocumentFragment->getFirstChild()->release();
		}

		bool deep = true;
		// create clone with m_Document as responsible xml doc
		DOMNode * clone = m_Document->importNode
			(unknownxml, deep);
		m_DocumentFragment->appendChild(clone);
	}
}

const DOMElement * Mp7JrsUnknown::GetUnknownXml() const
{
	// User must clone the result
	if(m_DocumentFragment != NULL)
	{
		return (DOMElement *)m_DocumentFragment->getFirstChild();
	}
	return NULL;
}

void Mp7JrsUnknown::SetUnknownXml(const XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * unknownxml)
{
	// We create a clone of the data
	if(m_DocumentFragment->getFirstChild() != NULL)
	{
		m_DocumentFragment->getFirstChild()->release();
	}
	if(unknownxml != NULL)
	{
		bool deep = true;
		// create clone with m_Document as responsible xml doc
		XERCES_CPP_NAMESPACE_QUALIFIER DOMNode * clone = m_Document->importNode
			((XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *)unknownxml, deep);
		m_DocumentFragment->appendChild(clone);
	}
}

void Mp7JrsUnknown::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** /*newElem*/)
{
	// doc MUST be valid!

	if(parent == NULL)
	{
		parent = doc->getDocumentElement();
	}

	if(m_DocumentFragment->getFirstChild() != NULL)
	{
		bool deep = true;
		// create clone with doc as responsible xml doc
		XERCES_CPP_NAMESPACE_QUALIFIER DOMNode * clone = doc->importNode
			(m_DocumentFragment->getFirstChild(), deep);
		parent->appendChild(clone);
	}
}

bool Mp7JrsUnknown::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * /*doc*/, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** /*current*/)
{
	if(parent == NULL)
	{
		return false;
	}
	if(m_DocumentFragment->getFirstChild() != NULL)
	{
		m_DocumentFragment->getFirstChild()->release();
	}
	bool deep = true;
	// create clone with m_Document as responsible xml doc
	XERCES_CPP_NAMESPACE_QUALIFIER DOMNode * clone = m_Document->importNode
		(parent, deep);
	m_DocumentFragment->appendChild(clone);
	return true;
}

Mp7JrsNodeEnum Mp7JrsUnknown::GetAllChildElements() const {
	Mp7JrsNodeEnum result;
	return result;
}
