 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsRightsType.h"
#include "Mp7JrsFinancialType.h"
#include "Mp7JrsUsageInformationType_Availability_CollectionType.h"
#include "Mp7JrsUsageInformationType_UsageRecord_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsUsageInformationType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsAvailabilityType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Availability
#include "Mp7JrsUsageRecordType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UsageRecord

#include <assert.h>
IMp7JrsUsageInformationType::IMp7JrsUsageInformationType()
{

// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtPropInit.h

}

IMp7JrsUsageInformationType::~IMp7JrsUsageInformationType()
{
// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtPropCleanup.h

}

Mp7JrsUsageInformationType::Mp7JrsUsageInformationType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUsageInformationType::~Mp7JrsUsageInformationType()
{
	Cleanup();
}

void Mp7JrsUsageInformationType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	


	// Init elements (element, union sequence choice all any)
	
	m_Rights = Mp7JrsRightsPtr(); // Class
	m_Rights_Exist = false;
	m_FinancialResults = Mp7JrsFinancialPtr(); // Class
	m_FinancialResults_Exist = false;
	m_Availability = Mp7JrsUsageInformationType_Availability_CollectionPtr(); // Collection
	m_UsageRecord = Mp7JrsUsageInformationType_UsageRecord_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtMyPropInit.h

}

void Mp7JrsUsageInformationType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Rights);
	// Mp7JrsFactory::DeleteObject(m_FinancialResults);
	// Mp7JrsFactory::DeleteObject(m_Availability);
	// Mp7JrsFactory::DeleteObject(m_UsageRecord);
}

void Mp7JrsUsageInformationType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UsageInformationTypePtr, since we
	// might need GetBase(), which isn't defined in IUsageInformationType
    const Mp7JrsPtr< Mp7JrsUsageInformationType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsUsageInformationType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	if (tmp->IsValidRights())
	{
		// Mp7JrsFactory::DeleteObject(m_Rights);
		this->SetRights(Mp7JrsFactory::CloneObject(tmp->GetRights()));
	}
	else
	{
		InvalidateRights();
	}
	if (tmp->IsValidFinancialResults())
	{
		// Mp7JrsFactory::DeleteObject(m_FinancialResults);
		this->SetFinancialResults(Mp7JrsFactory::CloneObject(tmp->GetFinancialResults()));
	}
	else
	{
		InvalidateFinancialResults();
	}
		// Mp7JrsFactory::DeleteObject(m_Availability);
		this->SetAvailability(Mp7JrsFactory::CloneObject(tmp->GetAvailability()));
		// Mp7JrsFactory::DeleteObject(m_UsageRecord);
		this->SetUsageRecord(Mp7JrsFactory::CloneObject(tmp->GetUsageRecord()));
}

Mp7JrsNodePtr Mp7JrsUsageInformationType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsUsageInformationType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsUsageInformationType::GetBase() const
{
	return m_Base;
}

Mp7JrsRightsPtr Mp7JrsUsageInformationType::GetRights() const
{
	return m_Rights;
}

// Element is optional
bool Mp7JrsUsageInformationType::IsValidRights() const
{
	return m_Rights_Exist;
}

Mp7JrsFinancialPtr Mp7JrsUsageInformationType::GetFinancialResults() const
{
	return m_FinancialResults;
}

// Element is optional
bool Mp7JrsUsageInformationType::IsValidFinancialResults() const
{
	return m_FinancialResults_Exist;
}

Mp7JrsUsageInformationType_Availability_CollectionPtr Mp7JrsUsageInformationType::GetAvailability() const
{
	return m_Availability;
}

Mp7JrsUsageInformationType_UsageRecord_CollectionPtr Mp7JrsUsageInformationType::GetUsageRecord() const
{
	return m_UsageRecord;
}

void Mp7JrsUsageInformationType::SetRights(const Mp7JrsRightsPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetRights().");
	}
	if (m_Rights != item || m_Rights_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Rights);
		m_Rights = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Rights) m_Rights->SetParent(m_myself.getPointer());
		if (m_Rights && m_Rights->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RightsType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Rights->UseTypeAttribute = true;
		}
		m_Rights_Exist = true;
		if(m_Rights != Mp7JrsRightsPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Rights->SetContentName(XMLString::transcode("Rights"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidateRights()
{
	m_Rights_Exist = false;
}
void Mp7JrsUsageInformationType::SetFinancialResults(const Mp7JrsFinancialPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetFinancialResults().");
	}
	if (m_FinancialResults != item || m_FinancialResults_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_FinancialResults);
		m_FinancialResults = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_FinancialResults) m_FinancialResults->SetParent(m_myself.getPointer());
		if (m_FinancialResults && m_FinancialResults->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FinancialType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_FinancialResults->UseTypeAttribute = true;
		}
		m_FinancialResults_Exist = true;
		if(m_FinancialResults != Mp7JrsFinancialPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_FinancialResults->SetContentName(XMLString::transcode("FinancialResults"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidateFinancialResults()
{
	m_FinancialResults_Exist = false;
}
void Mp7JrsUsageInformationType::SetAvailability(const Mp7JrsUsageInformationType_Availability_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetAvailability().");
	}
	if (m_Availability != item)
	{
		// Mp7JrsFactory::DeleteObject(m_Availability);
		m_Availability = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Availability) m_Availability->SetParent(m_myself.getPointer());
		if(m_Availability != Mp7JrsUsageInformationType_Availability_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Availability->SetContentName(XMLString::transcode("Availability"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::SetUsageRecord(const Mp7JrsUsageInformationType_UsageRecord_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetUsageRecord().");
	}
	if (m_UsageRecord != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UsageRecord);
		m_UsageRecord = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UsageRecord) m_UsageRecord->SetParent(m_myself.getPointer());
		if(m_UsageRecord != Mp7JrsUsageInformationType_UsageRecord_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UsageRecord->SetContentName(XMLString::transcode("UsageRecord"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsUsageInformationType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsUsageInformationType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsUsageInformationType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsUsageInformationType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsUsageInformationType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsUsageInformationType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsUsageInformationType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsUsageInformationType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsUsageInformationType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsUsageInformationType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsUsageInformationType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsUsageInformationType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsUsageInformationType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsUsageInformationType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsUsageInformationType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUsageInformationType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsUsageInformationType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsUsageInformationType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUsageInformationType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsUsageInformationType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Rights")) == 0)
	{
		// Rights is simple element RightsType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetRights()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:RightsType")))) != empty)
			{
				// Is type allowed
				Mp7JrsRightsPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetRights(p, client);
					if((p = GetRights()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("FinancialResults")) == 0)
	{
		// FinancialResults is simple element FinancialType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetFinancialResults()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:FinancialType")))) != empty)
			{
				// Is type allowed
				Mp7JrsFinancialPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetFinancialResults(p, client);
					if((p = GetFinancialResults()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("Availability")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:Availability is item of type AvailabilityType
		// in element collection UsageInformationType_Availability_CollectionType
		
		context->Found = true;
		Mp7JrsUsageInformationType_Availability_CollectionPtr coll = GetAvailability();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUsageInformationType_Availability_CollectionType;
				SetAvailability(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:AvailabilityType")))) != empty)
			{
				// Is type allowed
				Mp7JrsAvailabilityPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UsageRecord")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UsageRecord is item of type UsageRecordType
		// in element collection UsageInformationType_UsageRecord_CollectionType
		
		context->Found = true;
		Mp7JrsUsageInformationType_UsageRecord_CollectionPtr coll = GetUsageRecord();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUsageInformationType_UsageRecord_CollectionType;
				SetUsageRecord(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UsageRecordType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUsageRecordPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for UsageInformationType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "UsageInformationType");
	}		
	
	return result;
}

XMLCh * Mp7JrsUsageInformationType::ToText() const
{
	return m_Base->ToText();
}


bool Mp7JrsUsageInformationType::Parse(const XMLCh * const txt)
{
	return m_Base->Parse(txt);
}


void Mp7JrsUsageInformationType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UsageInformationType"));
	// Element serialization:
	// Class
	
	if (m_Rights != Mp7JrsRightsPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Rights->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Rights"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Rights->Serialize(doc, element, &elem);
	}
	// Class
	
	if (m_FinancialResults != Mp7JrsFinancialPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_FinancialResults->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("FinancialResults"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_FinancialResults->Serialize(doc, element, &elem);
	}
	if (m_Availability != Mp7JrsUsageInformationType_Availability_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_Availability->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_UsageRecord != Mp7JrsUsageInformationType_UsageRecord_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UsageRecord->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsUsageInformationType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Rights"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Rights")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateRightsType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetRights(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("FinancialResults"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("FinancialResults")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateFinancialType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetFinancialResults(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("Availability"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("Availability")) == 0))
        {
            // Deserialize factory type
            Mp7JrsUsageInformationType_Availability_CollectionPtr tmp = CreateUsageInformationType_Availability_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetAvailability(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("UsageRecord"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("UsageRecord")) == 0))
        {
            // Deserialize factory type
            Mp7JrsUsageInformationType_UsageRecord_CollectionPtr tmp = CreateUsageInformationType_UsageRecord_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetUsageRecord(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUsageInformationType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Rights")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateRightsType;
    }
    this->SetRights(child);
  }
  if (XMLString::compareString(elementname, X("FinancialResults")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateFinancialType;
    }
    this->SetFinancialResults(child);
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("Availability")) == 0))
  {
    Mp7JrsUsageInformationType_Availability_CollectionPtr tmp = CreateUsageInformationType_Availability_CollectionType;
    this->SetAvailability(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UsageRecord")) == 0))
  {
    Mp7JrsUsageInformationType_UsageRecord_CollectionPtr tmp = CreateUsageInformationType_UsageRecord_CollectionType;
    this->SetUsageRecord(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUsageInformationType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetRights() != Mp7JrsNodePtr())
		result.Insert(GetRights());
	if (GetFinancialResults() != Mp7JrsNodePtr())
		result.Insert(GetFinancialResults());
	if (GetAvailability() != Mp7JrsNodePtr())
		result.Insert(GetAvailability());
	if (GetUsageRecord() != Mp7JrsNodePtr())
		result.Insert(GetUsageRecord());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUsageInformationType_ExtMethodImpl.h


