 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionType.h"
#include "Mp7JrsUserActionHistoryType_UserActionList_CollectionType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsUserActionHistoryType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header
#include "Mp7JrsTimeType.h" // Element collection urn:mpeg:mpeg7:schema:2004:ObservationPeriod
#include "Mp7JrsUserActionListType.h" // Element collection urn:mpeg:mpeg7:schema:2004:UserActionList

#include <assert.h>
IMp7JrsUserActionHistoryType::IMp7JrsUserActionHistoryType()
{

// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtPropInit.h

}

IMp7JrsUserActionHistoryType::~IMp7JrsUserActionHistoryType()
{
// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtPropCleanup.h

}

Mp7JrsUserActionHistoryType::Mp7JrsUserActionHistoryType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsUserActionHistoryType::~Mp7JrsUserActionHistoryType()
{
	Cleanup();
}

void Mp7JrsUserActionHistoryType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_protected = Mp7JrsuserChoiceType::UninitializedEnumeration;
// old	m_protected = Mp7JrsuserChoiceType::Enumeration::;
// old	m_protected_Default = Mp7JrsuserChoiceType::Enumeration::_true;
	m_protected_Default = Mp7JrsuserChoiceType::_true;
	m_protected_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_ObservationPeriod = Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr(); // Collection
	m_UserActionList = Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr(); // Collection


// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtMyPropInit.h

}

void Mp7JrsUserActionHistoryType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_ObservationPeriod);
	// Mp7JrsFactory::DeleteObject(m_UserActionList);
}

void Mp7JrsUserActionHistoryType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use UserActionHistoryTypePtr, since we
	// might need GetBase(), which isn't defined in IUserActionHistoryType
    const Mp7JrsPtr< Mp7JrsUserActionHistoryType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsUserActionHistoryType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existprotected())
	{
		this->Setprotected(tmp->Getprotected());
	}
	else
	{
		Invalidateprotected();
	}
	}
		// Mp7JrsFactory::DeleteObject(m_ObservationPeriod);
		this->SetObservationPeriod(Mp7JrsFactory::CloneObject(tmp->GetObservationPeriod()));
		// Mp7JrsFactory::DeleteObject(m_UserActionList);
		this->SetUserActionList(Mp7JrsFactory::CloneObject(tmp->GetUserActionList()));
}

Mp7JrsNodePtr Mp7JrsUserActionHistoryType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsUserActionHistoryType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsUserActionHistoryType::GetBase() const
{
	return m_Base;
}

Mp7JrsuserChoiceType::Enumeration Mp7JrsUserActionHistoryType::Getprotected() const
{
	if (this->Existprotected()) {
		return m_protected;
	} else {
		return m_protected_Default;
	}
}

bool Mp7JrsUserActionHistoryType::Existprotected() const
{
	return m_protected_Exist;
}
void Mp7JrsUserActionHistoryType::Setprotected(Mp7JrsuserChoiceType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::Setprotected().");
	}
    m_protected = item;
	m_protected_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::Invalidateprotected()
{
	m_protected_Exist = false;
}
Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr Mp7JrsUserActionHistoryType::GetObservationPeriod() const
{
	return m_ObservationPeriod;
}

Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr Mp7JrsUserActionHistoryType::GetUserActionList() const
{
	return m_UserActionList;
}

void Mp7JrsUserActionHistoryType::SetObservationPeriod(const Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SetObservationPeriod().");
	}
	if (m_ObservationPeriod != item)
	{
		// Mp7JrsFactory::DeleteObject(m_ObservationPeriod);
		m_ObservationPeriod = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_ObservationPeriod) m_ObservationPeriod->SetParent(m_myself.getPointer());
		if(m_ObservationPeriod != Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_ObservationPeriod->SetContentName(XMLString::transcode("ObservationPeriod"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::SetUserActionList(const Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SetUserActionList().");
	}
	if (m_UserActionList != item)
	{
		// Mp7JrsFactory::DeleteObject(m_UserActionList);
		m_UserActionList = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_UserActionList) m_UserActionList->SetParent(m_myself.getPointer());
		if(m_UserActionList != Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_UserActionList->SetContentName(XMLString::transcode("UserActionList"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsUserActionHistoryType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsUserActionHistoryType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsUserActionHistoryType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsUserActionHistoryType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsUserActionHistoryType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsUserActionHistoryType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsUserActionHistoryType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsUserActionHistoryType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsUserActionHistoryType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsUserActionHistoryType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsUserActionHistoryType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsUserActionHistoryType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsUserActionHistoryType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsUserActionHistoryType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsUserActionHistoryType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsUserActionHistoryType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsUserActionHistoryType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsUserActionHistoryType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsUserActionHistoryType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsUserActionHistoryType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("ObservationPeriod")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:ObservationPeriod is item of type TimeType
		// in element collection UserActionHistoryType_ObservationPeriod_CollectionType
		
		context->Found = true;
		Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr coll = GetObservationPeriod();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUserActionHistoryType_ObservationPeriod_CollectionType;
				SetObservationPeriod(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:TimeType")))) != empty)
			{
				// Is type allowed
				Mp7JrsTimePtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	//------------------------------------------------------------------
	else if(XMLString::compareString(context->ElementName, X("UserActionList")) == 0)
	{
		// urn:mpeg:mpeg7:schema:2004:UserActionList is item of type UserActionListType
		// in element collection UserActionHistoryType_UserActionList_CollectionType
		
		context->Found = true;
		Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr coll = GetUserActionList();
		if(coll == empty) // Create one via factory
		{
			if(context->Create)
			{
				coll = CreateUserActionHistoryType_UserActionList_CollectionType;
				SetUserActionList(coll, client);
			}
			else
			{
				return result; // No collection, silently return empty enumeration
			}
		}
		if(!Mp7JrsXPathParseContext::PreProcessCollection(context, coll->size(), -1))
		{
			return result;
		}
		
		int i = 0;
		Mp7JrsNodePtr p;
		while(i < (int)coll->size())
		{
			if((p = coll->elementAt(i)) != empty)
			{
				i = Mp7JrsXPathParseContext::HandleCollection(context, i, p, result);
			}
			else
			{
				 Mp7JrsXPathParseContext::ErrorInvalidEmptyItem(context, i);
			}
		}
		
		if(Mp7JrsXPathParseContext::PostProcessElementCollection(context, i, result))
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:UserActionListType")))) != empty)
			{
				// Is type allowed
				Mp7JrsUserActionListPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Mp7JrsUnkown" type is allowed
				{
					coll->insertElementAt(p, i, client); // Insert or append
					if((p = coll->elementAt(i)) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			while(result.HasNext())
			{
				subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			}
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for UserActionHistoryType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "UserActionHistoryType");
	}		
	
	return result;
}

XMLCh * Mp7JrsUserActionHistoryType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsUserActionHistoryType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsUserActionHistoryType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("UserActionHistoryType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_protected_Exist)
	{
	// Enumeration
	if(m_protected != Mp7JrsuserChoiceType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsuserChoiceType::ToText(m_protected);
		element->setAttributeNS(X(""), X("protected"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	if (m_ObservationPeriod != Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_ObservationPeriod->Serialize(doc, element, NULL); // &element); - mio 041123
	}
	if (m_UserActionList != Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
	{
		// Collection
		// No extra child node needed for choices or sequences with maxOccurs > 1
		m_UserActionList->Serialize(doc, element, NULL); // &element); - mio 041123
	}

}

bool Mp7JrsUserActionHistoryType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("protected")))
	{
        this->Setprotected(Mp7JrsuserChoiceType::Parse(parent->getAttributeNS(X(""), X("protected"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("protected")))
	{
        this->Setprotected(Mp7JrsuserChoiceType::Parse(parent->getAttribute(X("protected"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("ObservationPeriod"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("ObservationPeriod")) == 0))
        {
            // Deserialize factory type
            Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr tmp = CreateUserActionHistoryType_ObservationPeriod_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetObservationPeriod(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
        
        // Element Collection
        if((parent != NULL) && 
        Mp7JrsUtil::HasNodeName(parent, X("UserActionList"), X("urn:mpeg:mpeg7:schema:2004")))
// OLD        (XMLString::compareString(parent->getNodeName(), X("UserActionList")) == 0))
        {
            // Deserialize factory type
            Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr tmp = CreateUserActionHistoryType_UserActionList_CollectionType;
            tmp->Deserialize(doc, parent, current);
            this->SetUserActionList(tmp);
            found = true;
            parent = * current; // For collections
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsUserActionHistoryType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("ObservationPeriod")) == 0))
  {
    Mp7JrsUserActionHistoryType_ObservationPeriod_CollectionPtr tmp = CreateUserActionHistoryType_ObservationPeriod_CollectionType;
    this->SetObservationPeriod(tmp);
    child = tmp;
  }
  // Non-Choice Collection
  if((XMLString::compareString(elementname, X("UserActionList")) == 0))
  {
    Mp7JrsUserActionHistoryType_UserActionList_CollectionPtr tmp = CreateUserActionHistoryType_UserActionList_CollectionType;
    this->SetUserActionList(tmp);
    child = tmp;
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsUserActionHistoryType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetObservationPeriod() != Mp7JrsNodePtr())
		result.Insert(GetObservationPeriod());
	if (GetUserActionList() != Mp7JrsNodePtr())
		result.Insert(GetUserActionList());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsUserActionHistoryType_ExtMethodImpl.h


