 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtImplInclude.h



#include "Mp7JrsDSType.h"
#include "Mp7JrsSignalType.h"
#include "Mp7JrsxPathRefType.h"
#include "Mp7JrsdurationType.h"
#include "Mp7JrsmediaDurationType.h"
#include "Mp7JrsDSType_Header_CollectionType.h"
#include "Mp7JrsViewDecompositionType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"
#include "Mp7JrsHeaderType.h" // Element collection urn:mpeg:mpeg7:schema:2004:Header

#include <assert.h>
IMp7JrsViewDecompositionType::IMp7JrsViewDecompositionType()
{

// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtPropInit.h

}

IMp7JrsViewDecompositionType::~IMp7JrsViewDecompositionType()
{
// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtPropCleanup.h

}

Mp7JrsViewDecompositionType::Mp7JrsViewDecompositionType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsViewDecompositionType::~Mp7JrsViewDecompositionType()
{
	Cleanup();
}

void Mp7JrsViewDecompositionType::Init()
{
	// Init base
	m_Base = CreateDSType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_complete = false; // Value
	m_complete_Exist = false;
	m_nonRedundant = false; // Value
	m_nonRedundant_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Source = Mp7JrsSignalPtr(); // Class
	m_Source_Exist = false;


// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtMyPropInit.h

}

void Mp7JrsViewDecompositionType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Source);
}

void Mp7JrsViewDecompositionType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use ViewDecompositionTypePtr, since we
	// might need GetBase(), which isn't defined in IViewDecompositionType
    const Mp7JrsPtr< Mp7JrsViewDecompositionType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsDSPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsViewDecompositionType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->Existcomplete())
	{
		this->Setcomplete(tmp->Getcomplete());
	}
	else
	{
		Invalidatecomplete();
	}
	}
	{
	if (tmp->ExistnonRedundant())
	{
		this->SetnonRedundant(tmp->GetnonRedundant());
	}
	else
	{
		InvalidatenonRedundant();
	}
	}
	if (tmp->IsValidSource())
	{
		// Mp7JrsFactory::DeleteObject(m_Source);
		this->SetSource(Mp7JrsFactory::CloneObject(tmp->GetSource()));
	}
	else
	{
		InvalidateSource();
	}
}

Mp7JrsNodePtr Mp7JrsViewDecompositionType::GetBaseRoot()
{
	return GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsViewDecompositionType::GetBaseRoot() const
{
	return GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsDSType > Mp7JrsViewDecompositionType::GetBase() const
{
	return m_Base;
}

bool Mp7JrsViewDecompositionType::Getcomplete() const
{
	return m_complete;
}

bool Mp7JrsViewDecompositionType::Existcomplete() const
{
	return m_complete_Exist;
}
void Mp7JrsViewDecompositionType::Setcomplete(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::Setcomplete().");
	}
    m_complete = item;
	m_complete_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::Invalidatecomplete()
{
	m_complete_Exist = false;
}
bool Mp7JrsViewDecompositionType::GetnonRedundant() const
{
	return m_nonRedundant;
}

bool Mp7JrsViewDecompositionType::ExistnonRedundant() const
{
	return m_nonRedundant_Exist;
}
void Mp7JrsViewDecompositionType::SetnonRedundant(bool item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SetnonRedundant().");
	}
    m_nonRedundant = item;
	m_nonRedundant_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidatenonRedundant()
{
	m_nonRedundant_Exist = false;
}
Mp7JrsSignalPtr Mp7JrsViewDecompositionType::GetSource() const
{
	return m_Source;
}

// Element is optional
bool Mp7JrsViewDecompositionType::IsValidSource() const
{
	return m_Source_Exist;
}

void Mp7JrsViewDecompositionType::SetSource(const Mp7JrsSignalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SetSource().");
	}
	if (m_Source != item || m_Source_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Source);
		m_Source = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Source) m_Source->SetParent(m_myself.getPointer());
		if (m_Source && m_Source->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Source->UseTypeAttribute = true;
		}
		m_Source_Exist = true;
		if(m_Source != Mp7JrsSignalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Source->SetContentName(XMLString::transcode("Source"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidateSource()
{
	m_Source_Exist = false;
}
XMLCh * Mp7JrsViewDecompositionType::Getid() const
{
	return GetBase()->Getid();
}

bool Mp7JrsViewDecompositionType::Existid() const
{
	return GetBase()->Existid();
}
void Mp7JrsViewDecompositionType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::Setid().");
	}
	GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::Invalidateid()
{
	GetBase()->Invalidateid();
}
Mp7JrsxPathRefPtr Mp7JrsViewDecompositionType::GettimeBase() const
{
	return GetBase()->GettimeBase();
}

bool Mp7JrsViewDecompositionType::ExisttimeBase() const
{
	return GetBase()->ExisttimeBase();
}
void Mp7JrsViewDecompositionType::SettimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SettimeBase().");
	}
	GetBase()->SettimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidatetimeBase()
{
	GetBase()->InvalidatetimeBase();
}
Mp7JrsdurationPtr Mp7JrsViewDecompositionType::GettimeUnit() const
{
	return GetBase()->GettimeUnit();
}

bool Mp7JrsViewDecompositionType::ExisttimeUnit() const
{
	return GetBase()->ExisttimeUnit();
}
void Mp7JrsViewDecompositionType::SettimeUnit(const Mp7JrsdurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SettimeUnit().");
	}
	GetBase()->SettimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidatetimeUnit()
{
	GetBase()->InvalidatetimeUnit();
}
Mp7JrsxPathRefPtr Mp7JrsViewDecompositionType::GetmediaTimeBase() const
{
	return GetBase()->GetmediaTimeBase();
}

bool Mp7JrsViewDecompositionType::ExistmediaTimeBase() const
{
	return GetBase()->ExistmediaTimeBase();
}
void Mp7JrsViewDecompositionType::SetmediaTimeBase(const Mp7JrsxPathRefPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SetmediaTimeBase().");
	}
	GetBase()->SetmediaTimeBase(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidatemediaTimeBase()
{
	GetBase()->InvalidatemediaTimeBase();
}
Mp7JrsmediaDurationPtr Mp7JrsViewDecompositionType::GetmediaTimeUnit() const
{
	return GetBase()->GetmediaTimeUnit();
}

bool Mp7JrsViewDecompositionType::ExistmediaTimeUnit() const
{
	return GetBase()->ExistmediaTimeUnit();
}
void Mp7JrsViewDecompositionType::SetmediaTimeUnit(const Mp7JrsmediaDurationPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SetmediaTimeUnit().");
	}
	GetBase()->SetmediaTimeUnit(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsViewDecompositionType::InvalidatemediaTimeUnit()
{
	GetBase()->InvalidatemediaTimeUnit();
}
Mp7JrsDSType_Header_CollectionPtr Mp7JrsViewDecompositionType::GetHeader() const
{
	return GetBase()->GetHeader();
}

void Mp7JrsViewDecompositionType::SetHeader(const Mp7JrsDSType_Header_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsViewDecompositionType::SetHeader().");
	}
	GetBase()->SetHeader(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}


Mp7JrsNodeEnum Mp7JrsViewDecompositionType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Source")) == 0)
	{
		// Source is simple element SignalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetSource()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:SignalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsSignalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetSource(p, client);
					if((p = GetSource()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for ViewDecompositionType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "ViewDecompositionType");
	}		
	
	return result;
}

XMLCh * Mp7JrsViewDecompositionType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsViewDecompositionType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsViewDecompositionType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("ViewDecompositionType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_complete_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_complete);
		element->setAttributeNS(X(""), X("complete"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_nonRedundant_Exist)
	{
	// Value
	{
		XMLCh * tmp = Mp7JrsConvert::BinToText(m_nonRedundant);
		element->setAttributeNS(X(""), X("nonRedundant"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Source != Mp7JrsSignalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Source->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Source"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Source->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsViewDecompositionType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("complete")))
	{
        // deserialize value type
        this->Setcomplete(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("complete"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("complete")))
	{
        // deserialize value type
        this->Setcomplete(Mp7JrsConvert::TextToBool(parent->getAttribute(X("complete"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("nonRedundant")))
	{
        // deserialize value type
        this->SetnonRedundant(Mp7JrsConvert::TextToBool(parent->getAttributeNS(X(""), X("nonRedundant"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("nonRedundant")))
	{
        // deserialize value type
        this->SetnonRedundant(Mp7JrsConvert::TextToBool(parent->getAttribute(X("nonRedundant"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsDSType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Source"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Source")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateSignalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetSource(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsViewDecompositionType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Source")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateSignalType;
    }
    this->SetSource(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsViewDecompositionType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetSource() != Mp7JrsNodePtr())
		result.Insert(GetSource());
	if (GetHeader() != Mp7JrsNodePtr())
		result.Insert(GetHeader());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsViewDecompositionType_ExtMethodImpl.h


