 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtImplInclude.h



#include "Mp7JrsWordFormType_terms_CollectionType.h"
#include "Mp7JrsWordFormType_type_CollectionType.h"
#include "Mp7JrsWordFormType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsWordFormType::IMp7JrsWordFormType()
{

// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtPropInit.h

}

IMp7JrsWordFormType::~IMp7JrsWordFormType()
{
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtPropCleanup.h

}

Mp7JrsWordFormType::Mp7JrsWordFormType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsWordFormType::~Mp7JrsWordFormType()
{
	Cleanup();
}

void Mp7JrsWordFormType::Init()
{
	// Init base
	m_Base = XMLString::transcode("");

	// Init attributes
	m_terms = Mp7JrsWordFormType_terms_CollectionPtr(); // Collection
	m_terms_Exist = false;
	m_id = NULL; // String
	m_id_Exist = false;
	m_equal = NULL; // String
	m_equal_Exist = false;
	m_type = Mp7JrsWordFormType_type_CollectionPtr(); // Collection
	m_type_Exist = false;
	m_baseForm = NULL; // String
	m_baseForm_Exist = false;



// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMyPropInit.h

}

void Mp7JrsWordFormType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMyPropCleanup.h


	XMLString::release(&m_Base);
	// Mp7JrsFactory::DeleteObject(m_terms);
	XMLString::release(&m_id);
	XMLString::release(&m_equal);
	// Mp7JrsFactory::DeleteObject(m_type);
	XMLString::release(&m_baseForm);
}

void Mp7JrsWordFormType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use WordFormTypePtr, since we
	// might need GetBase(), which isn't defined in IWordFormType
    const Mp7JrsPtr< Mp7JrsWordFormType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	XMLString::release(&m_Base);
	m_Base = XMLString::replicate(tmp->GetContent());
	{
	// Mp7JrsFactory::DeleteObject(m_terms);
	if (tmp->Existterms())
	{
		this->Setterms(Mp7JrsFactory::CloneObject(tmp->Getterms()));
	}
	else
	{
		Invalidateterms();
	}
	}
	{
	XMLString::release(&m_id);
	if (tmp->Existid())
	{
		this->Setid(XMLString::replicate(tmp->Getid()));
	}
	else
	{
		Invalidateid();
	}
	}
	{
	XMLString::release(&m_equal);
	if (tmp->Existequal())
	{
		this->Setequal(XMLString::replicate(tmp->Getequal()));
	}
	else
	{
		Invalidateequal();
	}
	}
	{
	// Mp7JrsFactory::DeleteObject(m_type);
	if (tmp->Existtype())
	{
		this->Settype(Mp7JrsFactory::CloneObject(tmp->Gettype()));
	}
	else
	{
		Invalidatetype();
	}
	}
	{
	XMLString::release(&m_baseForm);
	if (tmp->ExistbaseForm())
	{
		this->SetbaseForm(XMLString::replicate(tmp->GetbaseForm()));
	}
	else
	{
		InvalidatebaseForm();
	}
	}
}

Mp7JrsNodePtr Mp7JrsWordFormType::GetBaseRoot()
{
	return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsWordFormType::GetBaseRoot() const
{
	return m_myself.getPointer();
}


void Mp7JrsWordFormType::SetContent(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::SetContent().");
	}
    if (m_Base != item)
    {
		XMLString::release(&m_Base);
		m_Base = item;
	}
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

XMLCh * Mp7JrsWordFormType::GetContent() const
{
	return m_Base;
}
Mp7JrsWordFormType_terms_CollectionPtr Mp7JrsWordFormType::Getterms() const
{
	return m_terms;
}

bool Mp7JrsWordFormType::Existterms() const
{
	return m_terms_Exist;
}
void Mp7JrsWordFormType::Setterms(const Mp7JrsWordFormType_terms_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::Setterms().");
	}
    m_terms = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_terms) m_terms->SetParent(m_myself.getPointer());
	m_terms_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordFormType::Invalidateterms()
{
	m_terms_Exist = false;
}
XMLCh * Mp7JrsWordFormType::Getid() const
{
	return m_id;
}

bool Mp7JrsWordFormType::Existid() const
{
	return m_id_Exist;
}
void Mp7JrsWordFormType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::Setid().");
	}
    m_id = item;
	m_id_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordFormType::Invalidateid()
{
	m_id_Exist = false;
}
XMLCh * Mp7JrsWordFormType::Getequal() const
{
	return m_equal;
}

bool Mp7JrsWordFormType::Existequal() const
{
	return m_equal_Exist;
}
void Mp7JrsWordFormType::Setequal(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::Setequal().");
	}
    m_equal = item;
	m_equal_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordFormType::Invalidateequal()
{
	m_equal_Exist = false;
}
Mp7JrsWordFormType_type_CollectionPtr Mp7JrsWordFormType::Gettype() const
{
	return m_type;
}

bool Mp7JrsWordFormType::Existtype() const
{
	return m_type_Exist;
}
void Mp7JrsWordFormType::Settype(const Mp7JrsWordFormType_type_CollectionPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::Settype().");
	}
    m_type = item;
	// baw 27 04 2004: make sure that node on which parent is set is not null
	if (m_type) m_type->SetParent(m_myself.getPointer());
	m_type_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordFormType::Invalidatetype()
{
	m_type_Exist = false;
}
XMLCh * Mp7JrsWordFormType::GetbaseForm() const
{
	return m_baseForm;
}

bool Mp7JrsWordFormType::ExistbaseForm() const
{
	return m_baseForm_Exist;
}
void Mp7JrsWordFormType::SetbaseForm(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordFormType::SetbaseForm().");
	}
    m_baseForm = item;
	m_baseForm_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordFormType::InvalidatebaseForm()
{
	m_baseForm_Exist = false;
}

Mp7JrsNodeEnum Mp7JrsWordFormType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	Mp7JrsNodeEnum result = Mp7JrsNode::GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	if(isentrypoint)
	{
		// No Mp7JrsNodePtr subelement to get or create for WordFormType
		Mp7JrsXPathParseContext::ErrorNoSubElement(context, "WordFormType");
	}
	
	return result;
}

XMLCh * Mp7JrsWordFormType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsWordFormType::Parse(const XMLCh * const txt)
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return false;
}

bool Mp7JrsWordFormType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsWordFormType with workaround via Deserialise()
	return Mp7JrsXPathParseContext::ContentFromString(this, txt);
}
XMLCh* Mp7JrsWordFormType::ContentToString() const
{
	// Mp7JrsWordFormType with workaround via Serialise()
	return Mp7JrsXPathParseContext::ContentToString(this);
}

void Mp7JrsWordFormType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	else {
		// Create new element
		if (this->GetContentName() != (XMLCh*)NULL) {
			DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
			elem->setPrefix(prefix); // Might be NULL in case it is the default namespace and Archive::usePrefixForDefaultNS == false
			parent->appendChild(elem);
			element = elem;
		} else {
			// sat 2004-03-24: this should never happen
			assert(false);
		}
	}
	// baw 10 02 2004: added newElem to return new node to caller from derived class
	// sat 2004-03-18: set *newElem to parent unconditionally, not just in
	//   some branches (crashes if parent passed in, but no ContentName set)
	//   -- leaves out nodes, but at least doesn't crash.
	// baw 2004-04-19: set *newElem only if it is NULL, otherwise we would replace the value we get
	if (!(*newElem)) *newElem = element;


    element->appendChild(doc->createTextNode(m_Base));

	assert(element);
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("WordFormType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_terms_Exist)
	{
	// Collection
	if(m_terms != Mp7JrsWordFormType_terms_CollectionPtr())
	{
		XMLCh * tmp = m_terms->ToText();
		element->setAttributeNS(X(""), X("terms"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_id_Exist)
	{
	// String
	if(m_id != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("id"), m_id);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_equal_Exist)
	{
	// String
	if(m_equal != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("equal"), m_equal);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_type_Exist)
	{
	// Collection
	if(m_type != Mp7JrsWordFormType_type_CollectionPtr())
	{
		XMLCh * tmp = m_type->ToText();
		element->setAttributeNS(X(""), X("type"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
    // Attribute Serialization:
		

	// Optional attriute
	if(m_baseForm_Exist)
	{
	// String
	if(m_baseForm != (XMLCh *)NULL)
	{
		element->setAttributeNS(X(""), X("baseForm"), m_baseForm);
	}
		
	} // Optional attriute
	
	// Element serialization:

}

bool Mp7JrsWordFormType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("terms")))
	{
        // Deserialize collection type
		Mp7JrsWordFormType_terms_CollectionPtr tmp = CreateWordFormType_terms_CollectionType;
		tmp->Parse(parent->getAttributeNS(X(""), X("terms")));
        this->Setterms(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("terms")))
	{
     
		// Deserialize collection type
        Mp7JrsWordFormType_terms_CollectionPtr tmp = CreateWordFormType_terms_CollectionType;
        tmp->Parse(parent->getAttribute(X("terms")));
        this->Setterms(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("id"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("id")))
	{
        // Deserialize string type
        this->Setid(Mp7JrsConvert::TextToString(parent->getAttribute(X("id"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("equal")))
	{
        // Deserialize string type
        this->Setequal(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("equal"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("equal")))
	{
        // Deserialize string type
        this->Setequal(Mp7JrsConvert::TextToString(parent->getAttribute(X("equal"))));
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("type")))
	{
        // Deserialize collection type
		Mp7JrsWordFormType_type_CollectionPtr tmp = CreateWordFormType_type_CollectionType;
		tmp->Parse(parent->getAttributeNS(X(""), X("type")));
        this->Settype(tmp);
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("type")))
	{
     
		// Deserialize collection type
        Mp7JrsWordFormType_type_CollectionPtr tmp = CreateWordFormType_type_CollectionType;
        tmp->Parse(parent->getAttribute(X("type")));
        this->Settype(tmp);
        * current = parent;
    }

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("baseForm")))
	{
        // Deserialize string type
        this->SetbaseForm(Mp7JrsConvert::TextToString(parent->getAttributeNS(X(""), X("baseForm"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("baseForm")))
	{
        // Deserialize string type
        this->SetbaseForm(Mp7JrsConvert::TextToString(parent->getAttribute(X("baseForm"))));
        * current = parent;
    }

	XMLCh * tmp = Mp7JrsConvert::TextToString(Mp7JrsUtil::GetElementText(parent));
	if(tmp != NULL)
	{
		XMLString::release(&m_Base);
		m_Base = tmp;
		found = true;
	}
// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsWordFormType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = Mp7JrsNodePtr();
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsWordFormType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsWordFormType_ExtMethodImpl.h


