 

// Core library class
// based on ClassType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtImplInclude.h



#include "Mp7JrsLexiconType.h"
#include "Mp7JrsWordLexiconType_Token_LocalType.h"
#include "Mp7JrsWordLexiconType.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"
// XPath stuff
#include "Mp7JrsXPathParseContext.h"

#include <assert.h>
IMp7JrsWordLexiconType::IMp7JrsWordLexiconType()
{

// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtPropInit.h

}

IMp7JrsWordLexiconType::~IMp7JrsWordLexiconType()
{
// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtPropCleanup.h

}

Mp7JrsWordLexiconType::Mp7JrsWordLexiconType()
{
	//Init();  // Must be called by factory -- base init depends on m_myself being set up
}

Mp7JrsWordLexiconType::~Mp7JrsWordLexiconType()
{
	Cleanup();
}

void Mp7JrsWordLexiconType::Init()
{
	// Init base
	m_Base = CreateLexiconType;
	m_Base->m_myself = m_myself.getPointer();
	

	// Init attributes
	m_phoneticAlphabet = Mp7JrsphoneticAlphabetType::UninitializedEnumeration;
	m_phoneticAlphabet_Exist = false;

	// Init elements (element, union sequence choice all any)
	
	m_Token = Mp7JrsWordLexiconType_Token_LocalPtr(); // Class
	m_Token_Exist = false;


// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtMyPropInit.h

}

void Mp7JrsWordLexiconType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtMyPropCleanup.h


	// Mp7JrsFactory::DeleteObject(m_Base);
	// Mp7JrsFactory::DeleteObject(m_Token);
}

void Mp7JrsWordLexiconType::DeepCopy(const Mp7JrsNodePtr &original)
{
	// Don't use WordLexiconTypePtr, since we
	// might need GetBase(), which isn't defined in IWordLexiconType
    const Mp7JrsPtr< Mp7JrsWordLexiconType > tmp(original);
	if (tmp == NULL) return; // EXIT: passed argument of wrong type
	// Mp7JrsFactory::DeleteObject(m_Base);
	// FIXME: won't work with derived types
	m_Base = Mp7JrsFactory::CloneObject(tmp->GetBase());
//      m_Base = Mp7JrsLexiconPtr(Mp7JrsFactory::CloneObject((dynamic_cast< const Mp7JrsWordLexiconType *>(&original))->GetBase().Ptr())); // XXX version pre-smartptr-factory
	{
	if (tmp->ExistphoneticAlphabet())
	{
		this->SetphoneticAlphabet(tmp->GetphoneticAlphabet());
	}
	else
	{
		InvalidatephoneticAlphabet();
	}
	}
	if (tmp->IsValidToken())
	{
		// Mp7JrsFactory::DeleteObject(m_Token);
		this->SetToken(Mp7JrsFactory::CloneObject(tmp->GetToken()));
	}
	else
	{
		InvalidateToken();
	}
}

Mp7JrsNodePtr Mp7JrsWordLexiconType::GetBaseRoot()
{
	return GetBase()->GetBase()->GetBase();
}

const Mp7JrsNodePtr Mp7JrsWordLexiconType::GetBaseRoot() const
{
	return GetBase()->GetBase()->GetBase();
}


Mp7JrsPtr< Mp7JrsLexiconType > Mp7JrsWordLexiconType::GetBase() const
{
	return m_Base;
}

Mp7JrsphoneticAlphabetType::Enumeration Mp7JrsWordLexiconType::GetphoneticAlphabet() const
{
	return m_phoneticAlphabet;
}

bool Mp7JrsWordLexiconType::ExistphoneticAlphabet() const
{
	return m_phoneticAlphabet_Exist;
}
void Mp7JrsWordLexiconType::SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Enumeration item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordLexiconType::SetphoneticAlphabet().");
	}
    m_phoneticAlphabet = item;
	m_phoneticAlphabet_Exist = true;
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordLexiconType::InvalidatephoneticAlphabet()
{
	m_phoneticAlphabet_Exist = false;
}
Mp7JrsWordLexiconType_Token_LocalPtr Mp7JrsWordLexiconType::GetToken() const
{
	return m_Token;
}

// Element is optional
bool Mp7JrsWordLexiconType::IsValidToken() const
{
	return m_Token_Exist;
}

void Mp7JrsWordLexiconType::SetToken(const Mp7JrsWordLexiconType_Token_LocalPtr & item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordLexiconType::SetToken().");
	}
	if (m_Token != item || m_Token_Exist == false)
	{
		// Mp7JrsFactory::DeleteObject(m_Token);
		m_Token = item;
		// baw 27 04 2004: make sure that node on which parent is set is not null
		if (m_Token) m_Token->SetParent(m_myself.getPointer());
		if (m_Token && m_Token->GetClassId() 
    != Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:WordLexiconType_Token_LocalType"))) {
		  // Element is concrete, so set UseTypeAttribute to true only if item is derived!
		  m_Token->UseTypeAttribute = true;
		}
		m_Token_Exist = true;
		if(m_Token != Mp7JrsWordLexiconType_Token_LocalPtr()) // FTT removed extra parenthesis to make it compile under gcc 3.2.2
		{
			m_Token->SetContentName(XMLString::transcode("Token"));
		}
    }
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordLexiconType::InvalidateToken()
{
	m_Token_Exist = false;
}
unsigned Mp7JrsWordLexiconType::GetnumOfOriginalEntries() const
{
	return GetBase()->GetnumOfOriginalEntries();
}

bool Mp7JrsWordLexiconType::ExistnumOfOriginalEntries() const
{
	return GetBase()->ExistnumOfOriginalEntries();
}
void Mp7JrsWordLexiconType::SetnumOfOriginalEntries(unsigned item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordLexiconType::SetnumOfOriginalEntries().");
	}
	GetBase()->SetnumOfOriginalEntries(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordLexiconType::InvalidatenumOfOriginalEntries()
{
	GetBase()->InvalidatenumOfOriginalEntries();
}
XMLCh * Mp7JrsWordLexiconType::Getid() const
{
	return GetBase()->GetBase()->Getid();
}

bool Mp7JrsWordLexiconType::Existid() const
{
	return GetBase()->GetBase()->Existid();
}
void Mp7JrsWordLexiconType::Setid(XMLCh * item, Mp7JrsClientID client)
{
	if (!IsLocked(client)) {
		throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsWordLexiconType::Setid().");
	}
	GetBase()->GetBase()->Setid(item, client);
	Mp7JrsClientManager::SendUpdateNotification(
		m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
		client, m_myself.getPointer());
}

void Mp7JrsWordLexiconType::Invalidateid()
{
	GetBase()->GetBase()->Invalidateid();
}

Mp7JrsNodeEnum Mp7JrsWordLexiconType::GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client /* = MP7JRS_UNDEFINED_CLIENT */, Mp7JrsXPathCreation flag /* = MP7JRS_FORCECREATE */, Mp7JrsXPathParseContext * context /* = NULL */)
{
	Mp7JrsXPathParseContext c;
	bool isentrypoint = Mp7JrsXPathParseContext::Initialise(c, &context, xpath, flag);
	
	// Base class first
	Mp7JrsNodeEnum result = m_Base->GetOrCreateFromXPath(xpath, client, flag, context);
	if(context->Found) return result; // Ready
	const Mp7JrsNodePtr empty = Mp7JrsNodePtr();
	//------------------------------------------------------------------
	if(XMLString::compareString(context->ElementName, X("Token")) == 0)
	{
		// Token is simple element WordLexiconType_Token_LocalType
		context->Found = true;
		Mp7JrsXPathParseContext::PreProcessSimpleElement(context);
		
		Mp7JrsNodePtr p;
		if((p = GetToken()) != empty)
		{
			if(Mp7JrsXPathParseContext::HandleSimpleElement(context, p))
			{
				result.Insert(p);
			}
		}
		else if(context->Create)
		{
			if((p = Mp7JrsXPathParseContext::CreateNode(context, Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:WordLexiconType_Token_LocalType")))) != empty)
			{
				// Is type allowed
				Mp7JrsWordLexiconType_Token_LocalPtr test = p;
				if(test || p->GetClassId() == 1) // Deliberate "Unkown" type is allowed
				{
					SetToken(p, client);
					if((p = GetToken()) != empty) // Maybe some other restrictions are waiting...
					{
						result.Insert(p);
					}
				}
				if(result.Size() == 0)
				{
					Mp7JrsXPathParseContext::ErrorCannotSetElement(context);
				}
			}
		}
		if(result.Size() > 0 && XMLString::stringLen(context->UnparsedXPath) > 0)
		{
			Mp7JrsNodeEnum subresult;
			subresult.Insert(result.GetNext()->GetOrCreateFromXPath(context->UnparsedXPath, client, flag, NULL));
			return subresult;
		}		
		return result;
		
	}
	else if(isentrypoint)
	{
		// Sorry, unknown subelement name for WordLexiconType
		Mp7JrsXPathParseContext::ErrorUnknownSubElement(context, "WordLexiconType");
	}		
	
	return result;
}

XMLCh * Mp7JrsWordLexiconType::ToText() const
{
	// for Emtpy, ElementOnly, Mixed and TextOnly containing attributes intentionally dummy implemented
	return XMLString::transcode(this->GetTypeName());
}


bool Mp7JrsWordLexiconType::Parse(const XMLCh * const txt)
{
	return false;
}


void Mp7JrsWordLexiconType::Serialize(DOMDocument * doc, DOMElement* parent, DOMElement** newElem)
{
	// The element we serialize in.
	DOMElement* element;

	// baw 10 02 2004: added newElem to return new node to caller from derived class
	DOMElement* localNewElem = NULL;
	if (!(newElem)) newElem = &localNewElem;
	// ---

	// Check if we should add the namespace prefix
	const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));

	// baw 06 02 2004: create elements only if no base is present 
	// baw 17 02 2004: or if the base has a type that does not create a named node
	// ftt 04 03 2011: or if the base would create a named node with the wrong namespace (base namespace instead of derived one)
	if (*newElem) 
	{
		// Element was passed in -> use it
		element = *newElem;
	} else if(parent == NULL) 
	{
		// Root node (presumably)
		element = doc->getDocumentElement();
	}
	// Don't create own node: LocalType is from Choice or Sequence
	else {
		element = parent;
	}


    // pass newElem to base class, so that it contains new node after return
    m_Base->Serialize(doc, element, newElem);
    element = *newElem;

	assert(element);
// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtPreSerialize.h


	Mp7JrsUtil::HandleXsiType(element, doc, this->UseTypeAttribute, X("urn:mpeg:mpeg7:schema:2004"), X("WordLexiconType"));
    // Attribute Serialization:
		

	// Optional attriute
	if(m_phoneticAlphabet_Exist)
	{
	// Enumeration
	if(m_phoneticAlphabet != Mp7JrsphoneticAlphabetType::UninitializedEnumeration)
	{
		XMLCh * tmp = Mp7JrsphoneticAlphabetType::ToText(m_phoneticAlphabet);
		element->setAttributeNS(X(""), X("phoneticAlphabet"), tmp);
		XMLString::release(&tmp);
	}
		
	} // Optional attriute
	
	// Element serialization:
	// Class
	
	if (m_Token != Mp7JrsWordLexiconType_Token_LocalPtr())
	{
		// Class
		// Create child node to serialize into, only if current type is not unknown
		// (Unknown types are serialising the whole subtree including this element)
		DOMElement * elem = NULL;
  	if(m_Token->GetClassId() != Mp7JrsFactory::GetTypeIndex(X("Mp7JrsUnknown")))
		{
			elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), X("Token"));
			const XMLCh * prefix = Mp7JrsUtil::LookupNamespacePrefix(doc, X("urn:mpeg:mpeg7:schema:2004"));
			elem->setPrefix(prefix);
			
			element->appendChild(elem);
		}
		m_Token->Serialize(doc, element, &elem);
	}

}

bool Mp7JrsWordLexiconType::Deserialize(DOMDocument * doc, DOMElement * parent, DOMElement ** current)
{
	if(!parent) return false;

	bool found = false;

	// Deserialize attribute
	// For now, we silently assume attributeFormDefault = "unqualified"
#if 0	
	if(parent->hasAttributeNS(X(""), X("phoneticAlphabet")))
	{
        this->SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Parse(parent->getAttributeNS(X(""), X("phoneticAlphabet"))));
        * current = parent;
	}
#endif // 0	
	if(parent->hasAttribute(X("phoneticAlphabet")))
	{
        this->SetphoneticAlphabet(Mp7JrsphoneticAlphabetType::Parse(parent->getAttribute(X("phoneticAlphabet"))));
        * current = parent;
    }

	// Extensionbase is Mp7JrsLexiconType
	// Deserialize cce factory type
	if(m_Base->Deserialize(doc, parent, current))
	{
		found = true;
		parent = * current;
		parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
	}
	else
	{ // This is new for cce only
		parent = Mp7JrsUtil::GetNextChild(parent);
	}
        // Deserialize element
	if(Mp7JrsUtil::HasNodeName(parent, X("Token"), X("urn:mpeg:mpeg7:schema:2004")))
//        if((parent != NULL) && (XMLString::compareString(parent->getNodeName(), X("Token")) == 0))
        {
            // We have to deal with xsi:type classes...
            // Deserialize factory type
            Mp7JrsPtr< Mp7JrsNode > tmp;
            if(parent->hasAttribute(X("xsi:type")))
            {
              NamespaceHelper result;
              Mp7JrsUtil::GetXsiType(parent, doc, &result);
              XMLCh * xsitype = Mp7JrsUtil::QualifiedName(&result);
              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(xsitype))));
//              tmp = Mp7JrsPtr< Mp7JrsNode >(Mp7JrsFactory::CreateObject((Mp7JrsFactory::GetTypeIndex(parent->getAttribute(X("xsi:type"))))));
              if(xsitype) Mp7JrsUtil::deallocate(&xsitype);
            }else 
            {
              tmp = CreateWordLexiconType_Token_LocalType;
            }
            tmp->Deserialize(doc, parent, current);
            
            if(parent->hasAttribute(X("xsi:type"))) {
                    tmp->UseTypeAttribute = true;
            }
            this->SetToken(tmp);
            found = true;
            * current = parent; // We must adjust it!
            parent = Mp7JrsUtil::GetNextAkin(parent); // Previously GetNextSibling
        }
// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtPostDeserialize.h

	return found;
}

Mp7JrsNodePtr Mp7JrsWordLexiconType::CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype)
{
  assert(elementname);
  Mp7JrsNodePtr child = m_Base->CreateChild(elementname, xsitype);
  if (child == NULL) { // Base couldn't create child node
    child = Mp7JrsNodePtr();
  } else {
    return child;
  }
  if (XMLString::compareString(elementname, X("Token")) == 0) {
    // We check for xsi:type
    if (xsitype != NULL && XMLString::stringLen(xsitype) > 0) {
      child = Mp7JrsNodePtr(Mp7JrsFactory::CreateObject( Mp7JrsFactory::GetTypeIndex(xsitype) ) );
    } else { // no xsi:type
      child = CreateWordLexiconType_Token_LocalType;
    }
    this->SetToken(child);
  }
  return child;
 
}

Mp7JrsNodeEnum Mp7JrsWordLexiconType::GetAllChildElements() const
{
	Mp7JrsNodeEnum result;
	if (GetToken() != Mp7JrsNodePtr())
		result.Insert(GetToken());
	// Note that this returns only child elements (no attributes)
	// that are Patterns, Elements or Collections!
	return result;
}

// no includefile for extension defined 
// file Mp7JrsWordLexiconType_ExtMethodImpl.h


