
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtImplInclude.h


#include "Mp7JrsmediaTimeOffsetType.h"

Mp7JrsmediaTimeOffsetType::Mp7JrsmediaTimeOffsetType()
    : m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
        Init();
}

Mp7JrsmediaTimeOffsetType::~Mp7JrsmediaTimeOffsetType()
{
        Cleanup();
}

void Mp7JrsmediaTimeOffsetType::Init()
{       
        m_OffsetSign = 1;
        m_Days = -1;
        m_Hours = -1;
        m_Minutes = -1;
        m_Seconds = -1;
        m_NumberOfFractions = -1;
        m_Time_Exist = false;
        m_FractionsPerSecond = -1;

// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtPropInit.h

}

void Mp7JrsmediaTimeOffsetType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtPropCleanup.h


}

void Mp7JrsmediaTimeOffsetType::DeepCopy(const Mp7JrsNodePtr &original)
{
        Mp7JrsPtr< Mp7JrsmediaTimeOffsetType > tmp = original;
        if (tmp == NULL) return; // EXIT: wrong type passed
        Cleanup();
        // FIXME: this will fail for derived classes
  
        this->Parse(tmp->ToText());
        
        this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrsmediaTimeOffsetType::GetBaseRoot()
{
        return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrsmediaTimeOffsetType::GetBaseRoot() const
{
        return m_myself.getPointer();
}

int Mp7JrsmediaTimeOffsetType::GetOffsetSign() const
{
        return m_OffsetSign;
}

void Mp7JrsmediaTimeOffsetType::SetOffsetSign(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetOffsetSign().");
        }
        m_OffsetSign = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetDays() const
{
        return m_Days;
}

void Mp7JrsmediaTimeOffsetType::SetDays(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetDays().");
        }
        m_Days = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetHours() const
{
        return m_Hours;
}

void Mp7JrsmediaTimeOffsetType::SetHours(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetHours().");
        }
        m_Hours = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetMinutes() const
{
        return m_Minutes;
}

void Mp7JrsmediaTimeOffsetType::SetMinutes(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetMinutes().");
        }
        m_Minutes = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetSeconds() const
{
        return m_Seconds;
}

void Mp7JrsmediaTimeOffsetType::SetSeconds(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetSeconds().");
        }
        m_Seconds = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetNumberOfFractions() const
{
        return m_NumberOfFractions;
}

void Mp7JrsmediaTimeOffsetType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetNumberOfFractions().");
        }
        m_NumberOfFractions = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
bool Mp7JrsmediaTimeOffsetType::GetTime_Exist() const
{
        return m_Time_Exist;
}

void Mp7JrsmediaTimeOffsetType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetTime_Exist().");
        }
        m_Time_Exist = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrsmediaTimeOffsetType::GetFractionsPerSecond() const
{
        return m_FractionsPerSecond;
}

void Mp7JrsmediaTimeOffsetType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrsmediaTimeOffsetType::SetFractionsPerSecond().");
        }
        m_FractionsPerSecond = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}


XMLCh * Mp7JrsmediaTimeOffsetType::ToText() const
{
        XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
        XMLCh * tmp = NULL;
        
        if(m_OffsetSign < 0)
        {
                Mp7JrsUtil::CatString(&buf, X("-"));
        }

        Mp7JrsUtil::CatString(&buf, X("P"));

        if(m_Days >= 0)
        {
                tmp = Mp7JrsConvert::BinToText(m_Days);
                Mp7JrsUtil::CatString(&buf, tmp);
                Mp7JrsUtil::CatString(&buf, X("D"));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
        }

        if(m_Time_Exist)
        {
                Mp7JrsUtil::CatString(&buf, X("T"));

                if(m_Hours >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Hours);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("H"));
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Minutes >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Minutes);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("M"));
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Seconds >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Seconds);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("S"));
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_NumberOfFractions >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        Mp7JrsUtil::CatString(&buf, X("N"));
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }
        }

        if(m_FractionsPerSecond >= 0)
        {
                tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
                Mp7JrsUtil::CatString(&buf, tmp);
                Mp7JrsUtil::CatString(&buf, X("F"));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
        }
        return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrsmediaTimeOffsetType::Parse(const XMLCh * const txt)
{
        Match m;
        XMLCh substring [128] = {0};
        RegularExpression r("^\\-?P(\\d+D)?(T(\\d+H)?(\\d+M)?(\\d+S)?(\\d+N)?)?(\\d+F)?$");
        // This is for Xerces 3.1.1+
        if(r.matches(txt, &m))
        {
                for(int i = 0; i < m.getNoGroups(); i++)
                {
                        int i1 = m.getStartPos(i);
                        int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
                        if(i1 == i2) continue; // ignore missing optional or wrong group
                        switch(i)
                        {
                                case 0:
                                        if(i1 >= 0 && i2 - i1 > 1)
                                        {
                                                // \\-?P
                                                m_OffsetSign = -1;
                                        }
                                break;

                                case 1:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+D)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Days = Mp7JrsConvert::TextToUnsignedInt(substring);
                                        }
                                break;

                                case 2:
                                        // T
                                        // Nothing to do
                                break;

                                case 3:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+H)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 4:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+M)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 5:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+S)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 6:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+N)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 7:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+F)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2 - 1);
                                                m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
                                        }
                                break;
        
                                default: return true;
                                break;
                        }
                }
        }
        else return false;
        return true;
}

bool Mp7JrsmediaTimeOffsetType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrsmediaTimeOffsetType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrsmediaTimeOffsetType::ContentToString() const
{
	// Mp7JrsmediaTimeOffsetType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrsmediaTimeOffsetType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
        // The element we serialize into
        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
        if (!newElem) newElem = &localNewElem;

        if (*newElem) {
                element = *newElem;
        } else if(parent == NULL) {
                element = doc->getDocumentElement();
                *newElem = element;
        } else {
                if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD                      XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
                        DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
                        parent->appendChild(elem);
                        *newElem = elem;
                        element = elem;
                } else
                        assert(false);
        }
        assert(element);

        if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
        {
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:mediaTimeOffsetType")))));
        }

// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtPreImplementCleanup.h


        XMLCh * buf = this->ToText();
        if(buf != NULL)
        {
                element->appendChild(doc->createTextNode(buf));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
        }
}

bool Mp7JrsmediaTimeOffsetType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtPostDeserialize.h


        bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
        if(found)
        {
                * current = parent;
        }
        return found;   
}

// no includefile for extension defined 
// file Mp7JrsmediaTimeOffsetType_ExtMethodImpl.h








