
// Based on PatternType_CPP.template

/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



#include <stdio.h>
#include <xercesc/util/regx/RegularExpression.hpp>
#include <xercesc/util/regx/Match.hpp>
#include <assert.h>
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsConvert.h"
#include "Mp7JrsUtil.h"
#include "Mp7JrsClientManager.h"

// no includefile for extension defined 
// file Mp7JrstimePointType_ExtImplInclude.h


#include "Mp7JrstimePointType.h"

Mp7JrstimePointType::Mp7JrstimePointType()
    : m_nsURI(X("urn:mpeg:mpeg7:schema:2004"))
{
        Init();
}

Mp7JrstimePointType::~Mp7JrstimePointType()
{
        Cleanup();
}

void Mp7JrstimePointType::Init()
{       
        m_TimepointSign = 1;
        m_Year = -1;
        m_Month = -1;
        m_Day = -1;
        m_Date_Exist = false;
        m_Hours = -1;
        m_Minutes = -1;
        m_Seconds = -1;
        m_NumberOfFractions = -1;
        m_Time_Exist = false;
        m_FractionsPerSecond = -1;
        m_TimezoneSign = 1;
        m_TimezoneHours = -1;
        m_TimezoneMinutes = -1;
        m_Timezone_Exist = false;

// no includefile for extension defined 
// file Mp7JrstimePointType_ExtPropInit.h

}

void Mp7JrstimePointType::Cleanup()
{
// no includefile for extension defined 
// file Mp7JrstimePointType_ExtPropCleanup.h


}

void Mp7JrstimePointType::DeepCopy(const Mp7JrsNodePtr &original)
{
        Mp7JrsPtr< Mp7JrstimePointType > tmp = original;
        if (tmp == NULL) return; // EXIT: wrong type passed
        Cleanup();
        // FIXME: this will fail for derived classes
  
        this->Parse(tmp->ToText());
        
        this->m_ContentName = NULL;
}

Mp7JrsNodePtr Mp7JrstimePointType::GetBaseRoot()
{
        return m_myself.getPointer();
}

const Mp7JrsNodePtr Mp7JrstimePointType::GetBaseRoot() const
{
        return m_myself.getPointer();
}

int Mp7JrstimePointType::GetTimepointSign() const
{
        return m_TimepointSign;
}

void Mp7JrstimePointType::SetTimepointSign(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTimepointSign().");
        }
        m_TimepointSign = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetYear() const
{
        return m_Year;
}

void Mp7JrstimePointType::SetYear(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetYear().");
        }
        m_Year = val;
        m_Date_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetMonth() const
{
        return m_Month;
}

void Mp7JrstimePointType::SetMonth(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetMonth().");
        }
        m_Month = val;
        m_Date_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetDay() const
{
        return m_Day;
}

void Mp7JrstimePointType::SetDay(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetDay().");
        }
        m_Day = val;
        m_Date_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
bool Mp7JrstimePointType::GetDate_Exist() const
{
        return m_Date_Exist;
}

void Mp7JrstimePointType::SetDate_Exist(bool val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetDate_Exist().");
        }
        m_Date_Exist = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetHours() const
{
        return m_Hours;
}

void Mp7JrstimePointType::SetHours(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetHours().");
        }
        m_Hours = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetMinutes() const
{
        return m_Minutes;
}

void Mp7JrstimePointType::SetMinutes(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetMinutes().");
        }
        m_Minutes = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetSeconds() const
{
        return m_Seconds;
}

void Mp7JrstimePointType::SetSeconds(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetSeconds().");
        }
        m_Seconds = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetNumberOfFractions() const
{
        return m_NumberOfFractions;
}

void Mp7JrstimePointType::SetNumberOfFractions(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetNumberOfFractions().");
        }
        m_NumberOfFractions = val;
        m_Time_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
bool Mp7JrstimePointType::GetTime_Exist() const
{
        return m_Time_Exist;
}

void Mp7JrstimePointType::SetTime_Exist(bool val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTime_Exist().");
        }
        m_Time_Exist = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetFractionsPerSecond() const
{
        return m_FractionsPerSecond;
}

void Mp7JrstimePointType::SetFractionsPerSecond(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetFractionsPerSecond().");
        }
        m_FractionsPerSecond = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetTimezoneSign() const
{
        return m_TimezoneSign;
}

void Mp7JrstimePointType::SetTimezoneSign(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTimezoneSign().");
        }
        m_TimezoneSign = val;
        m_Timezone_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetTimezoneHours() const
{
        return m_TimezoneHours;
}

void Mp7JrstimePointType::SetTimezoneHours(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTimezoneHours().");
        }
        m_TimezoneHours = val;
        m_Timezone_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
int Mp7JrstimePointType::GetTimezoneMinutes() const
{
        return m_TimezoneMinutes;
}

void Mp7JrstimePointType::SetTimezoneMinutes(int val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTimezoneMinutes().");
        }
        m_TimezoneMinutes = val;
        m_Timezone_Exist = true;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}
bool Mp7JrstimePointType::GetTimezone_Exist() const
{
        return m_Timezone_Exist;
}

void Mp7JrstimePointType::SetTimezone_Exist(bool val, Mp7JrsClientID client)
{
        if (!IsLocked(client)) {
                throw Mp7JrsException(MP7JRS_ERROR, "Could not acquire lock in Mp7JrstimePointType::SetTimezone_Exist().");
        }
        m_Timezone_Exist = val;
        Mp7JrsClientManager::SendUpdateNotification(
                m_myself.getPointer(), MP7JRS_NA_NODE_UPDATED,
                client, m_myself.getPointer());
}


XMLCh * Mp7JrstimePointType::ToText() const
{
        XMLCh * buf = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode("");
        XMLCh * tmp = NULL;
        
        if(m_TimepointSign < 0)
        {
                Mp7JrsUtil::CatString(&buf, X("-"));
        }

        if(m_Date_Exist)
        {
                if(m_Year >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Year);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Month >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Month);
                        tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                        Mp7JrsUtil::CatString(&buf, X("-"));
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Day >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_Day);
                        tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                        Mp7JrsUtil::CatString(&buf, X("-"));
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }
        }

        if(m_Time_Exist)
        {
                if(m_Hours >= 0)
                {
                        Mp7JrsUtil::CatString(&buf, X("T"));
                        tmp = Mp7JrsConvert::BinToText(m_Hours);
                        tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Minutes >= 0)
                {
                        Mp7JrsUtil::CatString(&buf, X(":"));
                        tmp = Mp7JrsConvert::BinToText(m_Minutes);
                        tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_Seconds >= 0)
                {
                        Mp7JrsUtil::CatString(&buf, X(":"));
                        tmp = Mp7JrsConvert::BinToText(m_Seconds);
                        tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }

                if(m_NumberOfFractions >= 0)
                {
                        tmp = Mp7JrsConvert::BinToText(m_NumberOfFractions);
                        Mp7JrsUtil::CatString(&buf, X(":"));
                        Mp7JrsUtil::CatString(&buf, tmp);
                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                }
        }

        if(m_FractionsPerSecond >= 0)
        {
                tmp = Mp7JrsConvert::BinToText(m_FractionsPerSecond);
                Mp7JrsUtil::CatString(&buf, X("F"));
                Mp7JrsUtil::CatString(&buf, tmp);
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
        }

        if(m_Timezone_Exist)
        {
                if(m_TimezoneSign > 0)
                {
                        Mp7JrsUtil::CatString(&buf, X("+"));
                }
                else if(m_TimezoneSign < 0)
                {
                        Mp7JrsUtil::CatString(&buf, X("-"));
                }

                tmp = Mp7JrsConvert::BinToText(m_TimezoneHours);
                tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                Mp7JrsUtil::CatString(&buf, tmp);
                Mp7JrsUtil::CatString(&buf, X(":"));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
                tmp = Mp7JrsConvert::BinToText(m_TimezoneMinutes);
                tmp = Mp7JrsUtil::PadLeft(tmp, X("0"), 2);
                Mp7JrsUtil::CatString(&buf, tmp);
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&tmp);
        }
        return buf;
}


/////////////////////////////////////////////////////////////////

bool Mp7JrstimePointType::Parse(const XMLCh * const txt)
{
        Match m;
        XMLCh substring [128] = {0};
        RegularExpression r("^(\\-?\\d+(\\-\\d{2}(\\-\\d{2})?)?)?(T\\d{2}(:\\d{2}(:\\d{2}(:\\d+)?)?)?)?(F\\d+)?((\\-|\\+)\\d{2}:\\d{2})?$");
        // This is for Xerces 3.1.1+
        if(r.matches(txt, &m))
        {
                for(int i = 1; i < m.getNoGroups(); i++)
                {
                        int i1 = m.getStartPos(i);
                        int i2 = Mp7JrsUtil::GetNextEndPos(m, i);
                        if(i1 == i2) continue; // ignore missing optional or wrong group
                        switch(i)
                        {
                                case 0:
                                                // Nothing to do here...
                                break;

                                case 1:
                                        if(i1 >= 0)
                                        {
                                                // (\\d+...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i2);
                                                m_TimepointSign = Mp7JrsConvert::TextToSign(substring);
                                                if(m_TimepointSign < 0)
                                                {
                                                        XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                }
                                                m_Year = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Date_Exist = true;
                                        }
                                break;

                                case 2:
                                        if(i1 >= 0)
                                        {
                                                // (\\-\\d{2}...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_Month = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Date_Exist = true;
                                        }
                                break;

                                case 3:
                                        if(i1 >= 0)
                                        {
                                                // (\\-\\d{2})?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_Day = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Date_Exist = true;
                                        }
                                break;

                                case 4:
                                        if(i1 >= 0)
                                        {
                                                // (T\\d{2}...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_Hours = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 5:
                                        if(i1 >= 0)
                                        {
                                                // (:\\d{2}...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_Minutes = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 6:
                                        if(i1 >= 0)
                                        {
                                                // (:\\d{2}...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_Seconds = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 7:
                                        if(i1 >= 0)
                                        {
                                                // (:\\d+...)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_NumberOfFractions = Mp7JrsConvert::TextToUnsignedInt(substring);
                                                m_Time_Exist = true;
                                        }
                                break;

                                case 8:
                                        if(i1 >= 0)
                                        {
                                                // (F\\d+)?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i2);
                                                m_FractionsPerSecond = Mp7JrsConvert::TextToUnsignedInt(substring);
                                        }
                                break;

                                case 9:
                                        if(i1 >= 0)
                                        {
                                                // ((\\-|\\+)\\d{2}:\\d{2})?
                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1, i1 + 1);
                                                m_TimezoneSign = Mp7JrsConvert::TextToSign(substring);

                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 1, i1 + 3);
                                                m_TimezoneHours = Mp7JrsConvert::TextToUnsignedInt(substring);

                                                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::subString(substring, txt, i1 + 4, i2);
                                                m_TimezoneMinutes = Mp7JrsConvert::TextToUnsignedInt(substring);

                                                m_Timezone_Exist = true;
                                        }
                                break;

                                default: return true;
                                break;
                        }
                }
        }
        else return false;
        return true;
}

bool Mp7JrstimePointType::ContentFromString(const XMLCh * const txt)
{
	if(! txt) return false;
	// Mp7JrstimePointType has no attributes so forward it to Parse()
	return Parse(txt);
}
XMLCh* Mp7JrstimePointType::ContentToString() const
{
	// Mp7JrstimePointType has no attributes so forward it to ToText()
	return ToText();
}

void Mp7JrstimePointType::Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem)
{
        // The element we serialize into
        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* element;

        XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* localNewElem = NULL;
        if (!newElem) newElem = &localNewElem;

        if (*newElem) {
                element = *newElem;
        } else if(parent == NULL) {
                element = doc->getDocumentElement();
                *newElem = element;
        } else {
                if(this->GetContentName() != (XMLCh*)NULL) {
//  OLD                      XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * elem = doc->createElement(this->GetContentName());
                        DOMElement * elem = doc->createElementNS(X("urn:mpeg:mpeg7:schema:2004"), this->GetContentName());
                        parent->appendChild(elem);
                        *newElem = elem;
                        element = elem;
                } else
                        assert(false);
        }
        assert(element);

        if(this->UseTypeAttribute && ! element->hasAttribute(X("xsi:type")))
        {
			element->setAttribute(X("xsi:type"), X(Mp7JrsFactory::GetTypeName(Mp7JrsFactory::GetTypeIndex(X("urn:mpeg:mpeg7:schema:2004:timePointType")))));
        }

// no includefile for extension defined 
// file Mp7JrstimePointType_ExtPreImplementCleanup.h


        XMLCh * buf = this->ToText();
        if(buf != NULL)
        {
                element->appendChild(doc->createTextNode(buf));
                XERCES_CPP_NAMESPACE_QUALIFIER XMLString::release(&buf);
        }
}

bool Mp7JrstimePointType::Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent,  XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current)
{
// no includefile for extension defined 
// file Mp7JrstimePointType_ExtPostDeserialize.h


        bool found = this->Parse(Mp7JrsUtil::GetElementText(parent));
        if(found)
        {
                * current = parent;
        }
        return found;   
}

// begin extension included
// file Mp7JrstimePointType_ExtMethodImpl.h
#include "Mp7JrsdurationType.h"

void IMp7JrstimePointType::Add(Mp7JrsdurationPtr duration, Mp7JrsClientID client) {
	int nFractions = (int)(GetTotalNrFractions()+(duration->GetTotalNrFractions()*1.0/duration->GetFractionsPerSecond())*GetFractionsPerSecond());
	SetTotalNrFractions(nFractions,client);
}

Mp7JrsdurationPtr IMp7JrstimePointType::Subtract(Mp7JrstimePointPtr timePoint, Mp7JrsClientID client) {
	int nFractions = (int)(GetTotalNrFractions()-(timePoint->GetTotalNrFractions()*1.0/timePoint->GetFractionsPerSecond())*GetFractionsPerSecond());
	Mp7JrsdurationPtr duration = CreatedurationType;
	duration->SetFractionsPerSecond(GetFractionsPerSecond(),client);
	duration->SetTotalNrFractions(nFractions,client);
	return duration;
}

int IMp7JrstimePointType::GetTotalNrFractions() {
	int sum = 0;
	// days and months missing!!!!!
	sum = 60 * (GetHours() + sum);
	sum = 60 * (GetMinutes() + sum);
	sum = GetFractionsPerSecond() * (GetSeconds() + sum);
	sum += GetNumberOfFractions(); 
	return sum;
}

void IMp7JrstimePointType::SetTotalNrFractions(int fractions, Mp7JrsClientID client) {
	int remainder = fractions;
	SetNumberOfFractions(remainder % GetFractionsPerSecond(),client);
	remainder /= GetFractionsPerSecond();
	SetSeconds (remainder % 60,client);
	remainder /= 60;
	SetMinutes (remainder % 60,client);
	remainder /= 60;
	SetHours (remainder,client);
	// days and months ignored!!!!
}

// end extension included








